@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 /* Parameters derived from machine and compiler architecture. */
 /* This file was generated mechanically by genarch.c, for a 32bit */
@@ -25,6 +25,7 @@
 #define ARCH_ALIGN_PTR_MOD 4
 #define ARCH_ALIGN_FLOAT_MOD 4
 #define ARCH_ALIGN_DOUBLE_MOD 8
+#define ARCH_ALIGN_UINT64_T_MOD 8
 
 	 /* ---------------- Scalar sizes ---------------- */
 
