@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -313,7 +313,7 @@ open_UFST(fapi_ufst_server * r, const byte * server_param,
     code = gx_UFST_init(r->mem, (const UB8 *)ufst_root_dir);
     if (code < 0)
         return code;
-    r->ufst_is_singleton = (code == 1);
+    r->ufst_is_singleton = 0;
     CGIFfont_access(FSA DISK_ACCESS);
     if (bPlugIn) {
         if ((code = gx_UFST_open_static_fco(sPlugIn, &fcHandle)) != 0)
@@ -712,9 +712,10 @@ pack_long(LPUB8 * p, UL32 v)
 }
 
 static inline void
-pack_float(LPUB8 * p, float v)
+pack_float(LPUB8 * p, LPUB8 pe, float v)
 {
-    gs_sprintf((char *)(*p), "%f", v);
+    int plen = (int)(((char *)(*p)) - ((char *)(pe)));
+    gs_snprintf((char *)(*p), plen, "%f", v);
     *p += strlen((const char *)*p) + 1;
 }
 
@@ -770,7 +771,7 @@ pack_pseo_word_array(fapi_ufst_server * r, gs_fapi_font * ff, UB8 ** p,
 }
 
 static int
-pack_pseo_fhdr(fapi_ufst_server * r, gs_fapi_font * ff, UB8 * p)
+pack_pseo_fhdr(fapi_ufst_server * r, gs_fapi_font * ff, UB8 * p, UB8 * pe)
 {
     ushort j, n, skip = 0;
     int code;
@@ -784,7 +785,7 @@ pack_pseo_fhdr(fapi_ufst_server * r, gs_fapi_font * ff, UB8 * p)
         code = ff->get_float(ff, gs_fapi_font_feature_FontMatrix, j, &f);
         if (code < 0)
             return code;
-        pack_float(&p, f);
+        pack_float(&p, pe, f);
     }
     while (((uint64_t) p) & 0x03)       /* align to QUADWORD */
         PACK_ZERO(p);
@@ -918,6 +919,7 @@ pack_pseo_fhdr(fapi_ufst_server * r, gs_fapi_font * ff, UB8 * p)
     }
     if (skip)
         pack_word(&p, 0xFFFF);
+    return -1;
 }
 
 static char *
@@ -1165,7 +1167,7 @@ ufst_make_font_data(fapi_ufst_server * r, const char *font_file_path,
         if (ff->is_type1) {
             LPUB8 fontdata = (LPUB8) h + PCLETTOFONTHDRSIZE;
 
-            code = pack_pseo_fhdr(r, ff, fontdata);
+            code = pack_pseo_fhdr(r, ff, fontdata, (LPUB8)(buf + area_length));
             if (code < 0)
                 return code;
         }
@@ -1330,8 +1332,10 @@ gs_fapi_ufst_get_scaled_font_aux(gs_fapi_server * server, gs_fapi_font * ff,
     r->callback_error = 0;
 
     code = CGIFfont(FSA fc);
-    if (r->callback_error != 0)
+    if (r->callback_error != 0) {
+        gs_free(r->mem, ff->server_font_data, 0, 0, "gs_fapi_ufst_get_scaled_font_aux");
         return r->callback_error;
+    }
 
     if (d->font_type == FC_FCO_TYPE) {
         code = CGIFfont_metrics(&fm);
@@ -1339,6 +1343,7 @@ gs_fapi_ufst_get_scaled_font_aux(gs_fapi_server * server, gs_fapi_font * ff,
             return r->callback_error;
     }
     if (code != 0) {
+        gs_free(r->mem, ff->server_font_data, 0, 0, "gs_fapi_ufst_get_scaled_font_aux");
         code = gs_error_invalidfont;
     }
 
@@ -1805,7 +1810,7 @@ get_char(gs_fapi_server *server, gs_fapi_font *ff, gs_fapi_char_ref *c,
          */
         code = CGIFFwidth2(FSA &glyph_width_code, 1, sizeof(glyph_width), glyph_width);
     }
-    
+
     if (code >= 0) {
         code = CGIFchar_handle(FSA cc, &result, (SW16) 0);
     }
@@ -1816,7 +1821,7 @@ get_char(gs_fapi_server *server, gs_fapi_font *ff, gs_fapi_char_ref *c,
     if (code == ERR_TT_UNDEFINED_INSTRUCTION
 #if UFST_VERSION_MAJOR >= 6
         || (code >= ERR_TT_NULL_FUNCDEF && code <= ERR_TT_STACK_OUT_OF_RANGE)
-#endif   
+#endif
         ) {
         int savehint = FC_DONTHINTTT(fc);
 
@@ -1861,7 +1866,7 @@ get_char(gs_fapi_server *server, gs_fapi_font *ff, gs_fapi_char_ref *c,
         code2 = CGIFchar_handle(FSA c1, &result, (SW16) 0);
         if (code2 && code2 != ERR_fixed_space && code2 != ERR_bm_buff
             && code2 != ERR_bm_too_big) {
-            
+
             notdef_str.data = (byte *)space;
             notdef_str.size = strlen(space);
             ff->char_data = (void *)&notdef_str;
@@ -1911,7 +1916,7 @@ get_char(gs_fapi_server *server, gs_fapi_font *ff, gs_fapi_char_ref *c,
         design_bbox[1] = pbm->top_indent;
         design_bbox[2] = pbm->black_width;
         design_bbox[3] = pbm->black_depth;
-        
+
         if (ff->is_vertical) {
             /* FIXME: this probably isn't need - we can probably just use glyph_width */
             if (pIFS->glyphMetricsDU.aw.x == 0 && pIFS->glyphMetricsDU.aw.y == 0) {
@@ -1958,13 +1963,13 @@ get_char(gs_fapi_server *server, gs_fapi_font *ff, gs_fapi_char_ref *c,
         design_bbox[1] = pol->bottom;
         design_bbox[2] = pol->right;
         design_bbox[3] = pol->top;
-        
+
         r->char_data = (IFOUTLINE *) result;
     }
     else {
         design_escapement[0] = glyph_width[0];
         design_escapement[1] = glyph_width[1];
-        
+
         design_bbox[0] = design_bbox[1] = design_bbox[2] = design_bbox[3] = 0;
     }
 #if 1                           /* UFST 5.0 */
@@ -2224,7 +2229,7 @@ gs_fapi_ufst_set_mm_weight_vector(gs_fapi_server *server, gs_fapi_font *ff, floa
     (void)ff;
     (void)wvector;
     (void)length;
-    
+
     return gs_error_invalidaccess;
 }
 
@@ -2246,6 +2251,8 @@ static const gs_fapi_server ufstserver = {
     {0},
     0,
     false,
+    false,
+    {1, 0, 0, 1, 0, 0},
     1,
     {1, 0, 0, 1, 0, 0},
     gs_fapi_ufst_ensure_open,
@@ -2332,6 +2339,13 @@ gs_fapi_ufst_destroy(gs_fapi_server ** server)
     if (r->bInitialized && !r->ufst_is_singleton)
         gx_UFST_fini();
 
+    while (r->fco_list != NULL) {
+        fco_list_elem *e = r->fco_list;
+        r->fco_list = e->next;
+        gs_free(r->mem, e->file_path, 0, 0, "gs_fapi_ufst_destroy");
+        gs_free(r->mem, e, 0, 0, "gs_fapi_ufst_destroy");
+    }
+
     if (r->param) {
         gs_free(r->mem, r->param, 0, 0, "server_params");
     }
