@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Default and device-independent RasterOp algorithms */
@@ -140,10 +140,10 @@ gx_default_strip_copy_rop2(gx_device * dev,
     pmdev->height = block_height;
     pmdev->bitmap_memory = mem;
     pmdev->color_info = dev->color_info;
-    if (dev->is_planar)
+    if (dev->num_planar_planes)
     {
         gx_render_plane_t planes[GX_DEVICE_COLOR_MAX_COMPONENTS];
-        uchar num_comp = dev->color_info.num_components;
+        uchar num_comp = dev->num_planar_planes;
         uchar i;
         plane_depth = dev->color_info.depth / num_comp;
         for (i = 0; i < num_comp; i++)
@@ -405,6 +405,9 @@ pack_from_standard(gx_device_memory * dev, int y, int destx, const byte * src,
             vr = (vr >= 0x80 ? vr | chop : vr & ~chop);
             vg = (vg >= 0x80 ? vg | chop : vg & ~chop);
             vb = (vb >= 0x80 ? vb | chop : vb & ~chop);
+            /* Avoid overflow, CID 427553 */
+            if (chop & 0x80)
+                return;
             chop <<= 1;
         }
         if ((shift -= depth) >= 0)
@@ -487,6 +490,9 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
             vr = (vr >= 0x80 ? vr | chop : vr & ~chop);
             vg = (vg >= 0x80 ? vg | chop : vg & ~chop);
             vb = (vb >= 0x80 ? vb | chop : vb & ~chop);
+            /* Avoid overflow, CID 427561 */
+            if (chop & 0x80)
+                return;
             chop <<= 1;
         }
         switch (depth) {
@@ -598,7 +604,7 @@ do_strip_copy_rop(gx_device * dev,
     gx_device_memory mdev;
     union { long l; void *p; } mdev_storage[20];
     uint row_raster = bitmap_raster(width * depth);
-    ulong size_from_mem_device;
+    size_t size_from_mem_device;
     gs_logical_operation_t lop = lop_sanitize(olop);
     bool uses_d = rop3_uses_D(lop);
     bool uses_s = rop3_uses_S(lop);
@@ -650,7 +656,7 @@ do_strip_copy_rop(gx_device * dev,
 
     /* We know the device is a memory device, so we can store the
      * result directly into its scan lines, unless it is planar. */
-    if (!tdev->is_planar || tdev->color_info.num_components <= 1) {
+    if (!tdev->num_planar_planes || tdev->color_info.num_components <= 1) {
         if ((rop_depth == 24) && (dev_proc(dev, dev_spec_op)(dev,
                                       gxdso_is_std_cmyk_1bit, NULL, 0) > 0)) {
             pack = pack_cmyk_1bit_from_standard;
@@ -1849,7 +1855,7 @@ mem_transform_pixel_region_begin(gx_device *dev, int w, int h, int spp,
     if (state->posture == transform_pixel_region_portrait) {
 #ifdef WITH_CAL
         int factor;
-        if (mdev->is_planar) {
+        if (mdev->num_planar_planes > 1) {
             goto planar;
         } else if (pixels->x.step.dQ == fixed_1*8 && pixels->x.step.dR == 0 && rows->y.step.dQ == fixed_1*8 && rows->y.step.dR == 0) {
             state->render = mem_transform_pixel_region_render_portrait_1to8;
@@ -1886,7 +1892,7 @@ mem_transform_pixel_region_begin(gx_device *dev, int w, int h, int spp,
         } else
 no_cal:
 #endif
-        if (mdev->is_planar)
+        if (mdev->num_planar_planes > 1)
 #ifdef WITH_CAL
 planar:
 #endif
@@ -1895,7 +1901,7 @@ planar:
             state->render = mem_transform_pixel_region_render_portrait_1to1;
         else
             state->render = mem_transform_pixel_region_render_portrait;
-    } else if (mdev->is_planar)
+    } else if (mdev->num_planar_planes > 1)
         state->render = mem_transform_pixel_region_render_landscape_planar;
     else
         state->render = mem_transform_pixel_region_render_landscape;
