@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
-/* $Id$ */
+
 /* 32-bit-per-pixel "memory" (stored bitmap) device */
 #include "memory_.h"
 #include "gx.h"
@@ -28,11 +30,20 @@ declare_mem_procs(mem_true32_copy_mono, mem_true32_copy_color, mem_true32_fill_r
 
 /* The device descriptor. */
 const gx_device_memory mem_true32_device =
-mem_full_device("image32", 24, 8, mem_open,
-                gx_default_map_rgb_color, gx_default_map_color_rgb,
-     mem_true32_copy_mono, mem_true32_copy_color, mem_true32_fill_rectangle,
-            gx_default_cmyk_map_cmyk_color, gx_default_strip_tile_rectangle,
-                mem_default_strip_copy_rop, mem_get_bits_rectangle);
+    mem_device("image32", 24, 8, mem_dev_initialize_device_procs);
+
+const gdev_mem_functions gdev_mem_fns_32 =
+{
+    gx_default_map_rgb_color,
+    gx_default_map_color_rgb,
+    mem_true32_fill_rectangle,
+    mem_true32_copy_mono,
+    mem_true32_copy_color,
+    gx_default_copy_alpha,
+    gx_default_strip_tile_rectangle,
+    mem_default_strip_copy_rop2,
+    mem_get_bits_rectangle
+};
 
 /* Convert x coordinate to byte offset in scan line. */
 #undef x_to_byte
@@ -42,7 +53,7 @@ mem_full_device("image32", 24, 8, mem_open,
 #define color_swap_bytes(color)\
   ((((color) >> 24) & 0xff) + (((color) >> 8) & 0xff00) +\
    (((color) & 0xff00) << 8) + ((color) << 24))
-#if arch_is_big_endian
+#if ARCH_IS_BIG_ENDIAN
 #  define arrange_bytes(color) (color)
 #else
 #  define arrange_bytes(color) color_swap_bytes(color)
@@ -153,6 +164,10 @@ mem_true32_copy_mono(gx_device * dev,
             int sbyte = (*sptr++ << first_bit) & 0xff;
             int count = w_first;
 
+#ifdef PACIFY_VALGRIND
+	    sbyte &= 0x100-(0x100>>w_first);
+#endif
+
             if (sbyte)
                 do {
                     if (sbyte & 0x80)
@@ -235,18 +250,27 @@ mem_true32_copy_color(gx_device * dev,
 /* Note that on a big-endian machine, this is the same as the */
 /* standard byte-oriented-device. */
 
-#if !arch_is_big_endian
+#if !ARCH_IS_BIG_ENDIAN
 
 /* Procedures */
 declare_mem_procs(mem32_word_copy_mono, mem32_word_copy_color, mem32_word_fill_rectangle);
 
 /* Here is the device descriptor. */
 const gx_device_memory mem_true32_word_device =
-mem_full_device("image32w", 24, 8, mem_open,
-                gx_default_map_rgb_color, gx_default_map_color_rgb,
-     mem32_word_copy_mono, mem32_word_copy_color, mem32_word_fill_rectangle,
-            gx_default_cmyk_map_cmyk_color, gx_default_strip_tile_rectangle,
-                gx_no_strip_copy_rop, mem_word_get_bits_rectangle);
+    mem_device("image32w", 24, 8, mem_word_dev_initialize_device_procs);
+
+const gdev_mem_functions gdev_mem_fns_32w =
+{
+    gx_default_map_rgb_color,
+    gx_default_map_color_rgb,
+    mem32_word_fill_rectangle,
+    mem32_word_copy_mono,
+    mem32_word_copy_color,
+    gx_default_copy_alpha,
+    gx_default_strip_tile_rectangle,
+    gx_no_strip_copy_rop2,
+    mem_word_get_bits_rectangle
+};
 
 /* Fill a rectangle with a color. */
 static int
@@ -276,7 +300,7 @@ mem32_word_copy_color(gx_device * dev,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     byte *row;
-    uint raster;
+    size_t raster;
 
     fit_copy(dev, base, sourcex, sraster, id, x, y, w, h);
     row = scan_line_base(mdev, y);
@@ -287,4 +311,4 @@ mem32_word_copy_color(gx_device * dev,
     return 0;
 }
 
-#endif /* !arch_is_big_endian */
+#endif /* !ARCH_IS_BIG_ENDIAN */
