@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2023 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -147,9 +147,14 @@ gdev_mem_set_planar_interleaved(gx_device_memory * mdev, int num_planes,
            four planes then use the high level color filling procedure.  Also
            make use of the put_image operation to go from the pdf14 device
            directly to the planar buffer. */
-        /* Note this is only valid for contone (8 or more bits per component) */
-        if (num_planes >= 4 && mdev->color_info.depth/num_planes >= 8) {
-            set_dev_proc(mdev, put_image, mem_planar_put_image);
+        if (num_planes >= 4) {
+            /* put_image always gives us our data in bytes (or 16bits) per
+             * component. When we have a suitably deep device we can assume
+             * that this will match our own representation, and use a fast
+             * copy_planes based implementation. If not, we have to use the
+             * default's copy-via-image implementation. */
+            if (mdev->color_info.depth / num_planes >= 8)
+               set_dev_proc(mdev, put_image, mem_planar_put_image);
         }
         set_dev_proc(mdev, fill_rectangle, mem_planar_fill_rectangle);
         set_dev_proc(mdev, copy_alpha_hl_color, gx_default_copy_alpha_hl_color);
@@ -248,7 +253,7 @@ put_image_copy_planes(gx_device * dev, const byte **base_ptr, int sourcex,
     mem_save_params_t save;
     const gdev_mem_functions *fns;
     int code = 0;
-    uchar plane;
+    int plane;
     const byte *base;
     int last_plane = mdev->num_planar_planes - 1;
 
@@ -306,11 +311,13 @@ mem_planar_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    uchar pi;
+    int pi;
     int x = fixed2int(rect->p.x);
     int y = fixed2int(rect->p.y);
     int w = fixed2int(rect->q.x) - x;
     int h = fixed2int(rect->q.y) - y;
+    int has_tags = device_encodes_tags(dev);
+    int npp = mdev->num_planar_planes - has_tags;
 
     /* We can only handle devn cases, so use the default if not */
     /* We can get called here from gx_dc_devn_masked_fill_rectangle */
@@ -318,7 +325,7 @@ mem_planar_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
         return gx_fill_rectangle_device_rop( x, y, w, h, pdcolor, dev, lop_default);
     }
     MEM_SAVE_PARAMS(mdev, save);
-    for (pi = 0; pi < mdev->num_planar_planes; ++pi) {
+    for (pi = 0; pi < npp; ++pi) {
         int plane_depth = mdev->planes[pi].depth;
         gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
         int shift = 16 - plane_depth;
@@ -330,6 +337,17 @@ mem_planar_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
                             (pdcolor->colors.devn.values[pi]) >> shift & mask);
         mdev->line_ptrs += mdev->height;
     }
+    if (has_tags) {
+        int plane_depth = mdev->planes[pi].depth;
+        gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
+        const gdev_mem_functions *fns =
+                    gdev_mem_functions_for_bits(plane_depth);
+
+        MEM_SET_PARAMS(mdev, plane_depth);
+        fns->fill_rectangle(dev, x, y, w, h,
+                    (pdcolor->colors.devn.values[pi]) & mask);
+        mdev->line_ptrs += mdev->height;
+    }
     MEM_RESTORE_PARAMS(mdev, save);
     return 0;
 }
@@ -341,7 +359,7 @@ mem_planar_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    uchar pi;
+    int pi;
 
     MEM_SAVE_PARAMS(mdev, save);
     for (pi = 0; pi < mdev->num_planar_planes; ++pi) {
@@ -367,7 +385,7 @@ mem_planar_copy_mono(gx_device * dev, const byte * base, int sourcex,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    uchar pi;
+    int pi;
 
     MEM_SAVE_PARAMS(mdev, save);
     for (pi = 0; pi < mdev->num_planar_planes; ++pi) {
@@ -867,7 +885,7 @@ mem_planar_copy_color(gx_device * dev, const byte * base, int sourcex,
     } buf;
     int source_depth = dev->color_info.depth;
     mem_save_params_t save;
-    uchar pi;
+    int pi;
 
     /* This routine cannot copy from 3bit chunky data, as 3 bit
      * things don't pack nicely into bytes or words. Accordingly
@@ -980,7 +998,7 @@ mem_planar_copy_planes(gx_device * dev, const byte * base, int sourcex,
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
     int code = 0;
-    uchar plane;
+    int plane;
 
     MEM_SAVE_PARAMS(mdev, save);
     for (plane = 0; plane < mdev->num_planar_planes; plane++)
@@ -995,7 +1013,7 @@ mem_planar_copy_planes(gx_device * dev, const byte * base, int sourcex,
         else
             code = fns->copy_color(dev, base, sourcex, sraster,
                                    id, x, y, w, h);
-        base += sraster * plane_height;
+        base += sraster * (size_t)plane_height;
         mdev->line_ptrs += mdev->height;
     }
     MEM_RESTORE_PARAMS(mdev, save);
@@ -1010,7 +1028,7 @@ mem_planar_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tiles,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    uchar pi;
+    int pi;
 
     MEM_SAVE_PARAMS(mdev, save);
     for (pi = 0; pi < mdev->num_planar_planes; ++pi) {
@@ -1062,7 +1080,7 @@ mem_planar_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    uchar pi;
+    int pi;
 
     /* We can't split up the transfer if the tile is colored. */
     if (color0 == gx_no_color_index && color1 == gx_no_color_index)
@@ -1112,7 +1130,7 @@ planar_cmyk4bit_strip_copy_rop2(gx_device_memory * mdev,
                                 uint planar_height)
 {
     gs_rop3_t rop = (gs_rop3_t)lop;
-    uint draster = mdev->raster;
+    intptr_t draster = mdev->raster;
     int line_count;
     byte *cdrow, *mdrow, *ydrow, *kdrow;
     byte lmask, rmask;
@@ -1200,7 +1218,7 @@ planar_cmyk4bit_strip_copy_rop2(gx_device_memory * mdev,
          * textures is a pixmap (or constant, in which case we'll do it
          * below). */
         int ty;
-        uint traster;
+        intptr_t traster;
 
 /* Calculate the X offset for a given Y value, */
 /* taking shift into account if necessary. */
@@ -1737,7 +1755,7 @@ plane_strip_copy_rop2(gx_device_memory * mdev,
     /* assert(planar_height == 0); */
 
     MEM_SAVE_PARAMS(mdev, save);
-    mdev->line_ptrs += mdev->height * plane;
+    mdev->line_ptrs += mdev->height * (intptr_t)plane;
     fns = gdev_mem_functions_for_bits(mdev->planes[plane].depth);
     /* strip_copy_rop2 might end up calling get_bits_rectangle or fill_rectangle,
      * so ensure we have the right ones in there. */
@@ -1792,7 +1810,7 @@ plane_strip_copy_rop2(gx_device_memory * mdev,
  */
 static int
 planar_to_chunky(gx_device_memory *mdev, int x, int y, int w, int h,
-                 int offset, uint draster, byte *dest, byte **line_ptrs,
+                 int offset, intptr_t draster, byte *dest, byte **line_ptrs,
                  int plane_height)
 {
     int num_planes = mdev->num_planar_planes;
@@ -1934,32 +1952,33 @@ mem_planar_strip_copy_rop2(gx_device * dev,
          * convert whole lines of s, but only as many lines as we have to. */
         /* We assume that scolors == NULL here */
         int i;
-        uchar j;
-        uint chunky_sraster;
-        uint nbytes;
+        int j;
+        intptr_t chunky_sraster;
+        intptr_t nbytes;
         byte **line_ptrs;
         byte *sbuf, *buf;
 
-        chunky_sraster = sraster * mdev->num_planar_planes;
-        nbytes = height * chunky_sraster;
+        chunky_sraster = sraster * (intptr_t)mdev->num_planar_planes;
+        if (check_64bit_multiply(height, chunky_sraster, (size_t *)&nbytes) != 0)
+            return gs_note_error(gs_error_undefinedresult);
         buf = gs_alloc_bytes(mdev->memory, nbytes, "mem_planar_strip_copy_rop(buf)");
         if (buf == NULL) {
             return gs_note_error(gs_error_VMerror);
         }
-        nbytes = sizeof(byte *) * mdev->num_planar_planes * height;
+        nbytes = sizeof(byte *) * (intptr_t)mdev->num_planar_planes * height;
         line_ptrs = (byte **)gs_alloc_bytes(mdev->memory, nbytes, "mem_planar_strip_copy_rop(line_ptrs)");
         if (line_ptrs == NULL) {
             gs_free_object(mdev->memory, buf, "mem_planar_strip_copy_rop(buf)");
             return gs_note_error(gs_error_VMerror);
         }
         for (j = 0; j < mdev->color_info.num_components; j++) {
-            sbuf = (byte *)sdata + j * sraster * planar_height;
+            sbuf = (byte *)sdata + j * (intptr_t)sraster * planar_height;
             for (i = height; i > 0; i--) {
                 *line_ptrs++ = sbuf;
                 sbuf += sraster;
             }
         }
-        line_ptrs -= height * mdev->num_planar_planes;
+        line_ptrs -= height * (intptr_t)mdev->num_planar_planes;
         planar_to_chunky(mdev, sourcex, 0, width, height,
                          0, chunky_sraster, buf, line_ptrs, height);
         gs_free_object(mdev->memory, line_ptrs, "mem_planar_strip_copy_rop(line_ptrs)");
@@ -1980,10 +1999,11 @@ mem_planar_strip_copy_rop2(gx_device * dev,
          * convert whole lines of t, but only as many lines as we have to
          * (unless it loops). */
         /* We assume that tcolors == NULL here */
-        int ty, i;
-        uint chunky_t_raster;
-        uint chunky_t_height;
-        uint nbytes;
+        int ty;
+        intptr_t i;
+        intptr_t chunky_t_raster;
+        int chunky_t_height;
+        intptr_t nbytes;
         byte **line_ptrs;
         byte *tbuf, *buf;
         gx_strip_bitmap newtex;
@@ -1991,7 +2011,7 @@ mem_planar_strip_copy_rop2(gx_device * dev,
         ty = (y + phase_y) % textures->rep_height;
         if (ty < 0)
             ty += textures->rep_height;
-        chunky_t_raster = bitmap_raster(textures->rep_width * mdev->color_info.depth);
+        chunky_t_raster = bitmap_raster(textures->rep_width * (intptr_t)mdev->color_info.depth);
         if (ty + height <= textures->rep_height) {
             chunky_t_height = height;
             phase_y = -y;
@@ -1999,23 +2019,24 @@ mem_planar_strip_copy_rop2(gx_device * dev,
             ty = 0;
             chunky_t_height = textures->rep_height;
         }
-        nbytes = chunky_t_height * chunky_t_raster;
+        if (check_64bit_multiply(chunky_t_height, chunky_t_raster, &nbytes) != 0)
+            return gs_note_error(gs_error_undefinedresult);
         buf = gs_alloc_bytes(mdev->memory, nbytes, "mem_planar_strip_copy_rop(buf)");
         if (buf == NULL) {
             return gs_note_error(gs_error_VMerror);
         }
-        nbytes = sizeof(byte *) * mdev->num_planar_planes * textures->rep_height;
+        nbytes = sizeof(byte *) * (intptr_t)mdev->num_planar_planes * textures->rep_height;
         line_ptrs = (byte **)gs_alloc_bytes(mdev->memory, nbytes, "mem_planar_strip_copy_rop(line_ptrs)");
         if (line_ptrs == NULL) {
             gs_free_object(mdev->memory, buf, "mem_planar_strip_copy_rop(buf)");
             return gs_note_error(gs_error_VMerror);
         }
         tbuf = textures->data;
-        for (i = textures->rep_height * mdev->num_planar_planes; i > 0; i--) {
+        for (i = textures->rep_height * (intptr_t)mdev->num_planar_planes; i > 0; i--) {
             *line_ptrs++ = tbuf;
             tbuf += textures->raster;
         }
-        line_ptrs -= textures->rep_height * mdev->num_planar_planes;
+        line_ptrs -= textures->rep_height * (intptr_t)mdev->num_planar_planes;
         planar_to_chunky(mdev, 0, ty, textures->rep_width, chunky_t_height,
                          0, chunky_t_raster, buf, line_ptrs, textures->rep_height);
         gs_free_object(mdev->memory, line_ptrs, "mem_planar_strip_copy_rop(line_ptrs)");
@@ -2041,7 +2062,7 @@ mem_planar_strip_copy_rop2(gx_device * dev,
         /* No T in use, or constant T. */
         if ((!lop_uses_S(lop) || (scolors && (scolors[0] == scolors[1]))) &&
             ((mdev->color_info.num_components == 1) || (mdev->color_info.num_components == 3))) {
-            uchar plane;
+            int plane;
             /* No S in use, or constant S. And either greyscale or rgb,
              * so we can just do the rop on each plane in turn. */
             for (plane=0; plane < mdev->color_info.num_components; plane++)
@@ -2106,7 +2127,7 @@ mem_planar_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     gs_get_bits_options_t options = params->options;
     int x = prect->p.x, w = prect->q.x - x, y = prect->p.y, h = prect->q.y - y;
-    uchar num_planes = mdev->num_planar_planes;
+    int num_planes = mdev->num_planar_planes;
     gs_get_bits_params_t copy_params;
     int code;
 
@@ -2172,13 +2193,13 @@ mem_planar_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
      */
     if (!(~options & (GB_PACKING_PLANAR | GB_SELECT_PLANES))) {
         /* Check that only a single plane is being requested. */
-        uchar pi;
+        int pi;
 
         for (pi = 0; pi < num_planes; ++pi)
             if (params->data[pi] != 0)
                 break;
         if (pi < num_planes) {
-            uchar plane = pi++;
+            int plane = pi++;
 
             for (; pi < num_planes; ++pi)
                 if (params->data[pi] != 0)
@@ -2266,9 +2287,9 @@ mem_planar_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
     if (!(~options & (GB_COLORS_NATIVE | GB_ALPHA_NONE |
                       GB_PACKING_CHUNKY | GB_RETURN_COPY))) {
         int offset = (options & GB_OFFSET_SPECIFIED ? params->x_offset : 0);
-        uint draster =
+        intptr_t draster =
             (options & GB_RASTER_SPECIFIED ? params->raster :
-             bitmap_raster((offset + w) * mdev->color_info.depth));
+             bitmap_raster((offset + w) * (intptr_t)mdev->color_info.depth));
 
         planar_to_chunky(mdev, x, y, w, h, offset, draster, params->data[0],
                          mdev->line_ptrs, mdev->height);
@@ -2287,7 +2308,8 @@ mem_planar_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
             ulong l[BUF_LONGS];
             byte b[BUF_BYTES];
         } buf;
-        int br, bw, bh, cx, cy, cw, ch;
+        intptr_t br;
+        int bw, bh, cx, cy, cw, ch;
         int ddepth = mdev->color_info.depth;
         uint raster = bitmap_raster(ddepth * mdev->width);
         gs_get_bits_params_t dest_params;
