@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -24,6 +24,7 @@
 #include "gsmemory.h"		/* for gp.h */
 #include "gserrors.h"
 #include "gp.h"
+#include "stream.h"
 
 /* ------ Miscellaneous ------ */
 
@@ -70,24 +71,16 @@ gp_get_realtime(long *pdt)
 void
 gp_get_usertime(long *pdt)
 {
+    FILETIME CreationTime, ExitTime, KernelTime, UserTime;
 
-    LARGE_INTEGER freq;
-    LARGE_INTEGER count;
-    LARGE_INTEGER seconds;
-
-    if (!QueryPerformanceFrequency(&freq)) {
+    if (!GetProcessTimes(GetCurrentProcess(), &CreationTime, &ExitTime, &KernelTime, &UserTime))
         gp_get_realtime(pdt);	/* use previous method if high-res perf counter not available */
-        return;
+    else {
+        ULONGLONG t = (ULONGLONG)UserTime.dwLowDateTime + (ULONGLONG)KernelTime.dwLowDateTime
+                  + (((ULONGLONG)UserTime.dwHighDateTime + (ULONGLONG)KernelTime.dwHighDateTime) << 32);
+        pdt[0] = (long)(t / 10000000);
+        pdt[1] = (long)(t % 10000000)*100;
     }
-    /* Get the high resolution time as seconds and nanoseconds */
-    QueryPerformanceCounter(&count);
-
-    seconds.QuadPart = (count.QuadPart / freq.QuadPart);
-    pdt[0] = seconds.LowPart;
-    count.QuadPart -= freq.QuadPart * seconds.QuadPart;
-    count.QuadPart *= 1000000000;			/* we want nanoseconds */
-    count.QuadPart /= freq.QuadPart;
-    pdt[1] = count.LowPart;
 }
 
 /* ------ Console management ------ */
@@ -137,7 +130,7 @@ const char gp_current_directory_name[] = ".";
 
 #if defined(__WIN32__) && !defined(METRO)
 int
-gp_local_arg_encoding_get_codepoint(FILE *file, const char **astr)
+gp_local_arg_encoding_get_codepoint(stream *s, const char **astr)
 {
     int len;
     int c;
@@ -145,8 +138,8 @@ gp_local_arg_encoding_get_codepoint(FILE *file, const char **astr)
     wchar_t unicode[2];
     char utf8[4];
 
-    if (file) {
-        c = fgetc(file);
+    if (s) {
+        c = spgetc(s);
         if (c == EOF)
             return EOF;
     } else if (**astr) {
@@ -159,8 +152,8 @@ gp_local_arg_encoding_get_codepoint(FILE *file, const char **astr)
 
     arg[0] = c;
     if (IsDBCSLeadByte(c)) {
-        if (file) {
-            c = fgetc(file);
+        if (s) {
+            c = spgetc(s);
             if (c == EOF)
                 return EOF;
         } else if (**astr) {
