@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* prevent gp.h from defining fopen */
@@ -32,7 +32,7 @@ extern intptr_t _get_osfhandle(int fd);
 FILE *
 gp_fopen_impl(gs_memory_t *mem, const char *fname, const char *mode)
 {
-    int len = utf8_to_wchar(NULL, fname);
+    int len = gp_utf8_to_uint16(NULL, fname);
     wchar_t *uni;
     wchar_t wmode[4];
     FILE *file;
@@ -40,12 +40,16 @@ gp_fopen_impl(gs_memory_t *mem, const char *fname, const char *mode)
     if (len <= 0)
         return NULL;
 
-    uni = (wchar_t *)gs_alloc_bytes(mem, len*sizeof(wchar_t), "gp_fopen_impl");
+    uni = (wchar_t *)gs_alloc_bytes(mem, (size_t)len*sizeof(wchar_t), "gp_fopen_impl");
     if (uni == NULL)
         return NULL;
-    utf8_to_wchar(uni, fname);
-    utf8_to_wchar(wmode, mode);
-    file = _wfopen(uni, wmode);
+    gp_utf8_to_uint16(uni, fname);
+
+    len = gp_utf8_to_uint16(NULL, mode);
+    if (len >= 0) {
+        gp_utf8_to_uint16(wmode, mode);
+        file = _wfopen(uni, wmode);
+    }
     gs_free_object(mem, uni, "gs_fopen_impl");
 
     return file;
@@ -54,17 +58,17 @@ gp_fopen_impl(gs_memory_t *mem, const char *fname, const char *mode)
 int
 gp_unlink_impl(gs_memory_t *mem, const char *fname)
 {
-    int len = utf8_to_wchar(NULL, fname);
+    int len = gp_utf8_to_uint16(NULL, fname);
     wchar_t *uni;
     int ret;
 
     if (len <= 0)
         return gs_error_unknownerror;
 
-    uni = (wchar_t *)gs_alloc_bytes(mem, len*sizeof(wchar_t), "gp_unlink_impl");
+    uni = (wchar_t *)gs_alloc_bytes(mem, (size_t)len*sizeof(wchar_t), "gp_unlink_impl");
     if (uni == NULL)
         return gs_error_VMerror;
-    utf8_to_wchar(uni, fname);
+    gp_utf8_to_uint16(uni, fname);
     ret = _wunlink(uni);
     gs_free_object(mem, uni, "gs_unlink_impl");
 
@@ -74,24 +78,24 @@ gp_unlink_impl(gs_memory_t *mem, const char *fname)
 int
 gp_rename_impl(gs_memory_t *mem, const char *from, const char *to)
 {
-    int lenf = utf8_to_wchar(NULL, from);
-    int lent = utf8_to_wchar(NULL, to);
+    int lenf = gp_utf8_to_uint16(NULL, from);
+    int lent = gp_utf8_to_uint16(NULL, to);
     wchar_t *unif, *unit;
     int ret;
 
     if (lenf <= 0 || lent <= 0)
         return gs_error_unknownerror;
 
-    unif = (wchar_t *)gs_alloc_bytes(mem, lenf*sizeof(wchar_t), "gp_rename_impl");
+    unif = (wchar_t *)gs_alloc_bytes(mem, (size_t)lenf*sizeof(wchar_t), "gp_rename_impl");
     if (unif == NULL)
         return gs_error_VMerror;
-    unit = (wchar_t *)gs_alloc_bytes(mem, lent*sizeof(wchar_t), "gp_rename_impl");
+    unit = (wchar_t *)gs_alloc_bytes(mem, (size_t)lent*sizeof(wchar_t), "gp_rename_impl");
     if (unit == NULL) {
         gs_free_object(mem, unif, "gs_unlink_impl");
         return gs_error_VMerror;
     }
-    utf8_to_wchar(unif, from);
-    utf8_to_wchar(unit, to);
+    gp_utf8_to_uint16(unif, from);
+    gp_utf8_to_uint16(unit, to);
     ret = _wrename(unif, unit);
     gs_free_object(mem, unif, "gs_rename_impl");
     gs_free_object(mem, unit, "gs_rename_impl");
