@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -248,7 +248,7 @@ gx_ciedefg_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *m
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
     if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
-        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
+        return gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_fromdefg(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     abc_caches, lmn_caches, defg_caches);
@@ -531,7 +531,7 @@ gx_ciedef_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *me
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
     if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
-        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
+        return gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_fromdef(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     abc_caches, lmn_caches, def_caches);
@@ -643,7 +643,7 @@ gx_cieabc_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, bool *islab,
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
     if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
-        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
+        return gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_fromabc(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     abc_caches, lmn_caches, islab);
@@ -757,7 +757,7 @@ gx_ciea_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *memo
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
     if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
-        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
+        return gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_froma(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     a_cache, lmn_caches);
@@ -853,9 +853,12 @@ gs_colorspace_set_icc_equivalent(gs_color_space *pcs, bool *islab,
     int code = 0;
 
     *islab = false;  /* For non CIEABC cases */
-    if (pcs->icc_equivalent != NULL || !gs_color_space_is_PSCIE(pcs))
+    if (pcs->icc_equivalent != NULL)
         return 0;
 
+    if (!gs_color_space_is_PSCIE(pcs))
+        return_error(gs_error_unknownerror);
+
     switch( color_space_index ) {
        case gs_color_space_index_CIEDEFG:
             code = gx_ciedefg_to_icc(&picc_cs, pcs, memory->stable_memory);
@@ -871,6 +874,7 @@ gs_colorspace_set_icc_equivalent(gs_color_space *pcs, bool *islab,
             break;
         default:
              /* do nothing.  Sould never happen */
+             code = gs_note_error(gs_error_unknownerror);
              break;
     }
     return code;
