@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -97,9 +97,10 @@ gx_concretize_DevicePixel(const gs_client_color * pc, const gs_color_space * pcs
 }
 
 static int
-gx_remap_concrete_DevicePixel(const frac * pconc, const gs_color_space * pcs,
-        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                              gs_color_select_t select)
+gx_remap_concrete_DevicePixel(const gs_color_space * pcs, const frac * pconc,
+                              gx_device_color * pdc, const gs_gstate * pgs,
+                              gx_device * dev, gs_color_select_t select,
+                              const cmm_dev_profile_t *dev_profile)
 {
     color_set_pure(pdc, pconc[0] & ((1 << dev->color_info.depth) - 1));
     return 0;
@@ -109,10 +110,11 @@ gx_remap_concrete_DevicePixel(const frac * pconc, const gs_color_space * pcs,
 static int
 gx_set_overprint_DevicePixel(const gs_color_space * pcs, gs_gstate * pgs)
 {
-    gs_overprint_params_t   params;
+    gs_overprint_params_t params = { 0 };
 
     params.retain_any_comps = false;
-    pgs->effective_overprint_mode = 0;
+    params.effective_opm = pgs->color[0].effective_opm = 0;
+    params.is_fill_color = pgs->is_fill_color;
     return gs_gstate_update_overprint(pgs, &params);
 }
 
