@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -251,6 +251,7 @@ typedef struct gs_device_n_params_s {
     gs_color_space       *devn_process_space;
     uint num_process_names;
     char **process_names;
+    bool all_none;
 } gs_device_n_params;
 
 /* Define an abstract type for the client color space data */
@@ -284,6 +285,25 @@ typedef struct gs_pattern_params_s {
     bool has_base_space; /* {csrc} can't we just NULL-check the base_space? */
 } gs_pattern_params;
 
+typedef struct gs_calgray_params_s {
+    float WhitePoint[3];
+    float BlackPoint[3];
+    float Gamma;
+} gs_calgray_params;
+
+typedef struct gs_calrgb_params_s {
+    float WhitePoint[3];
+    float BlackPoint[3];
+    float Gamma[3];
+    float Matrix[9];
+} gs_calrgb_params;
+
+typedef struct gs_lab_params_s {
+    float WhitePoint[3];
+    float BlackPoint[3];
+    float Range[4];
+} gs_lab_params;
+
 /* id's 1 through 4 are reserved for static colorspaces; thus, dynamically
    assigned id's must begin at 5. */
 #define cs_DeviceGray_id 1
@@ -292,6 +312,16 @@ typedef struct gs_pattern_params_s {
 
 typedef void (*gs_cspace_free_proc_t) (gs_memory_t * mem, void *pcs);
 
+typedef enum {
+    gs_ICC_Alternate_None,
+    gs_ICC_Alternate_DeviceGray,
+    gs_ICC_Alternate_DeviceRGB,
+    gs_ICC_Alternate_DeviceCMYK,
+    gs_ICC_Alternate_CalGray,
+    gs_ICC_Alternate_CalRGB,
+    gs_ICC_Alternate_Lab,
+} gs_ICC_Alternate_space;
+
 /*
  * The colorspace object. For pattern and indexed colorspaces, the
  * base_space refers to the underlying colorspace. For separation,
@@ -305,6 +335,7 @@ struct gs_color_space_s {
     gs_id                      id;
     gs_color_space             *base_space;
     gs_color_space             *icc_equivalent;
+    gs_ICC_Alternate_space     ICC_Alternate_space;
     client_color_space_data_t  *pclient_color_space_data;
     void                       *interpreter_data;
     gs_cspace_free_proc_t      interpreter_free_cspace_proc;
@@ -319,7 +350,13 @@ struct gs_color_space_s {
         gs_device_n_params       device_n;
         gs_indexed_params        indexed;
         gs_pattern_params        pattern;
-
+        /* These are only used for the Alternate space of an ICCBased
+         * space, for the benefit of pdfwrite. For rendering, these
+         * spaces are converted into ICCBased spaces.
+         */
+        gs_calgray_params        calgray;
+        gs_calrgb_params         calrgb;
+        gs_lab_params            lab;
     } params;
 };
 
@@ -341,7 +378,7 @@ struct gs_color_space_s {
 gs_color_space *gs_cspace_new_DeviceGray(gs_memory_t *mem);
 gs_color_space *gs_cspace_new_DeviceRGB(gs_memory_t *mem);
 gs_color_space *gs_cspace_new_DeviceCMYK(gs_memory_t *mem);
-gs_color_space *gs_cspace_new_ICC(gs_memory_t *pmem, gs_gstate * pgs, 
+gs_color_space *gs_cspace_new_ICC(gs_memory_t *pmem, gs_gstate * pgs,
                                   int components);
 gs_color_space *gs_cspace_new_scrgb(gs_memory_t *pmem, gs_gstate * pgs);
 
@@ -397,6 +434,8 @@ void rc_decrement_only_cs(gs_color_space *pcs, const char *cname);
 
 void cs_adjust_counts_icc(gs_gstate *pgs, int delta);
 
+void cs_adjust_swappedcounts_icc(gs_gstate *pgs, int delta);
+
 /* backwards compatibility */
 #define gs_color_space_indexed_base_space(pcspace)\
     gs_cspace_base_space(pcspace)
