@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -26,20 +26,11 @@
 #include "gxhttile.h"
 #include "gxcindex.h"
 
-#ifndef gx_device_color_DEFINED
-#  define gx_device_color_DEFINED
 typedef struct gx_device_color_s gx_device_color;
-#endif
 
-#ifndef gx_device_saved_color_DEFINED
-#  define gx_device_saved_color_DEFINED
 typedef struct gx_device_color_saved_s  gx_device_color_saved;
-#endif
 
-#ifndef gx_device_halftone_DEFINED
-#  define gx_device_halftone_DEFINED
 typedef struct gx_device_halftone_s gx_device_halftone;
-#endif
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * The definitions in the following section of the file are the only
@@ -181,15 +172,7 @@ void gx_complete_halftone(gx_device_color *pdevc, int num_comps,
 
 /* Define opaque types for objects referenced by device colors. */
 
-#ifndef gx_ht_tile_DEFINED
-#  define gx_ht_tile_DEFINED
-typedef struct gx_ht_tile_s gx_ht_tile;
-#endif
-
-#ifndef gx_color_tile_DEFINED
-#  define gx_color_tile_DEFINED
 typedef struct gx_color_tile_s gx_color_tile;
-#endif
 
 /*
  * The device color in the graphics state is computed from client color
@@ -288,7 +271,6 @@ struct gx_device_color_s {
             ushort num_components;
             byte c_base[GX_DEVICE_COLOR_MAX_COMPONENTS];
             uint c_level[GX_DEVICE_COLOR_MAX_COMPONENTS];
-            ushort /*gx_color_value */ alpha;
 #if GX_DEVICE_COLOR_MAX_COMPONENTS <= ARCH_SIZEOF_SHORT * 8
             ushort plane_mask;
 #else
@@ -377,7 +359,7 @@ struct gx_device_color_s {
 
 struct gx_device_color_saved_s {
     gx_device_color_type    type;
-    gs_graphics_type_tag_t tag;	/* value used to set dev_color */
+    int                     devn_type;		/* for fill / stroke */
     union _svc {
         gx_color_index  pure;
         struct _svbin {
@@ -388,14 +370,12 @@ struct gx_device_color_saved_s {
         struct _svcol {
             byte    c_base[GX_DEVICE_COLOR_MAX_COMPONENTS];
             uint    c_level[GX_DEVICE_COLOR_MAX_COMPONENTS];
-            ushort  alpha;
         }               colored;
         struct _svdevn {
             ushort values[GX_DEVICE_COLOR_MAX_COMPONENTS];
         } devn;
         struct _pattern {
             gs_id id;
-            gs_int_point phase;
         }		pattern;
         struct _pattern2 {
             gs_id id;
