@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -95,6 +95,7 @@ gs_ht_build(
     /* initialize the halftone */
     pht->type = ht_type_multiple;
     pht->rc.free = free_comps;
+    pht->objtype = HT_OBJTYPE_DEFAULT;
     pht->params.ht_multiple.components = phtc;
     pht->params.ht_multiple.num_comp = num_comps;
 
