@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2023 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -24,6 +24,6 @@
 
 gsicc_profile_cache_t* gsicc_profilecache_new(gs_memory_t *memory);
 gs_color_space* gsicc_find_cs(uint64_t key_test, gs_gstate * pgs);
-void gsicc_add_cs(gs_gstate * pgs, gs_color_space * pcs, uint64_t dictkey);
+int gsicc_add_cs(gs_gstate * pgs, gs_color_space * pcs, uint64_t dictkey);
 
 #endif
