@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -55,7 +55,6 @@
 /* Test programs */
 static int test1(gs_gstate *, gs_memory_t *);	/* kaleidoscope */
 static int test2(gs_gstate *, gs_memory_t *);	/* pattern fill */
-static int test3(gs_gstate *, gs_memory_t *);	/* RasterOp */
 static int test4(gs_gstate *, gs_memory_t *);	/* set resolution */
 static int test5(gs_gstate *, gs_memory_t *);	/* images */
 static int test6(gs_gstate *, gs_memory_t *);	/* CIE API, snapping */
@@ -69,7 +68,7 @@ static int test10(gs_gstate *, gs_memory_t *);	/* captured data */
 #endif
 static int (*tests[]) (gs_gstate *, gs_memory_t *) =
 {
-    test1, test2, test3, test4, test5,
+    test1, test2, test4, test5,
         test6, test7, test8, 0
 #ifdef CAPTURE
         test10
@@ -121,9 +120,17 @@ main(int argc, const char *argv[])
     bbdev =
         gs_alloc_struct_immovable(mem, gx_device_bbox, &st_device_bbox,
                                   "bbox");
+    if (bbdev == NULL) {
+        lprintf1("\n", code);
+        gs_abort(mem);
+    }
     gx_device_bbox_init(bbdev, dev, mem);
 
     code = dev_proc(dev, get_profile)(dev, &bbdev->icc_struct);
+    if (code < 0) {
+        lprintf1("Failed to initialise device\n", code);
+        gs_abort(mem);
+    }
     rc_increment(bbdev->icc_struct);
 
     /* Print out the device name just to test the gsparam.c API. */
@@ -173,6 +180,10 @@ main(int argc, const char *argv[])
     }
     dev = (gx_device *) bbdev;
     pgs = gs_gstate_alloc(mem);
+    if (pgs == NULL) {
+        lprintf1("allocation of gstate failed\n", code);
+        gs_abort(mem);
+    }
     gs_setdevice_no_erase(pgs, dev);	/* can't erase yet */
     {
         gs_point dpi;
@@ -194,8 +205,11 @@ main(int argc, const char *argv[])
     gs_output_page(pgs, 1, 1);
     {
         gs_rect bbox;
+        int code1;
 
-        gx_device_bbox_bbox(bbdev, &bbox);
+        code1 = gx_device_bbox_bbox(bbdev, &bbox);
+        if (code1 < 0)
+            code = code1;
         dmprintf4(mem, "Bounding box: [%g %g %g %g]\n",
                  bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y);
     }
@@ -405,45 +419,6 @@ test2(gs_gstate * pgs, gs_memory_t * mem)
 /* Exercise RasterOp a little. */
 /* Currently, this only works with monobit devices. */
 
-static int
-test3(gs_gstate * pgs, gs_memory_t * mem)
-{
-    gx_device *dev = gs_currentdevice(pgs);
-    gx_color_index black = gx_device_black(dev);
-    gx_color_index white = gx_device_white(dev);
-    gx_color_index black2[2];
-    gx_color_index black_white[2];
-    gx_color_index white_black[2];
-    long pattern[max(align_bitmap_mod / sizeof(long), 1) * 4];
-
-#define pbytes ((byte *)pattern)
-    gx_tile_bitmap tile;
-
-    black2[0] = black2[1] = black;
-    black_white[0] = white_black[1] = black;
-    black_white[1] = white_black[0] = white;
-    pbytes[0] = 0xf0;
-    pbytes[align_bitmap_mod] = 0x90;
-    pbytes[align_bitmap_mod * 2] = 0x90;
-    pbytes[align_bitmap_mod * 3] = 0xf0;
-    tile.data = pbytes;
-    tile.raster = align_bitmap_mod;
-    tile.size.x = tile.size.y = 4;
-    tile.id = gs_next_ids(mem, 1);
-    tile.rep_width = tile.rep_height = 4;
-    (*dev_proc(dev, copy_rop))
-        (dev, NULL, 0, 0, gx_no_bitmap_id, black2,
-         &tile, white_black, 100, 100, 150, 150, 0, 0, rop3_T);
-    (*dev_proc(dev, copy_rop))
-        (dev, NULL, 0, 0, gx_no_bitmap_id, black2,
-         NULL, NULL, 120, 120, 110, 110, 0, 0, ~rop3_S & rop3_1);
-    (*dev_proc(dev, copy_rop))
-        (dev, NULL, 0, 0, gx_no_bitmap_id, black2,
-         &tile, white_black, 110, 110, 130, 130, 0, 0, rop3_T ^ rop3_D);
-#undef pbytes
-    return 0;
-}
-
 /* ---------------- Test program 4 ---------------- */
 /* Set the resolution dynamically. */
 
@@ -573,10 +548,11 @@ test5(gs_gstate * pgs, gs_memory_t * mem)
         image1.Width = W;
         image1.Height = H;
         image1.BitsPerComponent = 8;
+        image1.format = gs_image_format_chunky;
 
         gs_translate(pgs, 0.5, 4.0);
-        code = gx_device_begin_image(dev, pgs,
-                                     &image1, gs_image_format_chunky,
+        code = gx_device_begin_typed_image(dev, pgs, NULL
+                                     (gs_image_common_t *)&image1,
                                      NULL, &dcolor, NULL, mem, &info1);
 /****** TEST code >= 0 ******/
         planes[0].data = data3;
@@ -927,7 +903,7 @@ test10(gs_gstate * pgs, gs_memory_t * mem)
     gs_param_int_array HWSa;
     int HWSize[2];
     float HWResolution[2], PageSize[2];
-    long MaxBitmap;
+    size_t MaxBitmap;
     int code;
     gx_device *dev = gs_currentdevice(pgs);
     float xlate_x, xlate_y;
@@ -969,13 +945,13 @@ test10(gs_gstate * pgs, gs_memory_t * mem)
     }
     emprintf3(mem, "PageSize[%d] = [ %f, %f ]\n", PSa.size,
               PSa.data[0], PSa.data[1]);
-    code = param_read_long((gs_param_list *) & list,
-                           "MaxBitmap", &MaxBitmap);
+    code = param_read_size_t((gs_param_list *) & list,
+                             "MaxBitmap", &MaxBitmap);
     if (code < 0) {
         lprintf1("reading MaxBitmap failed! code = %d\n", code);
         gs_abort(mem);
     }
-    emprintf1(mem, "MaxBitmap = %ld\n", MaxBitmap);
+    emprintf1(mem, "MaxBitmap = %"PRIi64"\n", MaxBitmap);
     /* Switch to param list functions to "write" */
     gs_c_param_list_write(&list, mem);
     /* Always set the PageSize. */
@@ -1012,8 +988,8 @@ test10(gs_gstate * pgs, gs_memory_t * mem)
         code = param_write_int_array((gs_param_list *) & list,
                                      "HWSize", &HWSa);
         MaxBitmap = 1000000L;
-        code = param_write_long((gs_param_list *) & list,
-                                "MaxBitmap", &MaxBitmap);
+        code = param_write_size_t((gs_param_list *) & list,
+                                  "MaxBitmap", &MaxBitmap);
     }
     gs_c_param_list_read(&list);
     code = gs_putdeviceparams(dev, (gs_param_list *) & list);
