@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -232,21 +232,28 @@ alpha_buffer_init(gs_gstate * pgs, fixed extra_x, fixed extra_y, int alpha_bits,
     if (height > height2)
         height = height2;
     height <<= log2_scale.y;
+    /* We may have to update the marking parameters if we have a pdf14 device
+       as our target.  Need to do while dev is still active in pgs. Do this
+       before allocating the device to simplify cleanup. */
+    if (dev_proc(dev, dev_spec_op)(dev, gxdso_is_pdf14_device, NULL, 0) > 0) {
+        int code = gs_update_trans_marking_params(pgs);
+        if (code < 0)
+            return code;
+    }
     mem = pgs->memory;
-    mdev = gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
+    mdev = gs_alloc_struct_immovable(mem, gx_device_memory, &st_device_memory,
                            "alpha_buffer_init");
     if (mdev == 0)
         return 0;		/* if no room, don't buffer */
-    /* We may have to update the marking parameters if we have a pdf14 device
-       as our target.  Need to do while dev is still active in pgs */
-    if (dev_proc(dev, dev_spec_op)(dev, gxdso_is_pdf14_device, NULL, 0) > 0) {
-        gs_update_trans_marking_params(pgs);
-    }
     gs_make_mem_abuf_device(mdev, mem, dev, &log2_scale,
                             alpha_bits, ibox.p.x << log2_scale.x, devn);
     mdev->width = width;
     mdev->height = height;
     mdev->bitmap_memory = mem;
+    /* Set the horrible hacky flag that tells people that the width/height here
+     * have been set for *our* convenience, rather than accurately depicting the
+     * size of the device for callers. */
+    mdev->non_strict_bounds = 1;
     if ((*dev_proc(mdev, open_device)) ((gx_device *) mdev) < 0) {
         /* No room for bits, punt. */
         gs_free_object(mem, mdev, "alpha_buffer_init");
@@ -643,7 +650,7 @@ static int do_fill_stroke(gs_gstate *pgs, int rule, int *restart)
     bool black_vector = false;
     bool in_smask =
         (dev_proc(pgs->device, dev_spec_op)(pgs->device, gxdso_in_smask_construction, NULL, 0)) > 0;
-
+    gs_logical_operation_t orig_lop = pgs->log_op;
 
     /* It is either our first time, or the stroke was a pattern and
        we are coming back from the error if restart < 1 (0 is first
@@ -770,7 +777,6 @@ static int do_fill_stroke(gs_gstate *pgs, int rule, int *restart)
         fixed extra_adjust;
         float xxyy = fabs(pgs->ctm.xx) + fabs(pgs->ctm.yy);
         float xyyx = fabs(pgs->ctm.xy) + fabs(pgs->ctm.yx);
-        gs_logical_operation_t orig_lop = pgs->log_op;
         pgs->log_op |= lop_pdf14; /* Force stroking to happen all in 1 go */
         scale = (float)(1 << (abits / 2));
         orig_width = gs_currentlinewidth(pgs);
@@ -793,7 +799,6 @@ static int do_fill_stroke(gs_gstate *pgs, int rule, int *restart)
         gs_setlinewidth(pgs, new_width);
         scale_dash_pattern(pgs, scale);
         gs_setflat(pgs, (double)(orig_flatness * scale));
-        pgs->log_op = orig_lop;
     } else
         acode = 0;
     code = gx_fill_stroke_path(pgs, rule);
@@ -803,6 +808,7 @@ static int do_fill_stroke(gs_gstate *pgs, int rule, int *restart)
         scale_dash_pattern(pgs, 1.0 / scale);
         gs_setflat(pgs, orig_flatness);
         acode = alpha_buffer_release(pgs, code >= 0);
+        pgs->log_op = orig_lop;
     }
     if (pgs->is_fill_color) {
         /* The color _should_ be the fill color, so make sure it is unlocked	*/
