@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -107,7 +107,12 @@ process_dict(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name key, char *
 
     dict.size = 0;
     code = add_tokens(dict.list, NULL, p, &dict.size);
-    (*p) += 2;
+    if (code >= 0) {
+        if ((*p)[0] != '>' || (*p)[1] != '>')
+            code = gs_error_syntaxerror;
+        else
+           (*p) += 2;
+    }
     code2 = param_end_write_dict((gs_param_list *)plist, key, &dict);
     return code < 0 ? code : code2;
 }
@@ -289,7 +294,11 @@ process_array(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name key, char
                     if (new_max == 0)
                         new_max = 32;
                     if (array_data == NULL) {
-                        array_data = (char *)gs_alloc_bytes(mem, sizeof(gs_param_string) * new_max, "param string array in param parsing");
+                        array_data = (char *)gs_alloc_bytes(mem, sizeof(gs_param_string) * (size_t)new_max, "param string array in param parsing");
+                        if (array_data == NULL) {
+                            code = gs_error_VMerror;
+                            break;
+                        }
                     } else {
                         char *new_array = (char *)gs_resize_object(mem, array_data, sizeof(gs_param_string) * new_max, "param string array in param parsing");
                         if (new_array == NULL) {
@@ -348,7 +357,11 @@ process_array(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name key, char
                     if (new_max == 0)
                         new_max = 32;
                     if (array_data == NULL) {
-                        array_data = (char *)gs_alloc_bytes(mem, sizeof(gs_param_string) * new_max, "param string array in param parsing");
+                        array_data = (char *)gs_alloc_bytes(mem, sizeof(gs_param_string) * (size_t)new_max, "param string array in param parsing");
+                        if (array_data == NULL) {
+                            code = gs_error_VMerror;
+                            break;
+                        }
                     } else {
                         char *new_array = (char *)gs_resize_object(mem, array_data, sizeof(gs_param_string) * new_max, "param string array in param parsing");
                         if (new_array == NULL) {
@@ -380,7 +393,11 @@ process_array(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name key, char
                     if (new_max == 0)
                         new_max = 32;
                     if (array_data == NULL) {
-                        array_data = (char *)gs_alloc_bytes(mem, sizeof(gs_param_string) * new_max, "param string array in param parsing");
+                        array_data = (char *)gs_alloc_bytes(mem, sizeof(gs_param_string) * (size_t)new_max, "param string array in param parsing");
+                        if (array_data == NULL) {
+                            code = gs_error_VMerror;
+                            break;
+                        }
                     } else {
                         char *new_array = (char *)gs_resize_object(mem, array_data, sizeof(gs_param_string) * new_max, "param string array in param parsing");
                         if (new_array == NULL) {
@@ -418,7 +435,7 @@ process_array(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name key, char
             case '.':
             case '+':
             case '-':
-                if (array_type == gs_param_type_string_array) {
+                if (array_type == gs_param_type_string_array || array_type == gs_param_type_name_array ) {
                     code = gs_error_typecheck;
                     break;
                 } else {
@@ -434,7 +451,7 @@ process_array(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name key, char
 
                     if (array_type == gs_param_type_int_array && !integer) {
                         ints = (int *)array_data;
-                        floats = (float *)gs_alloc_bytes(mem, sizeof(float) * array_max, "param string array in param parsing");
+                        floats = (float *)gs_alloc_bytes(mem, sizeof(float) * (size_t)array_max, "param string array in param parsing");
                         if (floats == NULL){
                             code = gs_error_VMerror;
                             break;
@@ -454,9 +471,13 @@ process_array(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name key, char
                             array_type = integer ? gs_param_type_int_array : gs_param_type_float_array;
                         }
                         if (array_data == NULL) {
-                            array_data = (char *)gs_alloc_bytes(mem, sizeof(size_me) * new_max, "param string array in param parsing");
+                            array_data = (char *)gs_alloc_bytes(mem, sizeof(size_me) * (size_t)new_max, "param string array in param parsing");
+                            if (array_data == NULL) {
+                                code = gs_error_VMerror;
+                                break;
+                            }
                         } else {
-                            char *new_array = (char *)gs_resize_object(mem, array_data, sizeof(size_me) * new_max, "param string array in param parsing");
+                            char *new_array = (char *)gs_resize_object(mem, array_data, sizeof(size_me) * (size_t)new_max, "param string array in param parsing");
                             if (new_array == NULL) {
                                 code = gs_error_VMerror;
                                 break;
@@ -682,6 +703,7 @@ add_tokens(gs_param_list *plist, gs_param_name key, char **pp, uint *dict_count)
             return code;
     }
 
+    *pp = p;
     return 0;
 }
 
@@ -834,20 +856,20 @@ int_array_to_string(gs_param_int_array ia, outstate *out)
 
     out_string(out, "[");
     for (i = 0; i < ia.size; i++) {
-        gs_sprintf(text, "%d", ia.data[i]);
+        gs_snprintf(text, sizeof(text), "%d", ia.data[i]);
         out_string(out, text);
     }
     out_string(out, "]");
 }
 
 static void
-print_float(char *text, float f)
+print_float(char text[32], float f)
 {
     /* We attempt to tidy up %f's somewhat unpredictable output
      * here, so rather than printing 0.10000000 we print 0.1 */
     char *p = text;
     int frac = 0;
-    gs_sprintf(text, "%f", f);
+    gs_snprintf(text, 32, "%f", f);
     /* Find the terminator, or 'e' to spot exponent mode. */
     while (*p && *p != 'e' && *p != 'E') {
         if (*p == '.')
@@ -970,28 +992,28 @@ to_string(gs_param_list *plist, gs_param_name key, outstate *out)
     case gs_param_type_int:
     {
         char text[32];
-        gs_sprintf(text, "%d", pvalue.value.i);
+        gs_snprintf(text, sizeof(text), "%d", pvalue.value.i);
         out_string(out, text);
         break;
     }
     case gs_param_type_i64:
     {
         char text[32];
-        gs_sprintf(text, "%"PRId64, pvalue.value.i64);
+        gs_snprintf(text, sizeof(text), "%"PRId64, pvalue.value.i64);
         out_string(out, text);
         break;
     }
     case gs_param_type_long:
     {
         char text[32];
-        gs_sprintf(text, "%ld", pvalue.value.l);
+        gs_snprintf(text, sizeof(text), "%ld", pvalue.value.l);
         out_string(out, text);
         break;
     }
     case gs_param_type_size_t:
     {
         char text[32];
-        gs_sprintf(text, "%"PRIdSIZE, pvalue.value.z);
+        gs_snprintf(text, sizeof(text), "%"PRIdSIZE, pvalue.value.z);
         out_string(out, text);
         break;
     }
@@ -1046,3 +1068,31 @@ int gs_param_list_to_string(gs_param_list *plist, gs_param_name key, char *value
         *value = 0;
     return to_string(plist, key, &out);
 }
+
+int gs_param_list_dump(gs_param_list *plist)
+{
+    gs_param_enumerator_t enumerator;
+    gs_param_key_t key;
+    int code;
+    char buffer[4096];
+    int len;
+
+    param_init_enumerator(&enumerator);
+    while ((code = param_get_next_key(plist, &enumerator, &key)) == 0) {
+        char string_key[256];	/* big enough for any reasonable key */
+
+        if (key.size > sizeof(string_key) - 1) {
+            code = gs_note_error(gs_error_rangecheck);
+            break;
+        }
+        memcpy(string_key, key.data, key.size);
+        string_key[key.size] = 0;
+        dlprintf1("%s ", string_key);
+        code = gs_param_list_to_string(plist, string_key, buffer, &len);
+        if (code < 0)
+            break;
+        dlprintf1("%s ", buffer);
+    }
+    dlprintf("\n");
+    return code;
+}
