@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -22,7 +22,6 @@
 #include "gsutil.h"             /* for gs_next_ids */
 #include "gzstate.h"
 #include "gxcspace.h"           /* here for gscolor2.h */
-#include "gsalpha.h"
 #include "gscolor2.h"
 #include "gscoord.h"            /* for gs_initmatrix */
 #include "gscie.h"
@@ -37,14 +36,18 @@
 #include "gzcpath.h"
 #include "gsovrc.h"
 #include "gxcolor2.h"
+#include "gscolor3.h" /* for gs_smoothness() */
 #include "gxpcolor.h"
 #include "gsicc_manage.h"
+#include "gxdevsop.h"
 
 /* Forward references */
 static gs_gstate *gstate_alloc(gs_memory_t *, client_name_t,
                                const gs_gstate *);
-static gs_gstate *gstate_clone(gs_gstate *, gs_memory_t *, client_name_t,
-                               gs_gstate_copy_reason_t);
+static gs_gstate *gstate_clone_for_gsave(gs_gstate *,
+                                         client_name_t);
+static gs_gstate *gstate_clone_for_gstate(const gs_gstate *, gs_memory_t *,
+                                          client_name_t);
 static void gstate_free_contents(gs_gstate *);
 static int gstate_copy(gs_gstate *, const gs_gstate *,
                         gs_gstate_copy_reason_t, client_name_t);
@@ -65,7 +68,7 @@ static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t c
  *      is independent of the gstate.  These are garbage collected, not
  *      reference counted, so we don't need to do anything special with them
  *      when manipulating gstates.  Currently this includes:
- *              font, device
+ *              font
  *
  * (3) Objects that are referenced directly by the gstate, may be shared
  *      among gstates, and should disappear when no gstates reference them.
@@ -73,7 +76,7 @@ static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t c
  *
  *   (3a) Objects that are logically connected to individual gstates.
  *      We use reference counting to manage these.  Currently these are:
- *              halftone, dev_ht, cie_render, black_generation,
+ *              halftone, dev_ht(4), cie_render, black_generation,
  *              undercolor_removal, set_transfer.*, cie_joint_caches,
  *              clip_stack, {opacity,shape}.mask
  *      effective_transfer.* may point to some of the same objects as
@@ -110,7 +113,7 @@ static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t c
  *      The individual routines that manipulate these are responsible
  *      for doing the right kind of reference counting or whatever.
  *      Currently:
- *              path, clip_path, and (if different from both clip_path
+ *              devices, path, clip_path, and (if different from both clip_path
  *                and view_clip) effective_clip.path require
  *                gx_path_assign/free, which uses a reference count;
  *              color_space and ccolor require cs_adjust_color/cspace_count
@@ -160,7 +163,7 @@ extern_st(st_gs_gstate); /* for gstate_alloc() */
 /* Copy client data, using the copy_for procedure if available, */
 /* the copy procedure otherwise. */
 static int
-gstate_copy_client_data(gs_gstate * pgs, void *dto, void *dfrom,
+gstate_copy_client_data(const gs_gstate * pgs, void *dto, void *dfrom,
                         gs_gstate_copy_reason_t reason)
 {
     return (pgs->client_procs.copy_for != 0 ?
@@ -232,7 +235,9 @@ gs_gstate_alloc(gs_memory_t * mem)
     code = gs_nulldevice(pgs);
     if (code < 0)
         goto fail;
-    gs_setalpha(pgs, 1.0);
+    gs_setfillconstantalpha(pgs, 1.0);
+    gs_setstrokeconstantalpha(pgs, 1.0);
+    gs_setalphaisshape(pgs, false);
     gs_settransfer(pgs, gs_identity_transfer);
     gs_setflat(pgs, 1.0);
     gs_setfilladjust(pgs, 0.3, 0.3);
@@ -272,7 +277,7 @@ gs_gstate_client_data(const gs_gstate * pgs)
 }
 
 /* Free the chain of gstates.*/
-int
+void
 gs_gstate_free_chain(gs_gstate * pgs)
 {
    gs_gstate *saved = pgs, *tmp;
@@ -282,26 +287,25 @@ gs_gstate_free_chain(gs_gstate * pgs)
        gs_gstate_free(saved);
        saved = tmp;
    }
-   return 0;
 }
 
 /* Free a graphics state. */
-int
+void
 gs_gstate_free(gs_gstate * pgs)
 {
+    if (pgs == NULL)
+        return;
     gstate_free_contents(pgs);
     gs_free_object(pgs->memory, pgs, "gs_gstate_free");
-    return 0;
 }
 
 /* Save the graphics state. */
 int
 gs_gsave(gs_gstate * pgs)
 {
-    gs_gstate *pnew = gstate_clone(pgs, pgs->memory, "gs_gsave",
-                                  copy_for_gsave);
+    gs_gstate *pnew = gstate_clone_for_gsave(pgs, "gs_gsave");
 
-    if (pnew == 0)
+    if (pnew == NULL)
         return_error(gs_error_VMerror);
     /* As of PLRM3, the interaction between gsave and the clip stack is
      * now clear. gsave stores the clip stack into the saved graphics
@@ -311,13 +315,14 @@ gs_gsave(gs_gstate * pgs)
      * on pgs->clip_stack, but gstate_clone() has an exception for
      * the clip_stack field.
      */
-    pgs->clip_stack = 0;
+    pgs->clip_stack = NULL;
     pgs->saved = pnew;
     if (pgs->show_gstate == pgs)
         pgs->show_gstate = pnew->show_gstate = pnew;
+    pgs->trans_flags.xstate_change = false;
     pgs->level++;
-    if_debug2m('g', pgs->memory, "[g]gsave -> 0x%lx, level = %d\n",
-              (ulong) pnew, pgs->level);
+    if_debug2m('g', pgs->memory, "[g]gsave -> "PRI_INTPTR", level = %d\n",
+              (intptr_t)pnew, pgs->level);
     return 0;
 }
 
@@ -351,6 +356,14 @@ gs_gsave_for_save(gs_gstate * pgs, gs_gstate ** psaved)
     /* Cut the stack so we can't grestore past here. */
     *psaved = pgs->saved;
     pgs->saved = 0;
+
+    code = gs_gsave(pgs);
+    if (code < 0) {
+        pgs->saved = *psaved;
+        *psaved = NULL;
+        gs_grestore(pgs);
+        return code;
+    }
     return code;
 fail:
     if (new_cpath)
@@ -366,10 +379,9 @@ gs_grestore_only(gs_gstate * pgs)
     gs_gstate tmp_gstate;
     void *pdata = pgs->client_data;
     void *sdata;
-    bool prior_overprint = pgs->overprint;
 
-    if_debug2m('g', pgs->memory, "[g]grestore 0x%lx, level was %d\n",
-               (ulong) saved, pgs->level);
+    if_debug2m('g', pgs->memory, "[g]grestore "PRI_INTPTR", level was %d\n",
+               (intptr_t)saved, pgs->level);
     if (!saved)
         return 1;
     sdata = saved->client_data;
@@ -388,11 +400,6 @@ gs_grestore_only(gs_gstate * pgs)
     *saved = tmp_gstate;            /* restore "freed" state (pointers zeroed after contents freed) */
     gs_free_object(pgs->memory, saved, "gs_grestore");
 
-    /* update the overprint compositor, if necessary */
-    if (prior_overprint || pgs->overprint)
-    {
-        return gs_do_set_overprint(pgs);
-    }
     return 0;
 }
 
@@ -456,27 +463,23 @@ gs_grestoreall(gs_gstate * pgs)
 
 /* Allocate and return a new graphics state. */
 gs_gstate *
-gs_gstate_copy(gs_gstate * pgs, gs_memory_t * mem)
+gs_gstate_copy(const gs_gstate * pgs, gs_memory_t * mem)
 {
     gs_gstate *pnew;
-    /* Prevent 'capturing' the view clip path. */
-    gx_clip_path *view_clip = pgs->view_clip;
 
-    pgs->view_clip = 0;
-    pnew = gstate_clone(pgs, mem, "gs_gstate", copy_for_gstate);
-    if (pnew == 0)
-        return 0;
+    pnew = gstate_clone_for_gstate(pgs, mem, "gs_gstate");
+    if (pnew == NULL)
+        return NULL;
     clip_stack_rc_adjust(pnew->clip_stack, 1, "gs_gstate_copy");
-    pgs->view_clip = view_clip;
-    pnew->saved = 0;
+    pnew->saved = NULL;
     /*
      * Prevent dangling references from the show_gstate pointer.  If
      * this context is its own show_gstate, set the pointer in the clone
      * to point to the clone; otherwise, set the pointer in the clone to
-     * 0, and let gs_setgstate fix it up.
+     * NULL, and let gs_setgstate fix it up.
      */
     pnew->show_gstate =
-        (pgs->show_gstate == pgs ? pnew : 0);
+        (pgs->show_gstate == pgs ? pnew : NULL);
     return pnew;
 }
 
@@ -521,12 +524,7 @@ gs_setgstate(gs_gstate * pgs, const gs_gstate * pfrom)
     pgs->view_clip = view_clip;
     pgs->show_gstate =
         (pgs->show_gstate == pfrom ? pgs : saved_show);
-
-    /* update the overprint compositor, unconditionally. Unlike grestore, this */
-    /* may skip over states where overprint was set, so the prior state can    */
-    /* not be relied on to avoid this call. setgstate is not as commonly used  */
-    /* as grestore, so the overhead of the compositor call is acceptable.      */
-    return(gs_do_set_overprint(pgs));
+    return 0;
 }
 
 /* Get the allocator pointer of a graphics state. */
@@ -573,7 +571,7 @@ gs_gstate_swap_memory(gs_gstate * pgs, gs_memory_t * mem)
 /*
  * Push an overprint compositor onto the current device. Note that if
  * the current device already is an overprint compositor, the
- * create_compositor will update its parameters but not create a new
+ * composite will update its parameters but not create a new
  * compositor device.
  */
 int
@@ -586,15 +584,18 @@ gs_gstate_update_overprint(gs_gstate * pgs, const gs_overprint_params_t * pparam
 
     code = gs_create_overprint(&pct, pparams, pgs->memory);
     if (code >= 0) {
-        code = dev_proc(dev, create_compositor)( dev,
+        code = dev_proc(dev, composite)( dev,
                                                    &ovptdev,
                                                    pct,
                                                    pgs,
                                                    pgs->memory,
                                                    NULL);
         if (code >= 0 || code == gs_error_handled){
-            if (ovptdev != dev)
+            if (code == 1) {
                 gx_set_device_only(pgs, ovptdev);
+                /* Get rid of extra reference */
+                rc_decrement(ovptdev, "gs_gstate_update_overprint(ovptdev)");
+            }
             code = 0;
         }
     }
@@ -632,28 +633,64 @@ gs_do_set_overprint(gs_gstate * pgs)
 
     if (cs_num_components(pcs) < 0 && pcc->pattern != 0)
         code = pcc->pattern->type->procs.set_color(pcc, pgs);
-    else
-    {
+    else {
+        gx_device* dev = pgs->device;
+        cmm_dev_profile_t* dev_profile;
+        gs_color_space_index pcs_index = gs_color_space_get_index(pcs);
+
+        dev_proc(dev, get_profile)(dev, &dev_profile);
+        if (dev_profile->overprint_control == gs_overprint_control_disable)
+            return code;
+
+        /* Transparency device that supports spots and where we have
+           sep or devicen colors needs special consideration if the device
+           is in a additive blend mode.  This could
+           be written more compactly, but it would be unreadable. */
+        if (dev_proc(dev, dev_spec_op)(dev, gxdso_pdf14_sep_device, NULL, 0) &&
+            (dev->color_info.polarity != GX_CINFO_POLARITY_SUBTRACTIVE)) {
+            if (pcs_index == gs_color_space_index_Separation) {
+                if (!(pcs->params.separation.color_type == SEP_MIX ||
+                      pcs->params.separation.color_type == SEP_ENUM)) {
+                    /* Sep color is not a spot color.  We can't do OP and trans */
+                    return code;
+                }
+            } else if (pcs_index == gs_color_space_index_DeviceN) {
+                if (pcs->params.device_n.color_type != SEP_PURE_SPOT) {
+                    /* DeviceN has process colors  We can't do OP and trans. */
+                    return code;
+                }
+            }
+        }
+
+        /* If we have a CIE-based space, use the ICC equivalent space */
+        if (gs_color_space_is_PSCIE(pcs) && pcs->icc_equivalent != NULL)
+            pcs = pcs->icc_equivalent;
+
         /* The spaces that do not allow opm (e.g. ones that are not ICC or DeviceCMYK)
            will blow away any true setting later. But we have to be prepared
-           in case this is an CMYK ICC space for example. Hence we set effective mode
+           in case this is a CMYK ICC space for example. Hence we set effective mode
            to mode here (Bug 698721)*/
-        pgs->effective_overprint_mode = pgs->overprint_mode;
+        pgs->color[0].effective_opm = pgs->overprint_mode;
+
+        if_debug2m(gs_debug_flag_overprint, pgs->memory,
+            "[overprint] gs_do_set_overprint. Preset effective mode. pgs->color[0].effective_opm = %d pgs->color[1].effective_opm = %d\n",
+            pgs->color[0].effective_opm, pgs->color[1].effective_opm);
+
         pcs->type->set_overprint(pcs, pgs);
     }
     return code;
 }
 
-/* setoverprint */
+/* setoverprint (non-stroke case) interpreter code
+   ensures that this is called when appropriate. This
+   should only be coming when we are doing PS files.
+   As they don't have separate stroke and fill overprint
+   controls */
 void
 gs_setoverprint(gs_gstate * pgs, bool ovp)
 {
-    bool    prior_ovp = pgs->overprint;
-
     pgs->overprint = ovp;
     pgs->stroke_overprint = ovp;
-    if (prior_ovp != ovp)
-        (void)gs_do_set_overprint(pgs);
 }
 
 /* currentoverprint */
@@ -681,11 +718,7 @@ gs_currentstrokeoverprint(const gs_gstate * pgs)
 void
 gs_setfilloverprint(gs_gstate * pgs, bool ovp)
 {
-    bool    prior_ovp = pgs->overprint;
-
     pgs->overprint = ovp;
-    if (prior_ovp != ovp)
-        (void)gs_do_set_overprint(pgs);
 }
 
 /* currentstrokeoverprint */
@@ -699,15 +732,11 @@ gs_currentfilloverprint(const gs_gstate * pgs)
 int
 gs_setoverprintmode(gs_gstate * pgs, int mode)
 {
-    int     prior_mode = pgs->effective_overprint_mode;
-    int     code = 0;
-
     if (mode < 0 || mode > 1)
         return_error(gs_error_rangecheck);
     pgs->overprint_mode = mode;
-    if (pgs->overprint && prior_mode != mode)
-        code = gs_do_set_overprint(pgs);
-    return code;
+
+    return 0;
 }
 
 /* currentoverprintmode */
@@ -816,6 +845,8 @@ gs_initgraphics(gs_gstate * pgs)
     const gs_gstate gstate_initial = {
             gs_gstate_initial(1.0)
         };
+    gs_matrix m;
+    gs_make_identity(&m);
 
     gs_initmatrix(pgs);
     if ((code = gs_newpath(pgs)) < 0 ||
@@ -912,7 +943,52 @@ gs_initgraphics(gs_gstate * pgs)
     }
     pgs->in_cachedevice = 0;
 
-    return 0;
+    code = gs_settextspacing(pgs, (double)0.0);
+    if (code < 0)
+        goto exit;
+    code = gs_settextleading(pgs, (double)0.0);
+    if (code < 0)
+        goto exit;
+
+    gs_settextrenderingmode(pgs, 0);
+
+    code = gs_setwordspacing(pgs, (double)0.0);
+    if (code < 0)
+        goto exit;
+    code = gs_settexthscaling(pgs, (double)100.0);
+    if (code < 0)
+        goto exit;
+
+    gs_setaccuratecurves(pgs, true);
+
+    code = gs_setstrokeconstantalpha(pgs, 1.0);
+    if (code < 0)
+        goto exit;
+    code = gs_setfillconstantalpha(pgs, 1.0);
+    if (code < 0)
+        goto exit;
+    code = gs_setalphaisshape(pgs, 0);
+    if (code < 0)
+        goto exit;
+    code = gs_setblendmode(pgs, BLEND_MODE_Compatible);
+    if (code < 0)
+        goto exit;
+    code = gs_settextknockout(pgs, true);
+    if (code < 0)
+        goto exit;
+    code = gs_setsmoothness(pgs, 0.02); /* Match gs code */
+    if (code < 0)
+        goto exit;
+
+    code = gs_settextmatrix(pgs, &m);
+    if (code < 0)
+        goto exit;
+
+    code = gs_settextlinematrix(pgs, &m);
+    if (code < 0)
+        goto exit;
+exit:
+    return code;
 }
 
 /* setfilladjust */
@@ -977,6 +1053,13 @@ gs_currenttextspacing(const gs_gstate *pgs)
 int
 gs_settextspacing(gs_gstate *pgs, double Tc)
 {
+    int code = 0;
+    gs_fixed_point dxy;
+
+    code = gs_distance_transform2fixed(&pgs->ctm, Tc, 1, &dxy);
+    if (code < 0)
+        return code;
+
     pgs->textspacing = (float)Tc;
     return 0;
 }
@@ -1133,6 +1216,16 @@ gstate_free_parts(gs_gstate * parts, gs_memory_t * mem, client_name_t cname)
     }
 }
 
+static inline void
+gstate_parts_init_dev_color(gx_device_color *dc)
+{
+    gx_device_color_type dct = dc->type;
+    gs_graphics_type_tag_t gtt = dc->tag;
+    memset(dc, 0x00, sizeof(gx_device_color));
+    dc->type = dct;
+    dc->tag = gtt;
+}
+
 /* Allocate the privately allocated parts of a gstate. */
 static int
 gstate_alloc_parts(gs_gstate * parts, const gs_gstate * shared,
@@ -1177,6 +1270,8 @@ gstate_alloc_parts(gs_gstate * parts, const gs_gstate * shared,
         gstate_free_parts(parts, mem, cname);
         return_error(gs_error_VMerror);
     }
+    gstate_parts_init_dev_color(parts->color[0].dev_color);
+    gstate_parts_init_dev_color(parts->color[1].dev_color);
     return 0;
 }
 
@@ -1192,12 +1287,12 @@ gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_gstate * pfrom)
     gs_gstate *pgs =
         gs_alloc_struct(mem, gs_gstate, &st_gs_gstate, cname);
 
-    if (pgs == 0)
-        return 0;
+    if (pgs == NULL)
+        return NULL;
     memset(pgs, 0x00, sizeof(gs_gstate));
     if (gstate_alloc_parts(pgs, pfrom, mem, cname) < 0) {
         gs_free_object(mem, pgs, cname);
-        return 0;
+        return NULL;
     }
     pgs->memory = mem;
     return pgs;
@@ -1205,75 +1300,122 @@ gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_gstate * pfrom)
 
 /* Copy the dash pattern from one gstate to another. */
 static int
-gstate_copy_dash(gs_gstate * pto, const gs_gstate * pfrom)
+gstate_copy_dash(gs_memory_t *mem, gx_dash_params *dash , const gs_gstate * pfrom)
 {
-    return gs_setdash(pto, pfrom->line_params.dash.pattern,
+    return gx_set_dash(dash, pfrom->line_params.dash.pattern,
                       pfrom->line_params.dash.pattern_size,
-                      pfrom->line_params.dash.offset);
+                      pfrom->line_params.dash.offset, mem);
 }
 
-/* Clone an existing graphics state. */
-/* Return 0 if the allocation fails. */
-/* If reason is for_gsave, the clone refers to the old contents, */
-/* and we switch the old state to refer to the new contents. */
+typedef struct {
+    gs_gstate_parts  parts;
+    gx_dash_params   dash;
+} gs_gstate_clone_data;
+
 static gs_gstate *
-gstate_clone(gs_gstate * pfrom, gs_memory_t * mem, client_name_t cname,
-             gs_gstate_copy_reason_t reason)
+gstate_clone_core(const gs_gstate               *pfrom,
+                        gs_memory_t             *mem,
+                        client_name_t            cname,
+                        gs_gstate_clone_data    *clone_data,
+                        gs_gstate_copy_reason_t  reason)
 {
     gs_gstate *pgs = gstate_alloc(mem, cname, pfrom);
-    gs_gstate_parts parts;
+    void *pdata = NULL;
 
-    if (pgs == 0)
-        return 0;
-    GSTATE_ASSIGN_PARTS(&parts, pgs);
-    *pgs = *pfrom;
-    /* Copy the dash pattern if necessary. */
-    if (pgs->line_params.dash.pattern) {
+    if (pgs == NULL)
+        return NULL;
+    if (pfrom->client_data != NULL) {
+        pdata = (*pfrom->client_procs.alloc) (mem);
+
+        if (pdata == NULL ||
+            gstate_copy_client_data(pfrom, pdata, pfrom->client_data,
+                                    reason) < 0)
+            goto failEarly;
+    }
+    /* Copy the dash and dash pattern if necessary. */
+    clone_data->dash = gs_currentlineparams_inline(pfrom)->dash;
+    if (clone_data->dash.pattern) {
         int code;
 
-        pgs->line_params.dash.pattern = 0;      /* force allocation */
-        code = gstate_copy_dash(pgs, pfrom);
+        clone_data->dash.pattern = NULL; /* Ensures a fresh allocation */
+        code = gstate_copy_dash(mem, &clone_data->dash, pfrom);
         if (code < 0)
             goto fail;
     }
-    if (pgs->client_data != 0) {
-        void *pdata = pgs->client_data = (*pgs->client_procs.alloc) (mem);
+    /* Some records within pgs are allocated. We copy pfrom into pgs
+     * wholesale (to avoid problems with the structure being updated and
+     * us having to keep it in sync), so we copy those allocated regions
+     * out first. The caller of this routine will then put them back
+     * into either pgs or pfrom as appropriate. */
+    GSTATE_ASSIGN_PARTS(&clone_data->parts, pgs);
+    *pgs = *pfrom;
+    pgs->client_data = pdata;
 
-        if (pdata == 0 ||
-         gstate_copy_client_data(pgs, pdata, pfrom->client_data, reason) < 0
-            )
-            goto fail;
-    }
     gs_gstate_copied(pgs);
     /* Don't do anything to clip_stack. */
 
     rc_increment(pgs->device);
-    *parts.color[0].ccolor    = *pfrom->color[0].ccolor;
-    *parts.color[0].dev_color = *pfrom->color[0].dev_color;
-    *parts.color[1].ccolor    = *pfrom->color[1].ccolor;
-    *parts.color[1].dev_color = *pfrom->color[1].dev_color;
-    if (reason == copy_for_gsave) {
-        float *dfrom = pfrom->line_params.dash.pattern;
-        float *dto = pgs->line_params.dash.pattern;
+    *clone_data->parts.color[0].ccolor    = *pgs->color[0].ccolor;
+    *clone_data->parts.color[0].dev_color = *pgs->color[0].dev_color;
+    *clone_data->parts.color[1].ccolor    = *pgs->color[1].ccolor;
+    *clone_data->parts.color[1].dev_color = *pgs->color[1].dev_color;
+    cs_adjust_counts_icc(pgs, 1);
+    cs_adjust_swappedcounts_icc(pgs, 1);
 
-        GSTATE_ASSIGN_PARTS(pfrom, &parts);
-        pgs->line_params.dash.pattern = dfrom;
-        pfrom->line_params.dash.pattern = dto;
-    } else {
-        GSTATE_ASSIGN_PARTS(pgs, &parts);
-    }
-    gs_swapcolors_quick(pgs);
-    cs_adjust_counts_icc(pgs, 1);
-    gs_swapcolors_quick(pgs);
-    cs_adjust_counts_icc(pgs, 1);
     return pgs;
+
   fail:
-    memset(pgs->color, 0, 2*sizeof(gs_gstate_color));
-    gs_free_object(mem, pgs->line_params.dash.pattern, cname);
-    GSTATE_ASSIGN_PARTS(pgs, &parts);
+    gs_free_object(mem, clone_data->dash.pattern, cname);
+    if (pdata != NULL)
+        (*pfrom->client_procs.free) (pdata, mem, pgs);
+  failEarly:
     gstate_free_parts(pgs, mem, cname);
     gs_free_object(mem, pgs, cname);
-    return 0;
+
+    return NULL;
+}
+
+
+/* Clone an existing graphics state for use in gsave. The clone refers
+ * to the old contents, and the old state refers to the new contents. */
+/* Return NULL if the allocation fails. */
+static gs_gstate *
+gstate_clone_for_gsave(gs_gstate     *pfrom,
+                       client_name_t  cname)
+{
+    gs_gstate_clone_data clone_data;
+    gs_gstate *pgs = gstate_clone_core(pfrom, pfrom->memory, cname,
+                                       &clone_data, copy_for_gsave);
+
+    if (pgs == NULL)
+        return NULL;
+
+    /* Newly allocated parts go back into pfrom, not pgs! */
+    GSTATE_ASSIGN_PARTS(pfrom, &clone_data.parts);
+    gs_currentlineparams_inline(pfrom)->dash = clone_data.dash;
+
+    return pgs;
+}
+
+/* Clone an existing graphics state. The view_clip is not copied. */
+/* Return NULL if the allocation fails. */
+static gs_gstate *
+gstate_clone_for_gstate(const gs_gstate     *pfrom,
+                              gs_memory_t   *mem,
+                              client_name_t  cname)
+{
+    gs_gstate_clone_data clone_data;
+    gs_gstate *pgs = gstate_clone_core(pfrom, mem, cname, &clone_data,
+                                       copy_for_gstate);
+
+    if (pgs == NULL)
+        return NULL;
+    GSTATE_ASSIGN_PARTS(pgs, &clone_data.parts);
+    pgs->view_clip = NULL;
+    gs_currentlineparams_inline(pgs)->dash = clone_data.dash;
+    pgs->memory = mem;
+
+    return pgs;
 }
 
 /* Adjust reference counters for the whole clip stack */
@@ -1321,15 +1463,13 @@ gstate_free_contents(gs_gstate * pgs)
         gx_cpath_free(pgs->view_clip, cname);
         pgs->view_clip = NULL;
     }
-    gs_swapcolors_quick(pgs);
-    cs_adjust_counts_icc(pgs, -1);
-    gs_swapcolors_quick(pgs);
+    if (pgs->client_data != 0)
+        (*pgs->client_procs.free) (pgs->client_data, mem, pgs);
+    pgs->client_data = 0;
     cs_adjust_counts_icc(pgs, -1);
+    cs_adjust_swappedcounts_icc(pgs, -1);
     pgs->color[0].color_space = 0;
     pgs->color[1].color_space = 0;
-    if (pgs->client_data != 0)
-        (*pgs->client_procs.free) (pgs->client_data, mem);
-    pgs->client_data = 0;
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
     pgs->line_params.dash.pattern = 0;
     gstate_free_parts(pgs, mem, cname);     /* this also clears pointers to freed elements */
@@ -1346,7 +1486,8 @@ gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
     GSTATE_ASSIGN_PARTS(&parts, pto);
     /* Copy the dash pattern if necessary. */
     if (pfrom->line_params.dash.pattern || pto->line_params.dash.pattern) {
-        int code = gstate_copy_dash(pto, pfrom);
+        int code = gstate_copy_dash(pto->memory,
+                             &(gs_currentlineparams_inline(pto)->dash), pfrom);
 
         if (code < 0)
             return code;
@@ -1358,9 +1499,7 @@ gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
      * Handle references from contents.
      */
     cs_adjust_counts_icc(pto, -1);
-    gs_swapcolors_quick(pto);
-    cs_adjust_counts_icc(pto, -1);
-    gs_swapcolors_quick(pto);
+    cs_adjust_swappedcounts_icc(pto, -1);
     gx_path_assign_preserve(pto->path, pfrom->path);
     gx_cpath_assign_preserve(pto->clip_path, pfrom->clip_path);
     /*
@@ -1411,9 +1550,7 @@ gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
     }
     GSTATE_ASSIGN_PARTS(pto, &parts);
     cs_adjust_counts_icc(pto, 1);
-    gs_swapcolors_quick(pto);
-    cs_adjust_counts_icc(pto, 1);
-    gs_swapcolors_quick(pto);
+    cs_adjust_swappedcounts_icc(pto, 1);
     pto->show_gstate =
         (pfrom->show_gstate == pfrom ? pto : 0);
     return 0;
@@ -1425,8 +1562,13 @@ gs_id gx_get_clip_path_id(gs_gstate *pgs)
     return pgs->clip_path->id;
 }
 
-void gs_swapcolors_quick(gs_gstate *pgs)
+void gs_swapcolors_quick(const gs_gstate *cpgs)
 {
+    union {
+        const gs_gstate *cpgs;
+        gs_gstate *pgs;
+    } const_breaker;
+    gs_gstate *pgs;
     struct gx_cie_joint_caches_s *tmp_cie;
     gs_devicen_color_map          tmp_ccm;
     gs_client_color              *tmp_cc;
@@ -1434,6 +1576,11 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     gx_device_color              *tmp_dc;
     gs_color_space               *tmp_cs;
 
+    /* Break const just once, neatly, here rather than
+     * hackily in every caller. */
+    const_breaker.cpgs = cpgs;
+    pgs = const_breaker.pgs;
+
     tmp_cc               = pgs->color[0].ccolor;
     pgs->color[0].ccolor = pgs->color[1].ccolor;
     pgs->color[1].ccolor = tmp_cc;
@@ -1446,6 +1593,11 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     pgs->color[0].color_space = pgs->color[1].color_space;
     pgs->color[1].color_space = tmp_cs;
 
+    /* Overprint and effective_op vary with stroke/fill and cs */
+    tmp                         = pgs->color[0].effective_opm;
+    pgs->color[0].effective_opm = pgs->color[1].effective_opm;
+    pgs->color[1].effective_opm = tmp;
+
     /* Swap the bits of the gs_gstate that depend on the current color */
     tmp_cie                   = pgs->cie_joint_caches;
     pgs->cie_joint_caches     = pgs->cie_joint_caches_alt;
@@ -1455,32 +1607,23 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     pgs->color_component_map     = pgs->color_component_map_alt;
     pgs->color_component_map_alt = tmp_ccm;
 
-    tmp                = pgs->overprint;
-    pgs->overprint     = pgs->stroke_overprint;
-    pgs->stroke_overprint = tmp;
+    pgs->is_fill_color = !(pgs->is_fill_color);	/* used by overprint for fill_stroke */
 }
 
-int gs_swapcolors(gs_gstate *pgs)
+int
+gs_clip_bounds_in_user_space(gs_gstate *pgs, gs_rect *ubox)
 {
-    int prior_overprint = pgs->overprint;
+    gx_clip_path *clip_path;
+    gs_rect dbox;
+    int code;
 
-    gs_swapcolors_quick(pgs);
+    code = gx_effective_clip_path(pgs, &clip_path);
+    if (code < 0)
+        return code;
 
-    /* The following code will only call gs_do_set_overprint when we
-     * have a change:
-     * if ((prior_overprint != pgs->overprint) ||
-     *    ((prior_mode != pgs->effective_overprint_mode) &&
-     *     (pgs->overprint)))
-     *    return gs_do_set_overprint(pgs);
-     * Sadly, that's no good, as we need to call when we have swapped
-     * image space types too (separation <-> non separation for example).
-     *
-     * So instead, we call whenever at least one of them had overprint
-     * turned on.
-     */
-    if (prior_overprint || pgs->overprint)
-    {
-        return gs_do_set_overprint(pgs);
-    }
-    return 0;
+    dbox.p.x = fixed2float(clip_path->outer_box.p.x);
+    dbox.p.y = fixed2float(clip_path->outer_box.p.y);
+    dbox.q.x = fixed2float(clip_path->outer_box.q.x);
+    dbox.q.y = fixed2float(clip_path->outer_box.q.y);
+    return gs_bbox_transform_inverse(&dbox, &ctm_only(pgs), ubox);
 }
