@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -73,9 +73,6 @@ static byte const ver20[4] = {0x00, 0x02, 0x00, 0x00};
 
 /* ---------------- Font level ---------------- */
 
-GS_NOTIFY_PROC(gs_len_glyphs_release);
-GS_NOTIFY_PROC(gs_gsub_release);
-
 /* Read data from sfnts. */
 int
 gs_type42_read_data(gs_font_type42 * pfont, ulong pos, uint length, byte *buf)
@@ -201,6 +198,8 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
         memset(&pfont->data, 0, sizeof(pfont->data));
         pfont->data.string_proc = string_proc;
         pfont->data.proc_data = proc_data;
+        pfont->data.subfontID = (ushort)(subfontID >=0 ? subfontID : 0);
+        pfont->data.subfontOffset = (gs_offset_t)OffsetTableOffset;
     }
     for (i = 0; i < numTables; ++i) {
         const byte *tab = TableDirectory + i * 16;
@@ -1385,6 +1384,9 @@ gs_type42_glyph_info_by_gid(gs_font *font, gs_glyph glyph, const gs_matrix *pmat
 
     outline.memory = pfont->memory;
     if (default_members) {
+        if (pmat == NULL)
+            return gs_note_error(gs_error_undefinedresult);
+
         code = gs_default_glyph_info(font, glyph, pmat, default_members, info);
 
         if (code < 0)
@@ -2200,8 +2202,8 @@ gs_woff2sfnt(gs_memory_t *mem, stream *s, byte *outbuf, int *outbuflen)
     PUTU16(obuf, (ushort)rs, 4);
     obuf += 6;
 
-    tabbuf = gs_alloc_bytes(mem, WOFF_TABDIR_ENT_LEN * ntables, "gs_woff2sfnt(tabbuf)");
-    tabbufptrs = (byte **)gs_alloc_bytes(mem, (ntables + 1) * sizeof(byte *), "gs_woff2sfnt(tabbufptrs)");
+    tabbuf = gs_alloc_bytes(mem, WOFF_TABDIR_ENT_LEN * (size_t)ntables, "gs_woff2sfnt(tabbuf)");
+    tabbufptrs = (byte **)gs_alloc_bytes(mem, (ntables + 1) * (size_t)sizeof(byte *), "gs_woff2sfnt(tabbufptrs)");
     if (!tabbuf || !tabbufptrs) {
         code = gs_note_error(gs_error_VMerror);
         goto done;
