@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* PDF 1.4 blending functions */
@@ -24,11 +24,10 @@
 #include "gxdevcli.h"
 
 #define RAW_DUMP 0
-#define RAW_DUMP_AS_PAM 0
-/* Useful bash fragment to batch convert pams to pngs:
- * for f in *.pam; do g=${f%.*}; echo $g; convert $g.pam $g.png ; done
- * NB: "convert" is imagemagick convert, so may need to be installed.
- */
+/* We now dump as PAMs, not raws. This is because it's much easier
+ * (for me at least) to view them using:
+ *   http://ghostscript.com/~robin/pamview.html
+ * than anything else. */
 
 /* #define DUMP_TO_PNG */
 
@@ -88,20 +87,20 @@ typedef struct {
 typedef pdf14_parent_cs_params_s pdf14_parent_cs_params_t;
 
 /* This function is used for mapping Smask CMYK or RGB data to a monochrome alpha buffer */
-void smask_luminosity_mapping(int num_rows, int num_cols, int n_chan, int row_stride,
-                              int plane_stride, const byte *gs_restrict src, byte *gs_restrict des, bool isadditive,
+void smask_luminosity_mapping(int num_rows, int num_cols, int n_chan, intptr_t row_stride,
+                              intptr_t plane_stride, const byte *gs_restrict src, byte *gs_restrict des, bool isadditive,
                               gs_transparency_mask_subtype_t SMask_SubType, bool deep
 #if RAW_DUMP
                               , const gs_memory_t *mem
 #endif
                               );
-void smask_blend(byte *gs_restrict src, int width, int height, int rowstride,
-                 int planestride, bool deep);
+void smask_blend(byte *gs_restrict src, int width, int height, intptr_t rowstride,
+                 intptr_t planestride, bool deep);
 
-void smask_copy(int num_rows, int num_cols, int row_stride,
+void smask_copy(int num_rows, int num_cols, intptr_t row_stride,
                          byte *gs_restrict src, const byte *gs_restrict des);
-void smask_icc(gx_device *dev, int num_rows, int num_cols, int n_chan,
-               int row_stride, int plane_stride, byte *gs_restrict src, const byte *gs_restrict des,
+int smask_icc(gx_device *dev, int num_rows, int num_cols, int n_chan,
+               intptr_t row_stride, intptr_t plane_stride, byte *gs_restrict src, const byte *gs_restrict des,
                gsicc_link_t *icclink, bool deep);
 /* For spot colors, blend modes must be white preserving and separable */
 bool blend_valid_for_spot(gs_blend_mode_t blend_mode);
@@ -367,33 +366,31 @@ gx_color_index pdf14_encode_color16_tag(gx_device *dev, const gx_color_value col
 
 int pdf14_decode_color(gx_device * dev, gx_color_index color, gx_color_value * out);
 int pdf14_decode_color16(gx_device * dev, gx_color_index color, gx_color_value * out);
-void pdf14_gray_cs_to_cmyk_cm(gx_device * dev, frac gray, frac out[]);
-void pdf14_rgb_cs_to_cmyk_cm(gx_device * dev, const gs_gstate *pgs,
+void pdf14_gray_cs_to_cmyk_cm(const gx_device * dev, frac gray, frac out[]);
+void pdf14_rgb_cs_to_cmyk_cm(const gx_device * dev, const gs_gstate *pgs,
                            frac r, frac g, frac b, frac out[]);
-void pdf14_cmyk_cs_to_cmyk_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[]);
+void pdf14_cmyk_cs_to_cmyk_cm(const gx_device * dev, frac c, frac m, frac y, frac k, frac out[]);
 
-void pdf14_gray_cs_to_rgbspot_cm(gx_device * dev, frac gray, frac out[]);
-void pdf14_rgb_cs_to_rgbspot_cm(gx_device * dev, const gs_gstate *pgs,
+void pdf14_gray_cs_to_rgbspot_cm(const gx_device * dev, frac gray, frac out[]);
+void pdf14_rgb_cs_to_rgbspot_cm(const gx_device * dev, const gs_gstate *pgs,
     frac r, frac g, frac b, frac out[]);
-void pdf14_cmyk_cs_to_rgbspot_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[]);
+void pdf14_cmyk_cs_to_rgbspot_cm(const gx_device * dev, frac c, frac m, frac y, frac k, frac out[]);
 
-void pdf14_gray_cs_to_grayspot_cm(gx_device * dev, frac gray, frac out[]);
-void pdf14_rgb_cs_to_grayspot_cm(gx_device * dev, const gs_gstate *pgs,
+void pdf14_gray_cs_to_grayspot_cm(const gx_device * dev, frac gray, frac out[]);
+void pdf14_rgb_cs_to_grayspot_cm(const gx_device * dev, const gs_gstate *pgs,
     frac r, frac g, frac b, frac out[]);
-void pdf14_cmyk_cs_to_grayspot_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[]);
+void pdf14_cmyk_cs_to_grayspot_cm(const gx_device * dev, frac c, frac m, frac y, frac k, frac out[]);
 
-void gx_build_blended_image_row(const byte *gs_restrict buf_ptr, int planestride,
+void gx_build_blended_image_row(const byte *gs_restrict buf_ptr, intptr_t planestride,
                                 int width, int num_comp, uint16_t bg, byte *gs_restrict linebuf);
-void gx_build_blended_image_row16(const byte *gs_restrict buf_ptr, int planestride,
+void gx_build_blended_image_row16(const byte *gs_restrict buf_ptr, intptr_t planestride,
                                   int width, int num_comp, uint16_t bg, byte *gs_restrict linebuf);
 void gx_blend_image_buffer(byte *buf_ptr, int width, int height,
-                      int rowstride, int planestride, int num_comp, byte bg);
+                      intptr_t rowstride, intptr_t planestride, int num_comp, int additive);
 void gx_blend_image_buffer16(byte *buf_ptr, int width, int height,
-    int rowstride, int planestride, int num_comp, uint16_t bg);
-void gx_blend_image_buffer8to16(const byte *buf_ptr, unsigned short *buf_ptr_out,
-    int width, int height, int rowstride, int planestride, int num_comp, byte bg);
+    intptr_t rowstride, intptr_t planestride, int num_comp, int additive, bool keep_native);
 int gx_put_blended_image_custom(gx_device *target, byte *buf_ptr,
-                      int planestride, int rowstride,
+                      intptr_t planestride, intptr_t rowstride,
                       int x0, int y0, int width, int height, int num_comp, uint16_t bg, bool deep);
 
 /* Function moved between compilation units to allow inlining. */
@@ -407,11 +404,11 @@ int pdf14_mark_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 
 void dump_raw_buffer(const gs_memory_t *mem,
                      int num_rows, int width, int n_chan,
-                     int plane_stride, int rowstride,
+                     intptr_t plane_stride, intptr_t rowstride,
                      char filename[],const byte *Buffer, bool deep);
 void dump_raw_buffer_be(const gs_memory_t *mem,
                         int num_rows, int width, int n_chan,
-                        int plane_stride, int rowstride,
+                        intptr_t plane_stride, intptr_t rowstride,
                         char filename[],const byte *Buffer, bool deep);
 #endif
 
