@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -104,13 +104,13 @@ cl_cache_read_init(CL_CACHE *cache, int nslots, int64_t block_size, int64_t file
 
     if ((filesize+block_size)/block_size < nslots)
         nslots = (filesize + block_size)/block_size;	/* limit at blocks needed for entire file */
-    cache->slots = (CL_CACHE_SLOT *)gs_alloc_bytes(cache->memory, nslots * sizeof(CL_CACHE_SLOT),
+    cache->slots = (CL_CACHE_SLOT *)gs_alloc_bytes(cache->memory, (size_t)nslots * sizeof(CL_CACHE_SLOT),
                                                        "CL_CACHE slots array");
     if (cache->slots == NULL) {
         gs_free_object(cache->memory, cache, "Free CL_CACHE for IFILE");
         cache = NULL;			/* cache not possible */
     } else {
-        cache->slots[0].base = (byte *)gs_alloc_bytes(cache->memory, nslots * block_size,
+        cache->slots[0].base = (byte *)gs_alloc_bytes(cache->memory, (size_t)nslots * block_size,
                                                        "CL_CACHE_SLOT data");
         if (cache->slots[0].base == NULL) {
             gs_free_object(cache->memory, cache->slots, "Free CL_CACHE for IFILE");
