@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2024 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -21,6 +21,7 @@
 #include "gxpath.h"
 #include "gxcpath.h"
 #include "gzcpath.h"
+#include "gsutil.h"  /* For gs_next_ids */
 
 /* Define whether to look for vertical clipping regions. */
 #define CHECK_VERTICAL_CLIPPING
@@ -112,6 +113,11 @@ void
 gx_device_clip_finalize(const gs_memory_t *cmem, void *vpdev)
 {
     gx_device_clip *dev = (gx_device_clip *)vpdev;
+
+    if (dev->target != NULL) {
+        rc_decrement(dev->target, "gx_device_clip_finalize");
+        dev->target = NULL;
+    }
     if (dev->rect_list != NULL) {
         rc_decrement(dev->rect_list, "finalizing clipper device");
         dev->rect_list = NULL;
@@ -142,7 +148,7 @@ gx_make_clip_device_on_stack(gx_device_clip * dev, const gx_clip_path *pcpath, g
     dev->HWResolution[0] = target->HWResolution[0];
     dev->HWResolution[1] = target->HWResolution[1];
     dev->sgr = target->sgr;
-    dev->target = target;
+    gx_device_set_target((gx_device_forward *)dev, target);
     dev->pad = target->pad;
     dev->log2_align_mod = target->log2_align_mod;
     dev->num_planar_planes = target->num_planar_planes;
@@ -155,6 +161,8 @@ gx_make_clip_device_on_stack(gx_device_clip * dev, const gx_clip_path *pcpath, g
 void
 gx_destroy_clip_device_on_stack(gx_device_clip * dev)
 {
+    if (dev->target != NULL)
+        rc_decrement(dev->target, "gx_destroy_clip_device_on_stack");
     if (dev->cpath)
         ((gx_clip_path *)dev->cpath)->cached = (dev->current == &dev->list.single ? NULL : dev->current); /* Cast away const */
 }
@@ -187,7 +195,7 @@ gx_make_clip_device_on_stack_if_needed(gx_device_clip * dev, const gx_clip_path
     dev->HWResolution[0] = target->HWResolution[0];
     dev->HWResolution[1] = target->HWResolution[1];
     dev->sgr = target->sgr;
-    dev->target = target;
+    gx_device_set_target((gx_device_forward *)dev, target);
     dev->pad = target->pad;
     dev->log2_align_mod = target->log2_align_mod;
     dev->num_planar_planes = target->num_planar_planes;
@@ -1568,9 +1576,7 @@ static int
 do_clip_call_fill_path(clip_callback_data_t *pccd, int xc, int yc, int xec, int yec)
 {
     gx_device *tdev = pccd->tdev;
-    gx_clip_path cpath;
     gs_fixed_rect rect;
-    int code;
     dev_proc_fill_path((*proc));
 
     rect.p.x = int2fixed(xc);
@@ -1578,22 +1584,17 @@ do_clip_call_fill_path(clip_callback_data_t *pccd, int xc, int yc, int xec, int
     rect.q.x = int2fixed(xec);
     rect.q.y = int2fixed(yec);
 
-    /* The cpath calls might look expensive, but actually are
-     * achieved without any allocations. */
-    gx_cpath_init_local(&cpath, pccd->ppath->memory);
-    code = gx_cpath_from_rectangle(&cpath, &rect);
-    if (code < 0)
-        return code;
+    if (pccd->id_pool_len == 0) {
+        pccd->id_pool = gs_next_ids(pccd->tdev->memory, 100);
+        pccd->id_pool_len = 100;
+    }
+    gx_cpath_init_local_rectangle(pccd->rect_cpath, &rect, pccd->id_pool++);
+    pccd->id_pool_len--;
     proc = dev_proc(tdev, fill_path);
     if (proc == NULL)
         proc = gx_default_fill_path;
-    code = (*proc)(pccd->tdev, pccd->pgs, pccd->ppath, pccd->params,
-                   pccd->pdcolor, &cpath);
-    /* We could call gx_cpath_free here, but actually everything
-     * is allocated on the stack, so it serves no purpose. */
-    /* gx_cpath_free(&cpath, "clip_call_fill_path"); */
-
-    return code;
+    return (*proc)(pccd->tdev, pccd->pgs, pccd->ppath, pccd->params,
+                   pccd->pdcolor, pccd->rect_cpath);
 }
 
 static int
@@ -1619,7 +1620,11 @@ clip_fill_path(gx_device * dev, const gs_gstate * pgs,
     gx_device_clip *rdev = (gx_device_clip *) dev;
     clip_callback_data_t ccdata;
     gs_fixed_rect box;
+    gx_clip_path cpath;
 
+    ccdata.id_pool_len = 0;
+    gx_cpath_preinit_local_rectangle(&cpath, ppath->memory);
+    ccdata.rect_cpath = &cpath;
     ccdata.pgs = pgs;
     ccdata.ppath = ppath;
     ccdata.params = params;
