@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -79,6 +79,8 @@ typedef struct gx_saved_page_s {
     char bfname[gp_file_name_sizeof];	/* block file name */
     const clist_io_procs_t *io_procs;
     uint tile_cache_size;	/* size of tile cache */
+    size_t line_ptrs_offset;
+    int num_planar_planes;
     int64_t bfile_end_pos;		/* ftell at end of bfile */
     gx_band_params_t band_params;  /* parameters used when writing band list */
                                 /* (actual values, no 0s) */
@@ -179,7 +181,7 @@ typedef struct gx_clist_state_s gx_clist_state;
         gx_device_buf_procs_t buf_procs;\
         gs_memory_t *bandlist_memory;	/* allocator for in-memory bandlist files */\
         byte *data;			/* buffer area */\
-        uint data_size;			/* size of buffer */\
+        size_t data_size;		/* size of buffer */\
         gx_band_params_t band_params;	/* band buffering parameters */\
         bool do_not_open_or_close_bandfiles;	/* if true, do not open/close bandfiles */\
         dev_proc_dev_spec_op((*orig_spec_op)); /* Original dev spec op handler */\
@@ -261,11 +263,11 @@ typedef struct gx_device_clist_common_s {
     gx_device_clist_common_members;
 } gx_device_clist_common;
 
-#define clist_band_height(cldev) ((cldev)->page_band_height)
-#define clist_cfname(cldev) ((cldev)->page_cfname)
-#define clist_cfile(cldev) ((cldev)->page_cfile)
-#define clist_bfname(cldev) ((cldev)->page_bfname)
-#define clist_bfile(cldev) ((cldev)->page_bfile)
+#define clist_band_height(cldev) ((cldev)->page_info.band_params.BandHeight)
+#define clist_cfname(cldev) ((cldev)->page_info.cfname)
+#define clist_cfile(cldev) ((cldev)->page_info.cfile)
+#define clist_bfname(cldev) ((cldev)->page_info.bfname)
+#define clist_bfile(cldev) ((cldev)->page_info.bfile)
 
 /* Define the length of the longest dash pattern we are willing to store. */
 /* (Strokes with longer patterns are converted to fills.) */
@@ -310,7 +312,7 @@ struct gx_device_clist_writer_s {
     byte *cnext;		/* next slot in command buffer */
     byte *cend;			/* end of command buffer */
     cmd_list *ccl;		/* &clist_state.list of last command */
-    cmd_list band_range_list;	/* list of band-range commands */
+    cmd_list *band_range_list;	/* list of band-range commands */
     int band_range_min, band_range_max;		/* range for list */
     uint tile_max_size;		/* max size of a single tile (bytes) */
     uint tile_max_count;	/* max # of hash table entries */
@@ -398,7 +400,10 @@ typedef struct gx_device_clist_reader_s {
     int curr_render_thread;		/* index into array */
     int thread_lookahead_direction;	/* +1 or -1 */
     int next_band;			/* may be < 0 or >= num bands when no more remain to render */
-
+    struct gx_device_clist_reader_s *orig_clist_device;
+                                        /* This is NULL, unless we're in a worker thread for clist
+                                         * rendering, in which case it's a pointer back to the
+                                         * original clist device. */
 } gx_device_clist_reader;
 
 union gx_device_clist_s {
@@ -418,7 +423,7 @@ extern_st(st_device_clist);
 #define CLIST_IS_WRITER(cdev) ((cdev)->common.ymin < 0)
 
 /* setup before opening clist device */
-#define clist_init_params(xclist, xdata, xdata_size, xtarget, xbuf_procs, xband_params, xexternal, xmemory, xdisable, pageusestransparency)\
+#define clist_init_params(xclist, xdata, xdata_size, xtarget, xbuf_procs, xband_params, xexternal, xmemory, xdisable, pageusestransparency, pageusesoverprint)\
     BEGIN\
         (xclist)->common.data = (xdata);\
         (xclist)->common.data_size = (xdata_size);\
@@ -429,11 +434,11 @@ extern_st(st_device_clist);
         (xclist)->common.bandlist_memory = (xmemory);\
         (xclist)->writer.disable_mask = (xdisable);\
         (xclist)->writer.page_uses_transparency = (pageusestransparency);\
+        (xclist)->writer.page_uses_overprint = (pageusesoverprint);\
         (xclist)->writer.pinst = NULL;\
     END
 
-/* The device template itself is never used, only the procedures. */
-extern const gx_device_procs gs_clist_device_procs;
+void clist_initialize_device_procs(gx_device *dev);
 
 void clist_init_io_procs(gx_device_clist *pclist_dev, bool in_memory);
 
@@ -493,6 +498,10 @@ int clist_put_data(const gx_device_clist *cdev, int select, int64_t offset, cons
 /* Write out the array of color usage entries (one per band) */
 int clist_write_color_usage_array(gx_device_clist_writer *cldev);
 
+/* Write out simulated overprint CMYK equiv. values for spot colors */
+int clist_write_op_equiv_cmyk_colors(gx_device_clist_writer *cldev,
+    equivalent_cmyk_color_params *op_equiv_cmyk);
+
 /* get the color_usage summary over a Y range from the clist writer states */
 /* Not expected to be used */
 int clist_writer_color_usage(gx_device_clist_writer *cldev, int y, int height,
@@ -624,7 +633,7 @@ void clist_debug_set_ctm_imp(const gs_matrix *m);
 #define gx_device_clist_mutatable_common\
     byte skip[max(sizeof(gx_device_memory), sizeof(gx_device_clist)) -\
               sizeof(gx_device) + sizeof(double) /* padding */];\
-    long buffer_space;	          /* amount of space for clist buffer, */\
+    size_t buffer_space;          /* amount of space for clist buffer, */\
                                   /* 0 means not using clist */\
     byte *buf;	                  /* buffer for rendering */\
     gs_memory_t *buffer_memory;   /* allocator for command list */\
@@ -661,4 +670,14 @@ extern_st(st_device_clist_mutatable);
 #define CLIST_MUTATABLE_HAS_MUTATED(pdev) \
     (((gx_device_clist_mutatable *)(pdev))->buffer_space != 0)
 
+int
+clist_mutate_to_clist(gx_device_clist_mutatable  *pdev,
+                      gs_memory_t                *buffer_memory,
+                      byte                      **the_memory,
+                const gdev_space_params          *space_params,
+                      bool                        bufferSpace_is_exact,
+                const gx_device_buf_procs_t      *buf_procs,
+                      dev_proc_dev_spec_op(dev_spec_op),
+                      size_t                      min_buffer_space);
+
 #endif /* gxclist_INCLUDED */
