@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -76,7 +76,7 @@ struct gs_pattern1_instance_s {
     bool has_overlap;           /* true if step is smaller than bbox size */
                                 /* This is used to detect if we must do */
                                 /* blending if we have transparency */
-    int is_planar;              /* During clist writing we "lose" information
+    int num_planar_planes;      /* During clist writing we "lose" information
                                    about any intervening transparency device */
     /*
      * uses_mask is always true for PostScript patterns, but is false
