@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -508,6 +508,14 @@ static inline int colors_are_separable_and_linear(gx_device_color_info *info)
                               mg, mc, dg, dc, ta, ga,       \
                               GX_CINFO_UNKNOWN_SEP_LIN,     \
                               dci_std_cm_name(nc) )
+#define dci_alpha_values_add(nc, depth, mg, mc, dg, dc, ta, ga) \
+    dci_extended_alpha_values(nc, nc,			    \
+                              GX_CINFO_POLARITY_ADDITIVE,   \
+                              depth,                        \
+                              dci_std_gray_index(nc),       \
+                              mg, mc, dg, dc, ta, ga,       \
+                              GX_CINFO_UNKNOWN_SEP_LIN,     \
+                              dci_std_cm_name(nc) )
 
 /*
  * Determine the depth corresponding to a color_bits specification.
@@ -585,6 +593,8 @@ static inline int colors_are_separable_and_linear(gx_device_color_info *info)
 
 #define dci_values(nc,depth,mg,mc,dg,dc)\
   dci_alpha_values(nc, depth, mg, mc, dg, dc, 1, 1)
+#define dci_values_add(nc,depth,mg,mc,dg,dc)\
+  dci_alpha_values_add(nc, depth, mg, mc, dg, dc, 1, 1)
 #define dci_black_and_white dci_std_color(1)
 #define dci_black_and_white_() dci_black_and_white
 #define dci_color(depth,maxv,dither)\
@@ -742,7 +752,7 @@ typedef struct gdev_pagelist_s {
         int height;			/* height in pixels */\
         int pad;                        /* pad to use for buffers; 0 for default */\
         int log2_align_mod;             /* align to use for buffers; 0 for default */\
-        int is_planar;                  /* 1 planar, 0 for chunky */\
+        int num_planar_planes;          /* 1 planar, 0 for chunky */\
         int LeadingEdge;                /* see below */\
         float MediaSize[2];		/* media dimensions in points */\
         float ImagingBBox[4];		/* imageable region in points */\
@@ -780,6 +790,7 @@ typedef struct gdev_pagelist_s {
         gs_graphics_type_tag_t   graphics_type_tag;   /* e.g. vector, image or text */\
         int interpolate_control;      /* default 1 (use image /Interpolate value), 0 is NOINTERPOLATE. */\
                                       /* > 1 limits interpolation, < 0 forces interpolation */\
+        int non_strict_bounds;        /* If set, callers cannot rely on clipping fills etc to declared device bounds. */\
         gx_page_device_procs page_procs;       /* must be last */\
                 /* end of std_device_body */\
         gx_device_procs procs	/* object procedures */
@@ -1625,11 +1636,15 @@ static inline bool device_encodes_tags(const gx_device *dev)
     return (dev->graphics_type_tag & GS_DEVICE_ENCODES_TAGS) != 0;
 }
 
+static inline gs_graphics_type_tag_t device_current_tag(const gx_device *dev)
+{
+    gs_graphics_type_tag_t tag = dev->graphics_type_tag;
+    return (tag & GS_DEVICE_ENCODES_TAGS) ? (tag & ~GS_DEVICE_ENCODES_TAGS) : 0;
+}
+
 static inline bool device_is_deep(const gx_device *dev)
 {
-    bool has_tags = device_encodes_tags(dev);
-    int bits_per_comp = ((dev->color_info.depth - has_tags*8) /
-                         dev->color_info.num_components);
+    int bits_per_comp = (dev->color_info.depth / dev->color_info.num_components);
     if (bits_per_comp > 16)
         return 1;
     if (bits_per_comp == 16 && dev->color_info.num_components > 1)
@@ -1770,7 +1785,7 @@ gx_cm_opmsupported_t gx_get_opmsupported(gx_device *dev)
     if (dev->color_info.opmsupported != GX_CINFO_OPMSUPPORTED_UNKNOWN)
         return dev->color_info.opmsupported;
 
-    (void)check_cmyk_color_model_comps(dev);
+    check_opmsupported(dev);
 
     return dev->color_info.opmsupported;
 }
