@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -110,35 +110,30 @@ gx_ht_read_tf(
     --size;
     tf_type = (gx_ht_tf_type_t)*data++;
 
-    /* if no transfer function, exit now */
-    if (tf_type == gx_ht_tf_none) {
-        *ppmap = 0;
+    /* If no transfer function or identity set to NULL */
+    if (tf_type == gx_ht_tf_none || tf_type == gx_ht_tf_identity) {
+        *ppmap = NULL;
         return 1;
     }
 
-    /* allocate a transfer map */
+    /* If something strange then exit. Likely clist issue */
+    if (tf_type != gx_ht_tf_complete || size < sizeof(pmap->values))
+        return_error(gs_error_rangecheck);
+
+    /* Otherwise we have a real map. Allocate a transfer map */
     rc_alloc_struct_1( pmap,
                        gx_transfer_map,
                        &st_transfer_map,
                        mem,
                        return_error(gs_error_VMerror),
                        "gx_ht_read_tf" );
-
     pmap->id = gs_next_ids(mem, 1);
     pmap->closure.proc = 0;
     pmap->closure.data = 0;
-    if (tf_type == gx_ht_tf_identity) {
-        gx_set_identity_transfer(pmap);
-        return 1;
-    } else if (tf_type == gx_ht_tf_complete && size >= sizeof(pmap->values)) {
-        memcpy(pmap->values, data, sizeof(pmap->values));
-        pmap->proc = gs_mapped_transfer;
-        *ppmap = pmap;
-        return 1 + sizeof(pmap->values);
-    } else {
-        rc_decrement(pmap, "gx_ht_read_tf");
-        return_error(gs_error_rangecheck);
-    }
+    memcpy(pmap->values, data, sizeof(pmap->values));
+    pmap->proc = gs_mapped_transfer;
+    *ppmap = pmap;
+    return 1 + sizeof(pmap->values);
 }
 
 /*
@@ -584,7 +579,7 @@ gx_ht_read_and_install(
         /* save since the 'install' copies the order, but then clears the source order	*/
         for (i = 0; i < num_dev_comps; i++)
             components_save[i] = components[i];
-        code = gx_gstate_dev_ht_install(pgs, &dht, dht.type, dev);
+        code = gx_gstate_dev_ht_install(pgs, &dht, dht.type, dev, HT_OBJTYPE_DEFAULT);
         if (code >= 0) {
             for (i = 0; i < num_dev_comps; i++)
                 gx_ht_order_release(&components_save[i].corder, mem, false);
