@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -80,7 +80,7 @@
 #define TRY_TO_EXTEND_TRAP 0
 #endif
 
-#if defined(DEBUG) && !defined(GS_THREADSAFE)
+#ifdef COLLECT_STATS_FILL
 /* Define the statistics structure instance. */
 stats_fill_t stats_fill;
 #endif
@@ -196,7 +196,7 @@ is_spotan_device(gx_device * dev)
 {
     /* Use open_device procedure to identify the type of the device
      * instead of the standard gs_object_type() because gs_cpath_accum_device
-     * is allocaded on the stack i.e. has no block header with a descriptor
+     * is allocated on the stack i.e. has no block header with a descriptor
      * but has dev->memory set like a heap-allocated device.
      */
     return dev_proc(dev, open_device) == san_open;
@@ -426,8 +426,9 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
             int x1 = fixed2int_pixround(rbox.q.x + fo.adjust_right);
             int y1 = fixed2int_pixround(rbox.q.y + fo.adjust_above);
 
-            return gx_fill_rectangle_device_rop(x0, y0, x1 - x0, y1 - y0,
+            code = gx_fill_rectangle_device_rop(x0, y0, x1 - x0, y1 - y0,
                                                 pdevc, dev, lop);
+            goto exit;
         }
         if (fo.adjust_left | fo.adjust_right | fo.adjust_below | fo.adjust_above)
             fill_by_trapezoids = false; /* avoid double writing pixels */
@@ -444,7 +445,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
             code = gx_path_copy_reducing(ppath, &ffpath, fo.fixed_flat, NULL,
                                          pco_small_curves | pco_accurate);
             if (code < 0)
-                return code;
+                goto exit;
             ppath = &ffpath;
         }
 
@@ -472,7 +473,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
                                        (!fill_by_trapezoids && fo.fill_direct) ? -1 : (int)pgs->log_op);
         if (ppath == &ffpath)
             gx_path_free(ppath, "gx_general_fill_path");
-        return code;
+        goto exit;
     }
 
     gx_path_init_local(&ffpath, ppath->memory);
@@ -486,12 +487,12 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
         code = gx_path_copy_reducing(ppath, &ffpath, fo.fixed_flat, NULL,
                                      pco_small_curves | (pgs->accurate_curves ? pco_accurate : 0));
         if (code < 0)
-            return code;
+            goto exit;
         pfpath = &ffpath;
         if (big_path) {
             code = gx_path_merge_contacting_contours(pfpath);
             if (code < 0)
-                return code;
+                goto exit;
         }
     }
     fo.fill_by_trapezoids = fill_by_trapezoids;
@@ -511,6 +512,8 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
             if (lst.contour_count > countof(lst.local_windings)) {
                 lst.windings = (int *)gs_alloc_byte_array(pdev->memory, lst.contour_count,
                                 sizeof(int), "gx_general_fill_path");
+                if (lst.windings == NULL)
+                    return_error(gs_error_VMerror);
             } else
                 lst.windings = lst.local_windings;
             memset(lst.windings, 0, sizeof(lst.windings[0]) * lst.contour_count);
@@ -525,7 +528,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
     free_line_list(&lst);
     if (pfpath != ppath)        /* had to flatten */
         gx_path_free(pfpath, "gx_general_fill_path");
-#if defined(DEBUG) && !defined(GS_THREADSAFE)
+#ifdef COLLECT_STATS_FILL
     if (gs_debug_c('f')) {
         dmlputs(ppath->memory,
                 "[f]  # alloc    up  down horiz step slowx  iter  find  band bstep bfill\n");
@@ -548,6 +551,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
                    stats_fill.slow_order);
     }
 #endif
+exit:
     if (clipping)
         gx_destroy_clip_device_on_stack(&cdev);
     return code;
@@ -567,6 +571,97 @@ pass_shading_area_through_clip_path_device(gx_device * pdev, const gs_gstate * p
     return gx_default_fill_path(pdev, pgs, ppath, params, pdevc, pcpath);
 }
 
+/*  Optimization for shading and halftone fill :
+    The general filling algorithm subdivides the fill region into
+    trapezoid or rectangle subregions and then paints each subregion
+    with given color. If the color is complex, it also needs to be subdivided
+    into constant color rectangles. In the worst case it gives
+    a multiple of numbers of rectangles, which may be too slow.
+    A faster processing may be obtained with installing a clipper
+    device with the filling path, and then render the complex color
+    through it. The speeding up happens due to the clipper device
+    is optimised for fast scans through neighbour clipping rectangles.
+*/
+int
+gx_default_fill_path_shading_or_pattern(gx_device * pdev, const gs_gstate * pgs,
+                                        gx_path * ppath, const gx_fill_params * params,
+                                        const gx_device_color * pdevc, const gx_clip_path * pcpath)
+{
+    int code = 0, clipping = 0;
+    /*  We need a single clipping path here, because shadings and
+        halftones don't take 2 paths. Compute the clipping path intersection.
+    */
+    gx_clip_path cpath_intersection, cpath_with_shading_bbox;
+    const gx_clip_path *pcpath1, *pcpath2;
+    gs_gstate *pgs_noconst = (gs_gstate *)pgs; /* Break const. */
+
+    if (ppath != NULL) {
+        code = gx_cpath_init_local_shared_nested(&cpath_intersection, pcpath, pdev->memory, 1);
+        if (code < 0)
+            return code;
+        if (pcpath == NULL) {
+            gs_fixed_rect clip_box1;
+
+            (*dev_proc(pdev, get_clipping_box)) (pdev, &clip_box1);
+            code = gx_cpath_from_rectangle(&cpath_intersection, &clip_box1);
+        }
+        if (code >= 0)
+            code = gx_cpath_intersect_with_params(&cpath_intersection, ppath, params->rule,
+                                                  pgs_noconst, params);
+        pcpath1 = &cpath_intersection;
+    } else
+        pcpath1 = pcpath;
+    pcpath2 = pcpath1;
+    if (code >= 0)
+        code = gx_dc_pattern2_clip_with_bbox(pdevc, pdev, &cpath_with_shading_bbox, &pcpath1);
+    /* Do fill : */
+    if (code >= 0) {
+        gs_fixed_rect clip_box;
+        gs_int_rect cb;
+        const gx_rop_source_t *rs = NULL;
+        gx_device *dev;
+        gx_device_clip cdev;
+
+        gx_cpath_outer_box(pcpath1, &clip_box);
+        cb.p.x = fixed2int_pixround(clip_box.p.x);
+        cb.p.y = fixed2int_pixround(clip_box.p.y);
+        cb.q.x = fixed2int_pixround(clip_box.q.x);
+        cb.q.y = fixed2int_pixround(clip_box.q.y);
+        if (gx_dc_is_pattern2_color(pdevc) &&
+                (*dev_proc(pdev, dev_spec_op))(pdev,
+                         gxdso_pattern_handles_clip_path, NULL, 0) > 0) {
+            /* A special interaction with clist writer device :
+               pass the intersected clipping path. It uses an unusual call to
+               fill_path with NULL device color. */
+            code = (*dev_proc(pdev, fill_path))(pdev, pgs, ppath, params, NULL, pcpath1);
+            dev = pdev;
+        } else {
+            gx_make_clip_device_on_stack(&cdev, pcpath1, pdev);
+            dev = (gx_device *)&cdev;
+            if ((*dev_proc(pdev, dev_spec_op))(pdev,
+                        gxdso_pattern_shading_area, NULL, 0) > 0)
+                set_dev_proc(&cdev, fill_path, pass_shading_area_through_clip_path_device);
+            code = 0;
+            clipping = 1;
+        }
+        if (code >= 0) {
+            /* Check clip rectangle covers an actual area */
+            if (cb.p.x != cb.q.x && cb.p.y != cb.q.y)
+                code = pdevc->type->fill_rectangle(pdevc,
+                        cb.p.x, cb.p.y, cb.q.x - cb.p.x, cb.q.y - cb.p.y,
+                        dev, pgs->log_op, rs);
+        }
+        if (clipping)
+            gx_destroy_clip_device_on_stack(&cdev);
+    }
+    if (ppath != NULL)
+        gx_cpath_free(&cpath_intersection, "shading_fill_cpath_intersection");
+    if (pcpath1 != pcpath2)
+        gx_cpath_free(&cpath_with_shading_bbox, "shading_fill_cpath_intersection");
+
+    return code;
+}
+
 /*
  * Fill a path.  This is the default implementation of the driver
  * fill_path procedure.
@@ -576,91 +671,22 @@ gx_default_fill_path(gx_device * pdev, const gs_gstate * pgs,
                      gx_path * ppath, const gx_fill_params * params,
                  const gx_device_color * pdevc, const gx_clip_path * pcpath)
 {
-    int code = 0;
+    if (gx_dc_is_pattern2_color(pdevc) ||
+        pdevc->type == &gx_dc_type_data_ht_colored ||
+        (gx_dc_is_pattern1_color(pdevc) &&
+         gx_pattern_tile_is_clist(pdevc->colors.pattern.p_tile)))
+        return gx_default_fill_path_shading_or_pattern(pdev, pgs, ppath, params, pdevc, pcpath);
+    else
+        return gx_general_fill_path(pdev, pgs, ppath, params, pdevc, pcpath);
+}
 
-    if (gx_dc_is_pattern2_color(pdevc)
-        || pdevc->type == &gx_dc_type_data_ht_colored
-        || (gx_dc_is_pattern1_color(pdevc) && gx_pattern_tile_is_clist(pdevc->colors.pattern.p_tile))
-        ) {
-        /*  Optimization for shading and halftone fill :
-            The general filling algorithm subdivides the fill region into
-            trapezoid or rectangle subregions and then paints each subregion
-            with given color. If the color is complex, it also needs to be subdivided
-            into constant color rectangles. In the worst case it gives
-            a multiple of numbers of rectangles, which may be too slow.
-            A faster processing may be obtained with installing a clipper
-            device with the filling path, and then render the complex color
-            through it. The speeding up happens due to the clipper device
-            is optimised for fast scans through neighbour clipping rectangles.
-        */
-        /*  We need a single clipping path here, because shadings and
-            halftones don't take 2 paths. Compute the clipping path intersection.
-        */
-        gx_clip_path cpath_intersection, cpath_with_shading_bbox;
-        const gx_clip_path *pcpath1, *pcpath2;
-        gs_gstate *pgs_noconst = (gs_gstate *)pgs; /* Break const. */
-
-        if (ppath != NULL) {
-            code = gx_cpath_init_local_shared_nested(&cpath_intersection, pcpath, pdev->memory, 1);
-            if (code < 0)
-                return code;
-            if (pcpath == NULL) {
-                gs_fixed_rect clip_box1;
-
-                (*dev_proc(pdev, get_clipping_box)) (pdev, &clip_box1);
-                code = gx_cpath_from_rectangle(&cpath_intersection, &clip_box1);
-            }
-            if (code >= 0)
-                code = gx_cpath_intersect_with_params(&cpath_intersection, ppath, params->rule,
-                        pgs_noconst, params);
-            pcpath1 = &cpath_intersection;
-        } else
-            pcpath1 = pcpath;
-        pcpath2 = pcpath1;
-        if (code >= 0)
-            code = gx_dc_pattern2_clip_with_bbox(pdevc, pdev, &cpath_with_shading_bbox, &pcpath1);
-        /* Do fill : */
-        if (code >= 0) {
-            gs_fixed_rect clip_box;
-            gs_int_rect cb;
-            const gx_rop_source_t *rs = NULL;
-            gx_device *dev;
-            gx_device_clip cdev;
-
-            gx_cpath_outer_box(pcpath1, &clip_box);
-            cb.p.x = fixed2int_pixround(clip_box.p.x);
-            cb.p.y = fixed2int_pixround(clip_box.p.y);
-            cb.q.x = fixed2int_pixround(clip_box.q.x);
-            cb.q.y = fixed2int_pixround(clip_box.q.y);
-            if (gx_dc_is_pattern2_color(pdevc) &&
-                    (*dev_proc(pdev, dev_spec_op))(pdev,
-                         gxdso_pattern_handles_clip_path, NULL, 0) > 0) {
-                /* A special interaction with clist writer device :
-                   pass the intersected clipping path. It uses an unusual call to
-                   fill_path with NULL device color. */
-                code = (*dev_proc(pdev, fill_path))(pdev, pgs, ppath, params, NULL, pcpath1);
-                dev = pdev;
-            } else {
-                gx_make_clip_device_on_stack(&cdev, pcpath1, pdev);
-                dev = (gx_device *)&cdev;
-                if ((*dev_proc(pdev, dev_spec_op))(pdev,
-                        gxdso_pattern_shading_area, NULL, 0) > 0)
-                    set_dev_proc(&cdev, fill_path, pass_shading_area_through_clip_path_device);
-                code = 0;
-            }
-            if (code >= 0)
-                code = pdevc->type->fill_rectangle(pdevc,
-                        cb.p.x, cb.p.y, cb.q.x - cb.p.x, cb.q.y - cb.p.y,
-                        dev, pgs->log_op, rs);
-        }
-        if (ppath != NULL)
-            gx_cpath_free(&cpath_intersection, "shading_fill_cpath_intersection");
-        if (pcpath1 != pcpath2)
-            gx_cpath_free(&cpath_with_shading_bbox, "shading_fill_cpath_intersection");
-    } else {
-        code = gx_general_fill_path(pdev, pgs, ppath, params, pdevc, pcpath);
-    }
-    return code;
+int
+gx_default_lock_pattern(gx_device *pdev,
+                        gs_gstate *pgs,
+                        gs_id      pattern_id,
+                        int        lock)
+{
+    return gx_pattern_cache_entry_set_lock(pgs, pattern_id, lock);
 }
 
 /*
