@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* High level device color save/compare procedures */
@@ -114,6 +114,22 @@ bool gx_hld_saved_color_equal(const gx_hl_saved_color * psc1,
         return(false);
     }
 
+    /* early bailout for pattern comparison */
+    if (gx_dc_is_pattern1_color((gx_device_color *)(&psc1->saved_dev_color.type))) {
+
+        if (psc1->saved_dev_color.colors.pattern.id != psc2->saved_dev_color.colors.pattern.id)
+            return(false);
+        else
+            return true;
+    }
+    if (gx_dc_is_pattern2_color((gx_device_color *)(&psc1->saved_dev_color.type))) {
+        if (psc1->saved_dev_color.colors.pattern2.id != psc2->saved_dev_color.colors.pattern2.id
+         || psc1->saved_dev_color.colors.pattern2.shfill != psc2->saved_dev_color.colors.pattern2.shfill)
+            return(false);
+        else
+            return true;
+    }
+
     for (i = 0; i < GX_DEVICE_COLOR_MAX_COMPONENTS; i++) {
         if (psc1->ccolor.paint.values[i] != psc2->ccolor.paint.values[i]) {
             return(false);
@@ -154,22 +170,6 @@ bool gx_hld_saved_color_equal(const gx_hl_saved_color * psc1,
              }
         }
     }
-    else if (gx_dc_is_pattern1_color((gx_device_color *)(&psc1->saved_dev_color.type))) {
-
-        if (psc1->saved_dev_color.colors.pattern.id != psc2->saved_dev_color.colors.pattern.id
-         || psc1->saved_dev_color.colors.pattern.phase.x != psc2->saved_dev_color.colors.pattern.phase.x
-         || psc1->saved_dev_color.colors.pattern.phase.y != psc2->saved_dev_color.colors.pattern.phase.y) {
-
-            return(false);
-        }
-    }
-    else if (gx_dc_is_pattern2_color((gx_device_color *)(&psc1->saved_dev_color.type))) {
-        if (psc1->saved_dev_color.colors.pattern2.id != psc2->saved_dev_color.colors.pattern2.id
-         || psc1->saved_dev_color.colors.pattern2.shfill != psc2->saved_dev_color.colors.pattern2.shfill) {
-
-            return(false);
-        }
-    }
 
     return (true);
 }
