@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -120,23 +120,8 @@ gs_image_class_2_fracs(gx_image_enum * penum, irender_proc_t *render_fn)
                 penum->mask_color.values[i] =
                     bits2frac(penum->mask_color.values[i], 12);
         }
-        /* If the device has some unique color mapping procs due to its color space,
-           then we will need to use those and go through pixel by pixel instead
-           of blasting through buffers.  This is true for example with many of
-           the color spaces for CUPs */
-        std_cmap_procs = gx_device_uses_std_cmap_procs(penum->dev, penum->pgs);
-        if ( (gs_color_space_get_index(penum->pcs) == gs_color_space_index_DeviceN &&
-            penum->pcs->cmm_icc_profile_data == NULL) || penum->use_mask_color ||
-            penum->bps != 16 || !std_cmap_procs ||
-            gs_color_space_get_index(penum->pcs) == gs_color_space_index_DevicePixel ||
-            gs_color_space_get_index(penum->pcs) == gs_color_space_index_Indexed) {
-            /* DevicePixel color space used in mask from 3x type.  Basically
-               a simple color space that just is scaled to the device bit
-               depth when remapped. No CM needed */
-            if_debug0m('b', penum->memory, "[b]render=frac\n");
-            *render_fn =  &image_render_frac;
-            return 0;
-        } else {
+        if (penum->pcs->cmm_icc_profile_data != NULL)
+        {
             /* Set up the link now */
             const gs_color_space *pcs;
             gsicc_rendering_param_t rendering_params;
@@ -170,7 +155,8 @@ gs_image_class_2_fracs(gx_image_enum * penum, irender_proc_t *render_fn)
             } else {
                 pcs = penum->pcs;
             }
-            penum->icc_setup.is_lab = pcs->cmm_icc_profile_data->islab;
+            if (pcs->cmm_icc_profile_data != NULL)
+                penum->icc_setup.is_lab = pcs->cmm_icc_profile_data->islab;
             penum->icc_setup.must_halftone = gx_device_must_halftone(penum->dev);
             penum->icc_setup.has_transfer =
                 gx_has_transfer(penum->pgs, num_des_comps);
@@ -179,6 +165,27 @@ gs_image_class_2_fracs(gx_image_enum * penum, irender_proc_t *render_fn)
                 penum->icc_link = gsicc_get_link(penum->pgs, penum->dev, pcs, NULL,
                     &rendering_params, penum->memory);
             }
+        }
+        /* If the device has some unique color mapping procs due to its color space,
+           then we will need to use those and go through pixel by pixel instead
+           of blasting through buffers.  This is true for example with many of
+           the color spaces for CUPs */
+        std_cmap_procs = gx_device_uses_std_cmap_procs(penum->dev, penum->pgs);
+        if ( (gs_color_space_get_index(penum->pcs) == gs_color_space_index_DeviceN &&
+            penum->pcs->cmm_icc_profile_data == NULL) ||
+            (gs_color_space_get_index(penum->pcs) == gs_color_space_index_Separation &&
+            penum->pcs->cmm_icc_profile_data == NULL) ||
+            penum->use_mask_color ||
+            penum->bps != 16 || !std_cmap_procs ||
+            gs_color_space_get_index(penum->pcs) == gs_color_space_index_DevicePixel ||
+            gs_color_space_get_index(penum->pcs) == gs_color_space_index_Indexed) {
+            /* DevicePixel color space used in mask from 3x type.  Basically
+               a simple color space that just is scaled to the device bit
+               depth when remapped. No CM needed */
+            if_debug0m('b', penum->memory, "[b]render=frac\n");
+            *render_fn =  &image_render_frac;
+            return 0;
+        } else {
             /* Use the direct unpacking proc */
             penum->unpack = sample_unpackicc_16;
             if_debug0m('b', penum->memory, "[b]render=icc16\n");
@@ -405,8 +412,11 @@ image_render_frac(gx_image_enum * penum, const byte * buffer, int data_x,
                 }
                 break;
         }
-        mcode = remap_color(&cc, pcs, pdevc_next, pgs, dev,
-                           gs_color_select_source);
+        if (penum->icc_link == NULL || pcs->cmm_icc_profile_data == NULL)
+            mcode = remap_color(&cc, pcs, pdevc_next, pgs, dev, gs_color_select_source);
+        else
+            mcode = gx_remap_ICC_with_link(&cc, pcs, pdevc_next, pgs, dev,
+                                           gs_color_select_source, penum->icc_link);
         if (mcode < 0)
             goto fill;
 f:
@@ -631,8 +641,11 @@ image_render_icc16(gx_image_enum * penum, const byte * buffer, int data_x,
     } else {
         spp_cm = num_des_comps;
         psrc_cm = (unsigned short*) gs_alloc_bytes(pgs->memory,
-                        sizeof(unsigned short)  * w * spp_cm/spp,
+                        sizeof(unsigned short)  * (size_t)w * spp_cm/spp,
                         "image_render_icc16");
+        if (psrc_cm == NULL)
+            return_error(gs_error_VMerror);
+
         psrc_cm_start = psrc_cm;
         bufend = psrc_cm +  w * spp_cm/spp;
         if (penum->icc_link->is_identity) {
@@ -655,8 +668,12 @@ image_render_icc16(gx_image_enum * penum, const byte * buffer, int data_x,
             if (need_decode) {
                 /* Need decode and CM.  This is slow but does not happen that often */
                 psrc_decode = (unsigned short*) gs_alloc_bytes(pgs->memory,
-                                sizeof(unsigned short) * w * spp_cm/spp,
+                                sizeof(unsigned short) * (size_t)w * spp,
                                 "image_render_icc16");
+                if (psrc_decode == NULL) {
+                    gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_icc16");
+                    return_error(gs_error_VMerror);
+                }
                 if (!penum->use_cie_range) {
                     decode_row16(penum, psrc, spp, psrc_decode,
                                     (const unsigned short*) (psrc_decode+w));
@@ -666,19 +683,23 @@ image_render_icc16(gx_image_enum * penum, const byte * buffer, int data_x,
                                         (const unsigned short*) (psrc_decode+w),
                                          get_cie_range(penum->pcs));
                 }
-                (penum->icc_link->procs.map_buffer)(dev, penum->icc_link,
+                code = (penum->icc_link->procs.map_buffer)(dev, penum->icc_link,
                                                     &input_buff_desc,
                                                     &output_buff_desc,
                                                     (void*) psrc_decode,
                                                     (void*) psrc_cm);
                 gs_free_object(pgs->memory, (byte *)psrc_decode, "image_render_color_icc");
+                if (code < 0)
+                    return code;
             } else {
                 /* CM only. No decode */
-                (penum->icc_link->procs.map_buffer)(dev, penum->icc_link,
+                code = (penum->icc_link->procs.map_buffer)(dev, penum->icc_link,
                                                     &input_buff_desc,
                                                     &output_buff_desc,
                                                     (void*) psrc,
                                                     (void*) psrc_cm);
+                if (code < 0)
+                    return code;
             }
         }
     }
