@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -230,7 +230,7 @@ gx_overprint_generic_fill_rectangle(
     /* allocate space for a scanline of color indices */
     pcolor_buff = (gx_color_index *)
                       gs_alloc_bytes( mem,
-                                      w *  ARCH_SIZEOF_COLOR_INDEX,
+                                      (size_t)w *  ARCH_SIZEOF_COLOR_INDEX,
                                       "overprint generic fill rectangle" );
     if (pcolor_buff == 0)
         return gs_note_error(gs_error_VMerror);
@@ -308,8 +308,7 @@ gx_overprint_generic_fill_rectangle(
         gb_rect.q.y = y;
         code = dev_proc(tdev, get_bits_rectangle)( tdev,
                                                    &gb_rect,
-                                                   &gb_params,
-                                                   0 );
+                                                   &gb_params );
         if (code < 0)
             break;
         unpack_proc(pcolor_buff, gb_buff, 0, w, depth);
@@ -450,7 +449,7 @@ gx_overprint_sep_fill_rectangle_1(
     gs_get_bits_params_t    gb_params;
     gs_int_rect             gb_rect;
     int                     code = 0, bit_w, depth = tdev->color_info.depth;
-    int                     raster;
+    int64_t                  raster;
     mono_fill_chunk         rep_color, rep_mask;
 
     fit_fill(tdev, x, y, w, h);
@@ -466,7 +465,8 @@ gx_overprint_sep_fill_rectangle_1(
     }
 
     /* allocate a buffer for the returned data */
-    raster = bitmap_raster(w * depth);
+    if (check_64bit_multiply(w, depth, &raster) != 0)
+        return gs_note_error(gs_error_undefinedresult);
     gb_buff = gs_alloc_bytes(mem, raster, "overprint sep fill rectangle 1");
     if (gb_buff == 0)
         return gs_note_error(gs_error_VMerror);
@@ -496,8 +496,7 @@ gx_overprint_sep_fill_rectangle_1(
         gb_rect.q.y = y;
         code = dev_proc(tdev, get_bits_rectangle)( tdev,
                                                    &gb_rect,
-                                                   &gb_params,
-                                                   0 );
+                                                   &gb_params );
         if (code < 0)
             break;
         bits_fill_rectangle_masked( gb_buff,
@@ -536,7 +535,8 @@ gx_overprint_sep_fill_rectangle_2(
     byte *                  gb_buff = 0;
     gs_get_bits_params_t    gb_params;
     gs_int_rect             gb_rect;
-    int                     code = 0, byte_w, raster;
+    int                     code = 0, byte_w;
+    int64_t                  raster;
     int                     byte_depth = tdev->color_info.depth >> 3;
     byte *                  pcolor;
     byte *                  pmask;
@@ -553,7 +553,8 @@ gx_overprint_sep_fill_rectangle_2(
 #endif
 
     /* allocate a buffer for the returned data */
-    raster = bitmap_raster(w * (byte_depth << 3));
+    if (check_64bit_multiply(w, (byte_depth << 3), &raster) != 0)
+        return gs_note_error(gs_error_undefinedresult);
     gb_buff = gs_alloc_bytes(mem, raster, "overprint sep fill rectangle 2");
     if (gb_buff == 0)
         return gs_note_error(gs_error_VMerror);
@@ -586,8 +587,7 @@ gx_overprint_sep_fill_rectangle_2(
         gb_rect.q.y = y;
         code = dev_proc(tdev, get_bits_rectangle)( tdev,
                                                    &gb_rect,
-                                                   &gb_params,
-                                                   0 );
+                                                   &gb_params );
         if (code < 0)
             break;
         for (i = 0, j = 0; i < byte_w; i++, cp++) {
