@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -19,29 +19,24 @@
 #ifndef gxpcache_INCLUDED
 #  define gxpcache_INCLUDED
 
+#include "std.h"
+#include "gsdcolor.h"
+
 /*
  * Define a cache for rendered Patterns.  This is currently an open
  * hash table with single probing (no reprobing) and round-robin
  * replacement.  Obviously, we can do better in both areas.
  */
-#ifndef gx_pattern_cache_DEFINED
-#  define gx_pattern_cache_DEFINED
 typedef struct gx_pattern_cache_s gx_pattern_cache;
 
-#endif
-#ifndef gx_color_tile_DEFINED
-#  define gx_color_tile_DEFINED
-typedef struct gx_color_tile_s gx_color_tile;
-
-#endif
 struct gx_pattern_cache_s {
     gs_memory_t *memory;
     gx_color_tile *tiles;
     uint num_tiles;
     uint tiles_used;
     uint next;			/* round-robin index */
-    ulong bits_used;
-    ulong max_bits;
+    size_t bits_used;
+    size_t max_bits;
     void (*free_all) (gx_pattern_cache *);
 };
 
