@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -330,7 +330,7 @@ adjust_point_to_tangent(segment * pseg, const segment * next,
             return;		/* anomalous case */
         if_debug1('2', "[2]adjusting vertical: DT = %g\n",
                   fixed2float(DT));
-        if ((DT ^ fD) > 0)
+        if ((DT ^ fD) > 0) /* lgtm [cpp/bitwise-sign-check] */
             pseg->pt.y = DT + y0;
     } else if (fD == 0) {
         /* Horizontal tangent. */
@@ -338,7 +338,7 @@ adjust_point_to_tangent(segment * pseg, const segment * next,
 
         if_debug1('2', "[2]adjusting horizontal: CT = %g\n",
                   fixed2float(CT));
-        if ((CT ^ fC) > 0)
+        if ((CT ^ fC) > 0) /* lgtm [cpp/bitwise-sign-check] */
             pseg->pt.x = CT + x0;
     } else {
         /* General case. */
