@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -203,8 +203,7 @@ gx_flattened_iterator__init(gx_flattened_iterator *self,
         return false;
     self->curve = true;
     self->k = k;
-#ifndef GS_THREADSAFE
-#   ifdef DEBUG
+#ifdef DEBUG
         if (gs_debug_c('3')) {
             dlprintf4("[3]x0=%f y0=%f x1=%f y1=%f\n",
                       fixed2float(self->x0), fixed2float(self->y0),
@@ -213,7 +212,6 @@ gx_flattened_iterator__init(gx_flattened_iterator *self,
                       fixed2float(x2), fixed2float(y2),
                       fixed2float(self->x3), fixed2float(self->y3), self->k);
         }
-#   endif
 #endif
     if (k == -1) {
         /* A special hook for gx_subdivide_curve_rec.
@@ -333,7 +331,7 @@ gx_flattened_iterator__init_line(gx_flattened_iterator *self,
     return true;
 }
 
-#if defined(DEBUG) && !defined(GS_THREADSAFE)
+#ifdef DEBUG
 static inline void
 gx_flattened_iterator__print_state(gx_flattened_iterator *self)
 {
@@ -418,7 +416,7 @@ gx_flattened_iterator__next(gx_flattened_iterator *self)
         --self->i;
         if (self->i == 0)
             goto last; /* don't bother with last accum */
-#	if defined(DEBUG) && !defined(GS_THREADSAFE)
+#	ifdef DEBUG
             gx_flattened_iterator__print_state(self);
 #	endif
 #	define accum(i, r, di, dr, rmask)\
@@ -490,7 +488,7 @@ gx_flattened_iterator__prev(gx_flattened_iterator *self)
     }
     gx_flattened_iterator__unaccum(self);
     self->i++;
-#   if defined(DEBUG) && !defined(GS_THREADSAFE)
+#   ifdef DEBUG
     if_debug5('3', "[3]%s x=%g, y=%g x=%ld y=%ld\n",
               (((self->x ^ self->lx1) | (self->y ^ self->ly1)) & float2fixed(-0.5) ?
                "add" : "skip"),
@@ -562,7 +560,13 @@ top :
             return code;
         notes |= sn_not_first;
         goto top;
-    } else if (k == -1) {
+    } else if (k < 0) {
+        /* This used to be k == -1, but... If we have a very long curve, we will first
+         * go through the code above to split the long curve into 2. In fact for very
+         * long curves we can go through that multiple times. This can lead to k being
+         * < -1 by the time we finish subdividing the curve, and that meant we did not
+         * satisfy the exit condition here, leading to a loop until VM error.
+         */
         /* fixme : Don't need to init the iterator. Just wanted to check in_range. */
         return gx_path_add_curve_notes(ppath, pc->p1.x, pc->p1.y, pc->p2.x, pc->p2.y,
                         pc->pt.x, pc->pt.y, notes);
