@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -87,7 +87,8 @@
 #include <zlib.h>
 
 int gs_log_error(int err, const char *file, int line);
-
+/* Prototype to placate compiler */
+int spgetcc(stream *s, bool b);
 /*
  * The rom file system is an array of pointers to nodes, terminated by a NULL
  */
@@ -147,6 +148,19 @@ static inline int isbigendian(void)
     return u.c[0] != 1;
 }
 
+/* The gp_local_arg_encoding_get_codepoint() function presnet in (some) OS-specific
+ * files (gp_unix.c, gp_win32.c) now uses streams, so it won't compile unless there
+ * is an implementation of spgetcc(). We don't actually use the function for mkromfs
+ * so we just need to make sure it compiles.
+ */
+int spgetcc(stream *s, bool b)
+{
+    (void) s; /* avoid 'unused' warning from compilers */
+    (void) b;
+
+    return 0;
+}
+
 /* mkromfs doesn't use gp_stat, but it does link gp_misc.c which includes
    call to gp_stat_impl(). Rather than major build upheaval for something not
    used, just define a dummy here for Windows.
@@ -254,12 +268,27 @@ lprintf_file_and_line(const char *file, int line)
 {
     errprintf_nomem("%s(%d): ", file, line);
 }
+
+void
+dmprintf_file_and_line(const gs_memory_t *mem, const char *file, int line)
+{
+    (void)mem;
+    errprintf_nomem("%s(%d): ", file, line);
+}
 #else
 void
 lprintf_file_only(FILE * f, const char *file)
 {
     errprintf_nomem("%s(?): ", file);
 }
+
+void
+dmprintf_file_and_line(const gs_memory_t *mem, const char *file)
+{
+    (void)mem;
+    errprintf_nomem("%s(%d): ", file);
+}
+
 #endif
 
 gs_memory_t *gp_get_debug_mem_ptr(void)
@@ -313,6 +342,14 @@ gs_log_error(int err, const char *file, int line)
     return err;
 }
 
+int
+gs_add_control_path(const gs_memory_t *mem, gs_path_control_t type, const char *path)
+{
+    (void)mem;
+    (void)type;
+    (void)path;
+    return 0;
+}
 /*******************************************************************************
  * The following is a REALLY minimal gs_memory_t for use by the gp_ functions
  *******************************************************************************/
@@ -1988,6 +2025,8 @@ process_initfile(char *initfile, char *gconfig_h, const char *os_prefix,
     memset(linebuf, 0, sizeof(linebuf));
     node = calloc(1, sizeof(romfs_inode));
     node->name = rom_filename;	/* without -P prefix, with -d rom_prefix */
+    node->data = NULL;
+    node->data_lengths = NULL;
 
     merge_to_ps(os_prefix, initfile, in, config, verbose);
 
@@ -2070,9 +2109,9 @@ process_initfile(char *initfile, char *gconfig_h, const char *os_prefix,
     /* write data for this file */
     inode_write(out, node, compression, *inode_count, totlen, splits, verbose);
     /* clean up */
-    inode_clear(node);
     (*inode_count)++;
 done:
+    inode_clear(node);
     free(node);
     free(cbuf);
     free(ubuf);
