@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -25,7 +25,7 @@
 #include "sdct.h"
 #include "sjpeg.h"
 
-#define ICC_OVERHEAD  16		
+#define ICC_OVERHEAD  16
 #define MAX_MARKER_DATA_SIZE  (65535 - ICC_OVERHEAD)
 
 public_st_jpeg_compress_data();
@@ -107,7 +107,8 @@ s_DCTE_process(stream_state * st, stream_cursor_read * pr,
         case 1:		/* initialized, Markers not written */
             if (pw->limit - pw->ptr < ss->Markers.size)
                 return 1;
-            memcpy(pw->ptr + 1, ss->Markers.data, ss->Markers.size);
+            if (ss->Markers.size)
+                memcpy(pw->ptr + 1, ss->Markers.data, ss->Markers.size);
             pw->ptr += ss->Markers.size;
             ss->phase = 2;
             /* falls through */
@@ -135,9 +136,9 @@ s_DCTE_process(stream_state * st, stream_cursor_read * pr,
             dest->free_in_buffer = pw->limit - pw->ptr;
             ss->phase = 3;
             /* falls through */
-        case  3:     
+        case  3:
             /* If we have it, then write out the ICC profile */
-            /* Due to size limitations allowed in APP0 markers, the profile 
+            /* Due to size limitations allowed in APP0 markers, the profile
                may have to be written in mutiple markers */
 	        if (ss->icc_profile != NULL) {
 		        static const char marker[2] = {0xFF, 0xE2};  /* JPEG_APP0 + 2 */
@@ -156,7 +157,7 @@ s_DCTE_process(stream_state * st, stream_cursor_read * pr,
                     if (size > MAX_MARKER_DATA_SIZE)
                       size = MAX_MARKER_DATA_SIZE;
 
-                    /* In this case we are just getting started with the 
+                    /* In this case we are just getting started with the
                        header of the marker.  Write that portion out */
 		            if (ss->icc_position == -1) {
 			            byte length_byte[2];
