@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -240,9 +240,9 @@ s_ISpecialDownScale_init(stream_state * st)
     ss->sizeofPixelIn = ss->params.BitsPerComponentIn / 8;
     ss->sizeofPixelOut = ss->params.BitsPerComponentOut / 8;
 
-    ss->src_size = 
+    ss->src_size =
         ss->params.WidthIn * ss->sizeofPixelIn * ss->params.spp_interp;
-    ss->dst_size = 
+    ss->dst_size =
         ss->params.WidthOut * ss->sizeofPixelOut * ss->params.spp_interp;
 
     /* Initialize destination DDAs. */
@@ -254,15 +254,15 @@ s_ISpecialDownScale_init(stream_state * st)
     dda_init(ss->dda_y, 0, ss->params.HeightOut, ss->params.HeightIn);
 
     /* create intermediate image to hold horizontal zoom */
-    ss->tmp = 
-        gs_alloc_byte_array(mem, ss->params.WidthOut * ss->params.spp_interp,
+    ss->tmp =
+        gs_alloc_byte_array(mem, (size_t)ss->params.WidthOut * ss->params.spp_interp,
                             ss->sizeofPixelIn, "image_scale tmp");
     /* Allocate buffers for 1 row of source and destination. */
-    ss->dst = 
-        gs_alloc_byte_array(mem, ss->params.WidthOut * ss->params.spp_interp,
+    ss->dst =
+        gs_alloc_byte_array(mem, (size_t)ss->params.WidthOut * ss->params.spp_interp,
                             ss->sizeofPixelOut, "image_scale dst");
-    ss->src = 
-        gs_alloc_byte_array(mem, ss->params.WidthIn * ss->params.spp_interp,
+    ss->src =
+        gs_alloc_byte_array(mem, (size_t)ss->params.WidthIn * ss->params.spp_interp,
                             ss->sizeofPixelIn, "image_scale src");
     if (ss->tmp == 0 || ss->dst == 0 || ss->src == 0) {
         s_ISpecialDownScale_release(st);
