@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -94,12 +94,10 @@ s_write_ps_string(stream * s, const byte * str, uint size, int print_ok)
         stream_cursor_write w;
         int status;
 
-        r.ptr = str - 1;
-        r.limit = r.ptr + size;
-        w.limit = buf + sizeof(buf) - 1;
+        stream_cursor_read_init(&r, str, size);
         do {
-            /* One picky compiler complains if we initialize to buf - 1. */
-            w.ptr = buf;  w.ptr--;
+            stream_cursor_write_init(&w, buf, sizeof(buf));
+
             status = (*templat->process) (st, &r, &w, true);
             stream_write(s, buf, (uint) (w.ptr + 1 - buf));
         }
