@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -366,7 +366,8 @@ s_filter_close(register stream * s)
 
         if (status != 0 && status != EOFC)
             return status;
-        status = sflush(stemp);
+        if (status != EOFC)
+            status = sflush(stemp);
         if (status != 0 && status != EOFC)
             return status;
     }
@@ -842,7 +843,8 @@ sreadbuf(stream * s, stream_cursor_write * pbuf)
                        (uint) (pw->limit - pw->ptr), eof);
             oldpos = pw->ptr;
             status = (*curr->procs.process) (curr->state, pr, pw, eof);
-            pr->limit += left;
+            if (pr->limit != NULL)
+                pr->limit += left;
             if_debug5m('s', s->memory, "[s]after read "PRI_INTPTR", nr=%u, nw=%u, status=%d, position=%"PRId64"\n",
                        (intptr_t) curr, (uint) (pr->limit - pr->ptr),
                        (uint) (pw->limit - pw->ptr), status, s->position);
@@ -1008,7 +1010,8 @@ stream_move(stream_cursor_read * pr, stream_cursor_write * pw)
 static void
 stream_compact(stream * s, bool always)
 {
-    if (s->cursor.r.ptr >= s->cbuf && (always || s->end_status >= 0)) {
+    if (s->cbuf != NULL && s->cursor.r.ptr >= s->cbuf
+        && (always || s->end_status >= 0)) {
         uint dist = s->cursor.r.ptr + 1 - s->cbuf;
 
         memmove(s->cbuf, s->cursor.r.ptr + 1,
@@ -1283,8 +1286,11 @@ s_add_filter(stream **ps, const stream_template *templat,
     ess->templat = templat;
     ess->memory = mem;
     es->memory = mem;
-    if (s_init_filter(es, ess, buf, bsize, *ps) < 0)
+    if (s_init_filter(es, ess, buf, bsize, *ps) < 0) {
+        gs_free_object(mem, buf, "s_add_filter(buf)");
+        gs_free_object(mem, es, "s_add_filter(stream)");
         return 0;
+    }
     *ps = es;
     return es;
 }
