@@ -64,32 +64,27 @@ typedef struct gx_device_alps_s gx_device_alps;
 
 #define dev_alps ((gx_device_alps *) pdev)
 
-static gx_device_procs alps_procs = {
-    alps_open,
-    gx_default_get_initial_matrix,
-    NULL,			/* sync_output */
-    gdev_prn_output_page,
-    gdev_prn_close,
-    NULL,			/* map_rgb_color,*/
-    alps_map_color_rgb,
-    NULL,			/* fill_rectangle */
-    NULL,			/* tile_rectangle */
-    NULL,			/* copy_mono */
-    NULL,			/* copy_color */
-    NULL,			/* draw_line */
-    NULL,			/* get_bits */
-    alps_get_params,
-    alps_put_params,
-    alps_map_cmyk_color,
-    NULL,			/* get_xfont_procs */
-    NULL,			/* get_xfont_device */
-    NULL,			/* map_rgb_alpha_color */
-    gx_page_device_get_page_device /* get_page_device */
-};
+static void
+alps_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs(dev);
+
+    set_dev_proc(dev, open_device, alps_open);
+    set_dev_proc(dev, map_color_rgb, alps_map_color_rgb);
+    set_dev_proc(dev, get_params, alps_get_params);
+    set_dev_proc(dev, put_params, alps_put_params);
+    set_dev_proc(dev, map_cmyk_color, alps_map_cmyk_color);
+
+    /* The static init used in previous versions of the code leave
+     * encode_color and decode_color set to NULL (which are then rewritten
+     * by the system to the default. For compatibility we do the same. */
+    set_dev_proc(dev, encode_color, NULL);
+    set_dev_proc(dev, decode_color, NULL);
+}
 
 #define alps_device(dname, print_page) \
 {\
-    prn_device_body(gx_device_alps, alps_procs, dname,\
+    prn_device_body(gx_device_alps, alps_initialize_device_procs, dname,\
                     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,\
                     600, 600,\
                     0, 0, 0, 0, /* margin */\
@@ -162,10 +157,11 @@ alps_get_params(gx_device *pdev, gs_param_list *plist)
 }
 
 static int
-alps_put_param_bool(gs_param_list *plist, gs_param_name pname, int *pvalue,
+alps_put_param_bool(gs_param_list *plist, gs_param_name pname, bool *pvalue,
                    int ecode)
 {
-    int code, value;
+    int code;
+    bool value;
     switch (code = param_read_bool(plist, pname, &value)) {
     default:
         return code;
