@@ -34,11 +34,19 @@ static dev_proc_image_out(rpdl_image_out);
 static void rpdl_printer_initialize(gx_device_printer * pdev, gp_file * prn_stream, int num_copies);
 static void rpdl_paper_set(gx_device_printer * pdev, gp_file * prn_stream);
 
-static gx_device_procs rpdl_prn_procs =
-lprn_procs(rpdl_open, gdev_prn_output_page, rpdl_close);
+static void
+rpdl_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_mono(dev);
+
+    set_dev_proc(dev, open_device, rpdl_open);
+    set_dev_proc(dev, close_device, rpdl_close);
+    set_dev_proc(dev, get_params, lprn_get_params);
+    set_dev_proc(dev, put_params, lprn_put_params);
+}
 
 gx_device_lprn far_data gs_rpdl_device =
-lprn_device(gx_device_lprn, rpdl_prn_procs, "rpdl",
+lprn_device(gx_device_lprn, rpdl_initialize_device_procs, "rpdl",
             DPI, DPI, 0.0, 0.0, 0.0, 0.0, 1,
             rpdl_print_page_copies, rpdl_image_out);
 
@@ -48,8 +56,8 @@ lprn_device(gx_device_lprn, rpdl_prn_procs, "rpdl",
 static int
 rpdl_open(gx_device * pdev)
 {
-    int xdpi = pdev->x_pixels_per_inch;
-    int ydpi = pdev->y_pixels_per_inch;
+    int xdpi = (int)pdev->x_pixels_per_inch;
+    int ydpi = (int)pdev->y_pixels_per_inch;
 
     /* Resolution Check */
     if (xdpi != ydpi)
@@ -79,12 +87,14 @@ rpdl_print_page_copies(gx_device_printer * pdev, gp_file * prn_stream, int num_c
     int code = 0;
     int bpl = gdev_mem_bytes_per_scan_line(pdev);
     int maxY = lprn->BlockLine / lprn->nBh * lprn->nBh;
+    size_t Bufsize;
 
     /* printer initialize */
     if (pdev->PageCount == 0)
         rpdl_printer_initialize(pdev, prn_stream, num_coipes);
 
-    if (!(lprn->CompBuf = gs_malloc(pdev->memory->non_gc_memory, bpl * 3 / 2 + 1, maxY, "rpdl_print_page_copies(CompBuf)")))
+    Bufsize = bpl * 3 / 2 + 1;
+    if (!(lprn->CompBuf = gs_malloc(pdev->memory->non_gc_memory, Bufsize, maxY, "rpdl_print_page_copies(CompBuf)")))
         return_error(gs_error_VMerror);
 
     lprn->NegativePrint = false; /* Not Support */
@@ -93,7 +103,7 @@ rpdl_print_page_copies(gx_device_printer * pdev, gp_file * prn_stream, int num_c
     if (code < 0)
         return code;
 
-    gs_free(pdev->memory->non_gc_memory, lprn->CompBuf, bpl * 3 / 2 + 1, maxY, "rpdl_print_page_copies(CompBuf)");
+    gs_free(pdev->memory->non_gc_memory, lprn->CompBuf, Bufsize, maxY, "rpdl_print_page_copies(CompBuf)");
 
     gp_fprintf(prn_stream, "\014");	/* Form  Feed */
 
@@ -238,8 +248,8 @@ rpdl_paper_set(gx_device_printer * pdev, gp_file * prn_stream)
     /* Page size match tolerance in points */
     #define TOL 5
 
-    width = pdev->MediaSize[0];
-    height = pdev->MediaSize[1];
+    width = (int)pdev->MediaSize[0];
+    height = (int)pdev->MediaSize[1];
 
     if (width < height) {
         w = width;
