@@ -71,6 +71,7 @@
     ghostscript's header files are not self-contained. Therefore, if this file
     does not compile because of undefined symbols, just add include directives
     until it does.  */
+#include "gserrors.h"
 #include "iref.h"       /* needed by icstate.h */
 #include "gsmemraw.h"   /* needed by icstate.h */
 #include "gsmemory.h"   /* needed by icstate.h */
@@ -836,11 +837,12 @@ int eprn_set_page_layout(eprn_Device *dev)
 
 ******************************************************************************/
 
-void eprn_init_device(eprn_Device *dev, const eprn_PrinterDescription *desc)
+int eprn_init_device(eprn_Device *dev, const eprn_PrinterDescription *desc)
 {
   eprn_Eprn *eprn = &dev->eprn;
   int j;
   float hres, vres;
+  int code;
 
   if (dev->is_open) gs_closedevice((gx_device *)dev);
 
@@ -867,9 +869,11 @@ void eprn_init_device(eprn_Device *dev, const eprn_PrinterDescription *desc)
   eprn->intensity_rendering = eprn_IR_halftones;
   hres = dev->HWResolution[0];
   vres = dev->HWResolution[1];
-  eprn_check_colour_info(desc->colour_info, &eprn->colour_model,
+  code = eprn_check_colour_info(desc->colour_info, &eprn->colour_model,
       &hres, &vres, &eprn->black_levels, &eprn->non_black_levels);
-
+  if (code) {
+    return code;
+  }
   if (eprn->pagecount_file != NULL) {
     gs_free(dev->memory->non_gc_memory, eprn->pagecount_file, strlen(eprn->pagecount_file) + 1,
       sizeof(char), "eprn_init_device");
@@ -878,7 +882,7 @@ void eprn_init_device(eprn_Device *dev, const eprn_PrinterDescription *desc)
 
   eprn->media_position_set = false;
 
-  return;
+  return 0;
 }
 
 /******************************************************************************
@@ -1009,7 +1013,7 @@ int eprn_open_device(gx_device *device)
   /* Read the page count value */
   if (eprn->pagecount_file != NULL) {
     unsigned long count;
-    if (pcf_getcount(eprn->pagecount_file, &count) == 0)
+    if (pcf_getcount(device->memory, eprn->pagecount_file, &count) == 0)
       device->PageCount = count;
        /* unsigned to signed. The C standard permits
           an implementation to generate an overflow indication if the value is
@@ -1113,7 +1117,7 @@ static void eprn_forget_defaultmatrix(gx_device *device)
 {
   eprn_Eprn *eprn = &((eprn_Device *)device)->eprn;
 
-  gs_setdefaultmatrix((gs_state *)eprn->pis, NULL);
+  gs_setdefaultmatrix((gs_gstate *)eprn->pgs, NULL);
 
   return;
 }
@@ -1163,7 +1167,7 @@ int eprn_output_page(gx_device *dev, int num_copies, int flush)
   /* On success, record the number of pages printed */
   if (rc == 0 && eprn->pagecount_file != NULL) {
     assert(num_copies > 0);     /* because of signed/unsigned */
-    if (pcf_inccount(eprn->pagecount_file, num_copies) != 0) {
+    if (pcf_inccount(dev->memory, eprn->pagecount_file, num_copies) != 0) {
       /* pcf_inccount() has issued an error message. */
       eprintf(
         "  No further attempts will be made to access the page count file.\n");
