@@ -150,7 +150,7 @@ static void send_ERG(gp_file *out, pcl_Level level)
 
 int pcl3_init_file(gs_memory_t *mem, gp_file *out, pcl_FileData *data)
 {
-  pcl_bool needs_CRD = (data->level == pcl_level_3plus_CRD_only);
+  pcl_bool needs_CRD = (data && data->level == pcl_level_3plus_CRD_only);
     /* Do we need Configure Raster Data? */
   int j;
   const pcl_ColorantState *colorant = NULL;
@@ -469,7 +469,7 @@ int pcl3_init_file(gs_memory_t *mem, gp_file *out, pcl_FileData *data)
     gp_fprintf(out, "\033*r%dU",
         /* PCL: Set Number of Planes per Row/Simple Color */
       data->palette == pcl_RGB? 3:	/* RGB palette */
-      -data->number_of_colorants);	/* (K)(CMY) palette */
+      -(int)data->number_of_colorants);	/* (K)(CMY) palette */
 
   /* Configure Raster Data */
   if (needs_CRD) {
@@ -548,6 +548,7 @@ int pcl3_begin_raster(gp_file *out, pcl_RasterData *data)
 {
   const pcl_FileData *global = NULL;
   int j;
+  size_t seed_size = 0;
 
   /* Check 'data' for validity */
   {
@@ -584,14 +585,14 @@ int pcl3_begin_raster(gp_file *out, pcl_RasterData *data)
   }
 
   /* Allocate the seed plane array */
-  data->seed_plane = (pcl_OctetString **)
-    malloc(global->number_of_bitplanes*sizeof(pcl_OctetString *));
+  seed_size = (size_t)global->number_of_bitplanes*sizeof(pcl_OctetString *);
+  data->seed_plane = (pcl_OctetString **)malloc(seed_size);
   if (data->seed_plane == NULL) {
     errprintf(out->memory, ERRPREF "Memory allocation failure in pcl3_begin_raster().\n");
     return -1;
   }
   memset(data->seed_plane, 0,
-    global->number_of_bitplanes*sizeof(pcl_OctetString *));
+    seed_size);
 
   /* Use the seed plane array for differential compression methods */
   if (pcl_cm_is_differential(global->compression)) {
