@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -43,12 +43,14 @@ static dev_proc_put_params(bittag_put_params);
 static dev_proc_create_buf_device(bittag_create_buf_device);
 static dev_proc_fillpage(bittag_fillpage);
 static dev_proc_map_rgb_color(bit_mono_map_color);
+static dev_proc_decode_color(bit_mono_decode_color);
 #if 0 /* unused */
 static dev_proc_map_rgb_color(bit_forcemono_map_rgb_color);
 #endif
 static dev_proc_map_rgb_color(bitrgb_rgb_map_rgb_color);
 static dev_proc_map_color_rgb(bit_map_color_rgb);
 static dev_proc_map_cmyk_color(bit_map_cmyk_color);
+static dev_proc_decode_color(bit_map_color_cmyk);
 static dev_proc_get_params(bit_get_params);
 static dev_proc_put_params(bit_put_params);
 static dev_proc_print_page(bit_print_page);
@@ -57,74 +59,15 @@ static dev_proc_put_image(bit_put_image);
 static dev_proc_dev_spec_op(bit_dev_spec_op);
 dev_proc_get_color_comp_index(gx_default_DevRGB_get_color_comp_index);
 
-#define bit_procs(encode_color)\
-{	gdev_prn_open,\
-        gx_default_get_initial_matrix,\
-        NULL,	/* sync_output */\
-        /* Since the print_page doesn't alter the device, this device can print in the background */\
-        gdev_prn_bg_output_page,\
-        gdev_prn_close,\
-        encode_color,	/* map_rgb_color */\
-        bit_map_color_rgb,	/* map_color_rgb */\
-        NULL,	/* fill_rectangle */\
-        NULL,	/* tile_rectangle */\
-        NULL,	/* copy_mono */\
-        NULL,	/* copy_color */\
-        NULL,	/* draw_line */\
-        NULL,	/* get_bits */\
-        bit_get_params,\
-        bit_put_params,\
-        encode_color,	/* map_cmyk_color */\
-        NULL,	/* get_xfont_procs */\
-        NULL,	/* get_xfont_device */\
-        NULL,	/* map_rgb_alpha_color */\
-        gx_page_device_get_page_device,	/* get_page_device */\
-        NULL,	/* get_alpha_bits */\
-        NULL,	/* copy_alpha */\
-        NULL,	/* get_band */\
-        NULL,	/* copy_rop */\
-        NULL,	/* fill_path */\
-        NULL,	/* stroke_path */\
-        NULL,	/* fill_mask */\
-        NULL,	/* fill_trapezoid */\
-        NULL,	/* fill_parallelogram */\
-        NULL,	/* fill_triangle */\
-        NULL,	/* draw_thin_line */\
-        NULL,	/* begin_image */\
-        NULL,	/* image_data */\
-        NULL,	/* end_image */\
-        NULL,	/* strip_tile_rectangle */\
-        NULL,	/* strip_copy_rop */\
-        NULL,	/* get_clipping_box */\
-        NULL,	/* begin_typed_image */\
-        NULL,	/* get_bits_rectangle */\
-        NULL,	/* map_color_rgb_alpha */\
-        NULL,	/* create_compositor */\
-        NULL,	/* get_hardware_params */\
-        NULL,	/* text_begin */\
-        NULL,	/* finish_copydevice */\
-        NULL,	/* begin_transparency_group */\
-        NULL,	/* end_transparency_group */\
-        NULL,	/* begin_transparency_mask */\
-        NULL,	/* end_transparency_mask */\
-        NULL,	/* discard_transparency_layer */\
-        NULL,	/* get_color_mapping_procs */\
-        NULL,	/* get_color_comp_index */\
-        encode_color,		/* encode_color */\
-        bit_map_color_rgb,	/* decode_color */\
-        NULL,   /* pattern_manage */\
-        NULL,   /* fill_rectangle_hl_color */\
-        NULL,   /* include_color_space */\
-        NULL,   /* fill_linear_color_scanline */\
-        NULL,   /* fill_linear_color_trapezoid */\
-        NULL,   /* fill_linear_color_triangle */\
-        NULL,   /* update_spot_equivalent_colors */\
-        NULL,   /* ret_devn_params */\
-        NULL,   /* fillpage */\
-        NULL,   /* push_transparency_state */\
-        NULL,   /* pop_transparency_state */\
-        NULL,   /* put_image */\
-        bit_dev_spec_op\
+static void
+bit_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_bg(dev);
+
+    set_dev_proc(dev, map_color_rgb, bit_map_color_rgb);
+    set_dev_proc(dev, get_params, bit_get_params);
+    set_dev_proc(dev, put_params, bit_put_params);
+    set_dev_proc(dev, dev_spec_op, bit_dev_spec_op);
 }
 
 /*
@@ -144,110 +87,86 @@ struct gx_device_bit_s {
 };
 typedef struct gx_device_bit_s gx_device_bit;
 
-static const gx_device_procs bitmono_procs =
-bit_procs(bit_mono_map_color);
+static void
+bitmono_initialize_device_procs(gx_device *dev)
+{
+    bit_initialize_device_procs(dev);
+
+    set_dev_proc(dev, map_rgb_color, bit_mono_map_color);
+    set_dev_proc(dev, map_cmyk_color, bit_mono_map_color);
+    set_dev_proc(dev, encode_color, bit_mono_map_color);
+    set_dev_proc(dev, decode_color, bit_mono_decode_color);
+}
+
 const gx_device_bit gs_bit_device =
-{prn_device_body(gx_device_bit, bitmono_procs, "bit",
+{prn_device_body(gx_device_bit, bitmono_initialize_device_procs, "bit",
                  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                  X_DPI, Y_DPI,
                  0, 0, 0, 0,    /* margins */
                  1, 1, 1, 0, 2, 1, bit_print_page)
 };
 
-static const gx_device_procs bitrgb_procs =
-bit_procs(bitrgb_rgb_map_rgb_color);
+static void
+bitrgb_initialize_device_procs(gx_device *dev)
+{
+    bit_initialize_device_procs(dev);
+
+    set_dev_proc(dev, map_rgb_color, bitrgb_rgb_map_rgb_color);
+    set_dev_proc(dev, map_cmyk_color, bitrgb_rgb_map_rgb_color);
+    set_dev_proc(dev, encode_color, bitrgb_rgb_map_rgb_color);
+    set_dev_proc(dev, decode_color, bit_map_color_rgb);
+}
+
 const gx_device_bit gs_bitrgb_device =
-{prn_device_body(gx_device_bit, bitrgb_procs, "bitrgb",
+{prn_device_body(gx_device_bit, bitrgb_initialize_device_procs, "bitrgb",
                  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                  X_DPI, Y_DPI,
                  0, 0, 0, 0,	/* margins */
                  3, 4, 1, 1, 2, 2, bit_print_page)
 };
 
-static const gx_device_procs bitcmyk_procs =
-bit_procs(bit_map_cmyk_color);
+static void
+bitcmyk_initialize_device_procs(gx_device *dev)
+{
+    bit_initialize_device_procs(dev);
+
+    set_dev_proc(dev, map_rgb_color, bit_map_cmyk_color);
+    set_dev_proc(dev, map_cmyk_color, bit_map_cmyk_color);
+    set_dev_proc(dev, encode_color, bit_map_cmyk_color);
+    set_dev_proc(dev, decode_color, bit_map_color_cmyk);
+}
+
 const gx_device_bit gs_bitcmyk_device =
-{prn_device_body(gx_device_bit, bitcmyk_procs, "bitcmyk",
+{prn_device_body(gx_device_bit, bitcmyk_initialize_device_procs, "bitcmyk",
                  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
                  X_DPI, Y_DPI,
                  0, 0, 0, 0,	/* margins */
                  4, 4, 1, 1, 2, 2, bit_print_page)
 };
 
-static const gx_device_procs bitrgbtags_procs =
-    {
-        bittag_open,                        /* open_device */
-        gx_default_get_initial_matrix,        /* initial_matrix */
-        ((void *)0),                        /* sync_output */
-        gdev_prn_output_page,                 /* output page */
-        gdev_prn_close,                       /* close_device */
-        bittag_rgb_map_rgb_color,             /* map rgb color */
-        bittag_map_color_rgb,                 /* map color rgb */
-        ((void *)0),                        /* fill_rectangle */
-        ((void *)0),                        /* tile rectangle */
-        ((void *)0),                        /* copy mono */
-        ((void *)0),                        /* copy color */
-        ((void *)0),                        /* obsolete draw line */
-        ((void *)0),                        /* get_bits */
-        gdev_prn_get_params,                  /* get params */
-        bittag_put_params,                    /* put params */
-        bittag_rgb_map_rgb_color,             /* map_cmyk_color */
-        ((void *)0),                        /* get_xfonts */
-        ((void *)0),                        /* get_xfont_device */
-        ((void *)0),                        /* map_rgb_alpha_color */
-        gx_page_device_get_page_device,       /* get_page_device */
-        ((void *)0),                        /* get_alpha_bits */
-        ((void *)0),                        /* copy_alpha */
-        ((void *)0),                        /* get_band */
-        ((void *)0),                        /* copy_rop */
-        ((void *)0),                       /* fill_path */
-        ((void *)0),                       /* stroke_path */
-        ((void *)0),                       /* fill_mask */
-        ((void *)0),                        /* fill_trapezoid */
-        ((void *)0),                        /* fill_parallelogram */
-        ((void *)0),                        /* fill_triangle */
-        ((void *)0),                        /* draw_thin_line */
-        ((void *)0),                        /* begin_image */
-        ((void *)0),                        /* image_data */
-        ((void *)0),                        /* end_image */
-        ((void *)0),                        /* strip_tile_rectangle */
-        ((void *)0),                        /* strip_copy_rop */
-        ((void *)0),                        /* get_clipping_box */
-        ((void *)0),                        /* begin_typed_image */
-        ((void *)0),                        /* get_bits_rectangle */
-        ((void *)0),                        /* map_color_rgb_alpha */
-        ((void *)0),                       /* create_compositor */
-        ((void *)0),                       /* get_hardware_params */
-        ((void *)0),                       /* text_begin */
-        ((void *)0),                       /* finish_copydevice */
-        ((void *)0),                       /* begin_transparency_group */
-        ((void *)0),                       /* end_transparency_group */
-        ((void *)0),                       /* begin_transparency_mask */
-        ((void *)0),                       /* end_transparency_mask */
-        ((void *)0),                       /* discard_transparency_layer */
-        bittag_get_color_mapping_procs,      /* get_color_mapping_procs */
-        gx_default_DevRGB_get_color_comp_index, /* get_color_comp_index */
-        bittag_rgb_map_rgb_color,           /* encode_color */
-        bittag_map_color_rgb,               /* decode_color */
-        ((void *)0),                        /* pattern_manage */
-        ((void *)0),                        /* fill_rectangle_hl_color */
-        ((void *)0),                        /* include_color_space */
-        ((void *)0),                        /* fill_linear_color_scanline */
-        ((void *)0),                        /* fill_linear_color_trapezoid */
-        ((void *)0),                        /* fill_linear_color_triangle */
-        ((void *)0),                        /* update_spot_equivalent_colors */
-        ((void *)0),                        /* ret_devn_params */
-        bittag_fillpage,                    /* fillpage */
-        ((void *)0),                        /* push_transparency_state */
-        ((void *)0),                        /* pop_transparency_state */
-        bit_put_image,                      /* put_image */
-        bit_dev_spec_op                     /* dev_spec_op */
-    };
+static void
+bitrgbtags_initialize_device_procs(gx_device *dev)
+{
+    bit_initialize_device_procs(dev);
+
+    set_dev_proc(dev, open_device, bittag_open);
+    set_dev_proc(dev, map_rgb_color, bittag_rgb_map_rgb_color);
+    set_dev_proc(dev, map_color_rgb, bittag_map_color_rgb);
+    set_dev_proc(dev, get_params, gdev_prn_get_params);
+    set_dev_proc(dev, put_params, bittag_put_params);
+    set_dev_proc(dev, map_cmyk_color, bittag_rgb_map_rgb_color);
+    set_dev_proc(dev, get_color_mapping_procs, bittag_get_color_mapping_procs);
+    set_dev_proc(dev, get_color_comp_index, gx_default_DevRGB_get_color_comp_index);
+    set_dev_proc(dev, encode_color, bittag_rgb_map_rgb_color);
+    set_dev_proc(dev, decode_color, bittag_map_color_rgb);
+    set_dev_proc(dev, fillpage, bittag_fillpage);
+    set_dev_proc(dev, put_image, bit_put_image);
+}
 
 const gx_device_bit gs_bitrgbtags_device =
     {
         sizeof(gx_device_bit),
-        &bitrgbtags_procs,
+        bitrgbtags_initialize_device_procs,
         "bitrgbtags",
         0,                              /* memory */
         &st_device_printer,
@@ -262,8 +181,8 @@ const gx_device_bit gs_bitrgbtags_device =
         0,                              /* is open */
         0,                              /* max_fill_band */
         {                               /* color infor */
-            3,                          /* max_components */
-            3,                          /* num_components */
+            4,                          /* max_components */
+            4,                          /* num_components */
             GX_CINFO_POLARITY_ADDITIVE, /* polarity */
             32,                         /* depth */
             GX_CINFO_COMP_NO_INDEX,     /* gray index */
@@ -277,7 +196,7 @@ const gx_device_bit gs_bitrgbtags_device =
             { 8, 8, 8, 8 },             /* comp bits */
             { 0xFF0000, 0x00FF00, 0x0000FF, 0xFF000000 },  /* comp mask */
             ( "DeviceRGBT" ),            /* color model name */
-            GX_CINFO_OPMODE_UNKNOWN,     /* overprint mode */
+            GX_CINFO_OPMSUPPORTED_UNKNOWN,     /* overprint mode */
             0,                           /* process comps */
             0                            /* icc_locations */
         },
@@ -289,7 +208,7 @@ const gx_device_bit gs_bitrgbtags_device =
         (int)((float)(110) * (Y_DPI) / 10 + 0.5),/* height */
         0, /* Pad */
         0, /* log2_align_mod */
-        0, /* is_planar */
+        0, /* num_planar_planes */
         0, /* LeadingEdge */
         {
             (float)(((((int)((float)(85) * (X_DPI) / 10 + 0.5)) * 72.0 + 0.5) - 0.5) / (X_DPI)),
@@ -319,6 +238,8 @@ const gx_device_bit gs_bitrgbtags_device =
         0,  /*DisablePageHandler*/
         0,  /*ObjectFilter*/
         0,  /*ObjectHandlerPushed*/
+        0,  /*NupControl*/
+        0,  /*NupHandlerPushed*/
         0,  /*PageCount*/
         0,  /*ShowPageCount*/
         1,  /*NumCopies*/
@@ -332,6 +253,7 @@ const gx_device_bit gs_bitrgbtags_device =
         {false}, /*sgr*/
         0, /*MaxPatternBitmap*/
         0, /*page_uses_transparency*/
+        0, /*page_uses_overprint*/
         {
           MAX_BITMAP, BUFFER_SPACE,
           { BAND_PARAMS_INITIAL_VALUES },
@@ -341,6 +263,7 @@ const gx_device_bit gs_bitrgbtags_device =
         0, /*icc_struct*/
         GS_UNKNOWN_TAG,         /* this device supports tags */
         1,			/* default interpolate_control */
+        0,                      /* default non_srict_bounds */
         {
             gx_default_install,
             gx_default_begin_page,
@@ -365,20 +288,20 @@ const gx_device_bit gs_bitrgbtags_device =
         false, /* file_is_new */
         NULL,  /* file */
         false, /* bg_print_requested */
-        {0},   /* bg_print */
+        0,     /* bg_print *  */
         0,     /* num_render_threads_requested */
         NULL,  /* saved_pages_list */
         {0}    /* save_procs_while_delaying_erasepage */
     };
 
 static void
-cmyk_cs_to_rgb_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
+cmyk_cs_to_rgb_cm(const gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
 {
     color_cmyk_to_rgb(c, m, y, k, NULL, out, dev->memory);
 }
 
 static void
-private_rgb_cs_to_rgb_cm(gx_device * dev, const gs_gstate *pgs,
+private_rgb_cs_to_rgb_cm(const gx_device * dev, const gs_gstate *pgs,
                                   frac r, frac g, frac b, frac out[])
 {
     out[0] = r;
@@ -387,7 +310,7 @@ private_rgb_cs_to_rgb_cm(gx_device * dev, const gs_gstate *pgs,
 }
 
 static void
-gray_cs_to_rgb_cm(gx_device * dev, frac gray, frac out[])
+gray_cs_to_rgb_cm(const gx_device * dev, frac gray, frac out[])
 {
     out[0] = out[1] = out[2] = gray;
 }
@@ -397,8 +320,9 @@ static const gx_cm_color_map_procs bittag_DeviceRGB_procs = {
 };
 
 static const gx_cm_color_map_procs *
-bittag_get_color_mapping_procs(const gx_device *dev)
+bittag_get_color_mapping_procs(const gx_device *dev, const gx_device **map_dev)
 {
+    *map_dev = dev;
     return &bittag_DeviceRGB_procs;
 }
 
@@ -413,7 +337,7 @@ bittag_rgb_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 }
 
 static int
-bittag_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value cv[4])
+bittag_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value cv[3])
 {
     int depth = 24;
     int ncomp = 3;
@@ -444,6 +368,19 @@ bit_mono_map_color(gx_device * dev, const gx_color_value cv[])
     return (bpc == 1 ? gx_max_color_value - gray : gray) >> drop;
 }
 
+static int
+bit_mono_decode_color(gx_device * dev, gx_color_index color, gx_color_value *cv)
+{
+    int bpc = dev->color_info.depth;
+    int max = (1<<bpc)-1;
+
+    if (bpc == 1)
+        cv[0] = -((gx_color_value)color ^ 1);
+    else
+        cv[0] = color * gx_max_color_value / max;
+    return 0;
+}
+
 #if 0 /* unused */
 /* Map RGB to gray shade. */
 /* Only used in CMYK mode when put_params has set ForceMono=1 */
@@ -490,7 +427,7 @@ bitrgb_rgb_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 /* Map color to RGB.  This has 3 separate cases, but since it is rarely */
 /* used, we do a case test rather than providing 3 separate routines. */
 static int
-bit_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value cv[4])
+bit_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value cv[3])
 {
     int depth = dev->color_info.depth;
     int ncomp = REAL_NUM_COMPONENTS(dev);
@@ -553,6 +490,19 @@ bit_map_cmyk_color(gx_device * dev, const gx_color_value cv[])
     return (color == gx_no_color_index ? color ^ 1 : color);
 }
 
+static int
+bit_map_color_cmyk(gx_device * dev, gx_color_index color, gx_color_value *cv)
+{
+    int bpc = dev->color_info.depth / 4;
+    int max = (1<<bpc)-1;
+
+    cv[0] = (color>>(3*bpc)) * gx_max_color_value / max;
+    cv[1] = (color>>(2*bpc)) * gx_max_color_value / max;
+    cv[2] = (color>>(1*bpc)) * gx_max_color_value / max;
+    cv[3] = (color>>(0*bpc)) * gx_max_color_value / max;
+    return 0;
+}
+
 static int
 bittag_open(gx_device * pdev)
 {
@@ -777,7 +727,7 @@ static int
 bit_print_page(gx_device_printer * pdev, gp_file * prn_stream)
 {				/* Just dump the bits on the file. */
     /* If the file is 'nul', don't even do the writes. */
-    int line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
+    size_t line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
     byte *in = gs_alloc_bytes(pdev->memory, line_size, "bit_print_page(in)");
     byte *data;
     int nul = !strcmp(pdev->fname, "nul") || !strcmp(pdev->fname, "/dev/null");
@@ -811,7 +761,7 @@ static int
 bittags_print_page(gx_device_printer * pdev, gp_file * prn_stream)
 {				/* Just dump the bits on the file. */
     /* If the file is 'nul', don't even do the writes. */
-    int line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
+    size_t line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
     byte *in = gs_alloc_bytes(pdev->memory, line_size, "bit_print_page(in)");
     byte *data;
     int nul = !strcmp(pdev->fname, "nul") || !strcmp(pdev->fname, "/dev/null");
@@ -833,6 +783,22 @@ bittags_print_page(gx_device_printer * pdev, gp_file * prn_stream)
     for (i = 0; i <= line_count; i++, lnum += step) {
         if ((code = gdev_prn_get_bits(pdev, lnum, in, &data)) < 0)
             goto done;
+        {
+            /* The data is written internally as TTRRGGBB, where we want it written as: RRGGBBTT.
+             * Post process that here. */
+            int ls = line_size;
+            byte *d = data;
+            while (ls > 0)
+            {
+                byte t = d[0];
+                d[0] = d[1];
+                d[1] = d[2];
+                d[2] = d[3];
+                d[3] = t;
+                d += 4;
+                ls -= 4;
+            }
+        }
         if (!nul)
             gp_fwrite(data, 1, line_size, prn_stream);
     }
