@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Epson color dot-matrix printer driver by dave@exlog.com */
@@ -143,12 +143,20 @@ epson_map_color_rgb(gx_device * dev, gx_color_index color,
 static dev_proc_print_page(epsc_print_page);
 
 /* Since the print_page doesn't alter the device, this device can print in the background */
-static gx_device_procs epson_procs =
-prn_color_procs(gdev_prn_open, gdev_prn_bg_output_page, gdev_prn_close,
-                epson_map_rgb_color, epson_map_color_rgb);
+static void
+epson_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_bg(dev);
+
+    set_dev_proc(dev, map_rgb_color, epson_map_rgb_color);
+    set_dev_proc(dev, map_color_rgb, epson_map_color_rgb);
+    set_dev_proc(dev, encode_color, epson_map_rgb_color);
+    set_dev_proc(dev, decode_color, epson_map_color_rgb);
+}
 
 const gx_device_printer far_data gs_epsonc_device =
-prn_device(epson_procs, "epsonc",
+prn_device(epson_initialize_device_procs,
+           "epsonc",
            DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
            X_DPI, Y_DPI,
            0, 0, 0.25, 0,       /* margins */
@@ -173,8 +181,8 @@ epsc_print_page(gx_device_printer * pdev, gp_file * prn_stream)
     int y_24pin = pdev->y_pixels_per_inch > 72;
     int y_mult = (y_24pin ? 3 : 1);
     int line_size = (pdev->width + 7) >> 3;     /* always mono */
-    int in_size = line_size * (8 * y_mult);
-    int out_size = ((pdev->width + 7) & -8) * y_mult;
+    size_t in_size = line_size * (8 * y_mult);
+    size_t out_size = ((pdev->width + 7) & -8) * y_mult;
     byte *in;
     byte *out;
     int x_dpi = (int)pdev->x_pixels_per_inch;
@@ -182,12 +190,12 @@ epsc_print_page(gx_device_printer * pdev, gp_file * prn_stream)
     char start_graphics;
     int first_pass;
     int last_pass;
-    int dots_per_space; 
+    int dots_per_space;
     int bytes_per_space;
     int skip = 0, lnum = 0, code = 0, pass;
 
     byte *color_in;
-    int color_line_size, color_in_size;
+    size_t color_line_size, color_in_size;
     int spare_bits;
     int whole_bits;
 
@@ -201,7 +209,7 @@ epsc_print_page(gx_device_printer * pdev, gp_file * prn_stream)
     if (x_dpi > max_dpi) {
         return_error(gs_error_rangecheck);
     }
-    
+
     start_graphics = (char)
         ((y_24pin ? graphics_modes_24 : graphics_modes_9)[x_dpi / 60]);
     first_pass = (start_graphics & DD ? 1 : 0);
