@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -67,7 +67,7 @@ pdf_find_named(gx_device_pdf * pdev, const gs_param_string * pname,
  */
 int
 pdf_create_named(gx_device_pdf *pdev, const gs_param_string *pname,
-                 cos_type_t cotype, cos_object_t **ppco, long id)
+                 cos_type_t cotype, cos_object_t **ppco, int64_t id)
 {
     cos_object_t *pco;
     cos_value_t value;
@@ -91,7 +91,7 @@ pdf_create_named(gx_device_pdf *pdev, const gs_param_string *pname,
 }
 int
 pdf_create_named_dict(gx_device_pdf *pdev, const gs_param_string *pname,
-                      cos_dict_t **ppcd, long id)
+                      cos_dict_t **ppcd, int64_t id)
 {
     cos_object_t *pco;
     int code = pdf_create_named(pdev, pname, cos_type_dict, &pco, id);
@@ -470,7 +470,7 @@ pdf_replace_names(gx_device_pdf * pdev, const gs_param_string * from,
 
         size += sname - scan;
         if (pco) {
-            gs_snprintf(ref, sizeof(ref), " %ld 0 R ", pco->id);
+            gs_snprintf(ref, sizeof(ref), " %"PRId64" 0 R ", pco->id);
             size += strlen(ref);
             /* Special 'name' escaping convention (see gs_pdfwr.ps, /.pdf===dict
              * the /nametype procedure). We do not want to write out the NULL
@@ -509,7 +509,7 @@ pdf_replace_names(gx_device_pdf * pdev, const gs_param_string * from,
         memcpy(sto, scan, copy);
         sto += copy;
         if (pco) {
-            gs_snprintf(ref, sizeof(ref), " %ld 0 R ", pco->id);
+            gs_snprintf(ref, sizeof(ref), " %"PRId64" 0 R ", pco->id);
             rlen = strlen(ref);
             if (sname >= (start + 2) && sname[-1] == 0x00 && sname[-2] == 0x00 && next[0] == 0x00) {
                 sto -= 2;
