@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -57,13 +57,13 @@ pdf_make_soft_mask_dict(gx_device_pdf * pdev, const gs_pdf14trans_params_t * ppa
             return code;
     }
     if (pdev->CompatibilityLevel <= 1.7 && pparams->transfer_function != NULL && pdev->params.TransferFunctionInfo == tfi_Preserve) {
-        long id;
+        int64_t id;
         char buf[20];
 
         code = pdf_write_function(pdev, pparams->transfer_function, &id);
         if (code < 0)
             return code;
-        gs_sprintf(buf, " %ld 0 R", id);
+        gs_snprintf(buf, sizeof(buf), " %"PRId64" 0 R", id);
         code = cos_dict_put_c_key_string(soft_mask_dict, "/TR", (const byte *)buf, strlen(buf));
         if (code < 0)
             return code;
@@ -107,11 +107,15 @@ pdf_make_group_dict(gx_device_pdf * pdev, const gs_pdf14trans_params_t * pparams
        a group color specified.
        In this case, the parent group is inherited from
        the previous group or the device color space */
-    if (pgs != NULL && pparams->group_color_type != UNKNOWN) {
-        const gs_color_space *cs = gs_currentcolorspace_inline(pgs);
+    if (pgs != NULL && pparams->ColorSpace != NULL) {
+        const gs_color_space *cs = pparams->ColorSpace;
 
-        code = pdf_color_space_named(pdev, pgs, &cs_value, NULL, cs,
-                &pdf_color_space_names, false, NULL, 0, false);
+        if (pparams->ColorSpace == NULL)
+            code = pdf_color_space_named(pdev, pgs, &cs_value, NULL, cs,
+                    &pdf_color_space_names, false, NULL, 0, false);
+        else
+            code = pdf_color_space_named(pdev, pgs, &cs_value, NULL, pparams->ColorSpace,
+                    &pdf_color_space_names, false, NULL, 0, false);
         if (code < 0)
             return code;
         code = cos_dict_put_c_key(group_dict, "/CS", &cs_value);
@@ -162,6 +166,56 @@ pdf_make_form_dict(gx_device_pdf * pdev, const gs_pdf14trans_params_t * pparams,
     code = cos_dict_put_c_key_object(form_dict, "/BBox", (cos_object_t *)bbox_array);
     if (code < 0)
         return code;
+    if (pdev->PendingOC != 0) {
+        if (pdev->CompatibilityLevel < 1.4999) {
+            if (pdev->PDFA) {
+                switch (pdev->PDFACompatibilityPolicy) {
+                    case 0:
+                        emprintf(pdev->memory,
+                                 "Optional Content not valid in this version of PDF, reverting to normal PDF output\n");
+                        pdev->AbortPDFAX = true;
+                        pdev->PDFA = 0;
+                        break;
+                    case 1:
+                        emprintf(pdev->memory,
+                                 "Optional Content not valid in this version of PDF. Dropping feature to preserve PDF/A compatibility\n");
+                        break;
+                    case 2:
+                        emprintf(pdev->memory,
+                                 "Optional Content not valid in this version of PDF,  aborting conversion\n");
+                        return_error (gs_error_typecheck);
+                        break;
+                    default:
+                        emprintf(pdev->memory,
+                                 "Optional Content not valid in this version of PDF, unrecognised PDFACompatibilityLevel,\nreverting to normal PDF output\n");
+                        pdev->AbortPDFAX = true;
+                        pdev->PDFA = 0;
+                        break;
+                }
+            } else {
+                emprintf(pdev->memory,
+                         "Optional Content not valid in this version of PDF. Dropping feature to preserve compatibility\n");
+            }
+        } else {
+            char str[256];
+            gs_param_string param;
+            cos_object_t *pco = NULL;
+
+            param.data = (const byte *)pdev->PendingOC;
+            param.size = strlen(pdev->PendingOC);
+            code = pdf_refer_named(pdev, &param, &pco);
+            if(code < 0)
+                return code;
+
+            gs_snprintf(str, sizeof(str), "%"PRId64" 0 R", pco->id);
+            code = cos_dict_put_string_copy(form_dict, "/OC", str);
+            if (code < 0)
+                return code;
+
+            gs_free_object(pdev->memory->non_gc_memory, pdev->PendingOC, "");
+            pdev->PendingOC = NULL;
+        }
+    }
     return cos_dict_put_c_key_object(form_dict, "/Group", (cos_object_t *)group_dict);
 }
 
@@ -251,6 +305,8 @@ pdf_end_transparency_group(gs_gstate * pgs, gx_device_pdf * pdev)
         int code;
         uint ignore;
 
+        if (pres == NULL)
+            return_error(gs_error_unregistered);
         pdev->FormDepth--;
         pdev->PatternsSinceForm = 0;
         code = pdf_exit_substream(pdev);
@@ -334,6 +390,7 @@ pdf_begin_transparency_mask(gs_gstate * pgs, gx_device_pdf * pdev,
     } else {
         int code;
 
+        pdev->smask_construction = true;
         code = pdf_make_soft_mask_dict(pdev, pparams);
         if (code < 0)
             return code;
@@ -348,6 +405,7 @@ static int
 pdf_end_transparency_mask(gs_gstate * pgs, gx_device_pdf * pdev,
                                 const gs_pdf14trans_params_t * pparams)
 {
+    pdev->smask_construction = false;
     if (pdev->image_mask_skip)
         pdev->image_mask_skip = false;
     else {
@@ -355,6 +413,8 @@ pdf_end_transparency_mask(gs_gstate * pgs, gx_device_pdf * pdev,
         int code;
         char buf[20];
 
+        if (pres == NULL)
+            return_error(gs_error_unregistered);
         code = pdf_exit_substream(pdev);
         if (code < 0)
             return code;
@@ -363,7 +423,7 @@ pdf_end_transparency_mask(gs_gstate * pgs, gx_device_pdf * pdev,
             return 0;
         /* We need to update the 'where_used' field, in case we substituted a resource */
         pres->where_used |= pdev->used_mask;
-        gs_sprintf(buf, "%ld 0 R", pdf_resource_id(pres));
+        gs_snprintf(buf, sizeof(buf), "%"PRId64" 0 R", pdf_resource_id(pres));
         if (pdev->pres_soft_mask_dict == 0L) {
             /* something went horribly wrong, we have an 'end' wihtout a matching 'begin'
              * Give up, throw an error.
@@ -399,7 +459,7 @@ pdf_set_blend_params(gs_gstate * pgs, gx_device_pdf * dev,
 }
 
 int
-gdev_pdf_create_compositor(gx_device *dev,
+gdev_pdf_composite(gx_device *dev,
     gx_device **pcdev, const gs_composite_t *pct,
     gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev)
 {
@@ -449,7 +509,7 @@ gdev_pdf_create_compositor(gx_device *dev,
         }
         return 0;
     }
-    return psdf_create_compositor(dev, pcdev, pct, pgs, memory, cdev);
+    return psdf_composite(dev, pcdev, pct, pgs, memory, cdev);
 }
 
 /* We're not sure why the folllowing device methods are never called.
