@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -101,12 +101,13 @@ typedef struct psdf_distiller_params_s {
         "AbsoluteColorimetric"
     bool DetectBlends;
     bool DoThumbnails;
-    long ImageMemory;
+    int64_t ImageMemory;
     bool LockDistillerParams;
     bool LZWEncodePages;
     int OPM;
     bool PreserveOPIComments;
     bool UseFlateCompression;
+    bool UseBrotliCompression;
 
     /* Color processing parameters */
 
@@ -159,6 +160,11 @@ typedef struct psdf_distiller_params_s {
 
     psdf_image_params MonoImage;
 
+    /* Font outlining parameters */
+
+    gs_param_string_array AlwaysOutline;
+    gs_param_string_array NeverOutline;
+
     /* Font embedding parameters */
 
     gs_param_string_array AlwaysEmbed;
@@ -174,8 +180,10 @@ typedef struct psdf_distiller_params_s {
     int MaxSubsetPct;
     bool SubsetFonts;
     bool PassThroughJPEGImages;
+    bool PassThroughJPXImages;
     gs_param_string PSDocOptions;
     gs_param_string_array PSPageOptions;
+    bool PSPageOptionsWrap;
 } psdf_distiller_params;
 
 /* Declare templates for default image compression filters. */
@@ -197,6 +205,7 @@ extern const stream_template s_zlibE_template;
     0,		    /* Overprintmode (OPM) */ \
     0,		    /* PreserveOPIComments (false) */ \
     1,		    /* UseFlateCompression (true) */ \
+    0,		    /* UseBrotliCompression (false) */ \
         /* Color processing parameters */\
     {0},	    /* calCMYKProfile */ \
     {0},	    /* CalGrayProfile */ \
@@ -211,7 +220,7 @@ extern const stream_template s_zlibE_template;
 #define psdf_color_image_param_defaults\
   { NULL,	    /* ACSDict (JPEG) */ \
     0,		    /* AntiAlias (false) */ \
-    0,		    /* AutoFilter (false) */ \
+    1,		    /* AutoFilter (false) */ \
     -1,		    /* Depth */ \
     NULL,	    /* Dict (JPEG or CCITTFax) */ \
     0,		    /* Downsample (false) */ \
@@ -228,7 +237,7 @@ extern const stream_template s_zlibE_template;
 #define psdf_gray_image_param_defaults\
   { NULL,	    /* ACSDict (JPEG) */ \
     0,		    /* AntiAlias (false) */ \
-    0,		    /* AutoFilter (false) */ \
+    1,		    /* AutoFilter (false) */ \
     -1,		    /* Depth */ \
     NULL,	    /* Dict (JPEG or CCITTFax) */ \
     0,		    /* Downsample (false) */ \
@@ -244,7 +253,7 @@ extern const stream_template s_zlibE_template;
 #define psdf_mono_image_param_defaults\
   { NULL,	    /* ACSDict (JPEG) */ \
     0,		    /* AntiAlias (false) */ \
-    0,		    /* AutoFilter (false) */ \
+    1,		    /* AutoFilter (false) */ \
     -1,		    /* Depth */ \
     NULL,	    /* Dict (JPEG or CCITTFax) */ \
     0,		    /* Downsample (false) */ \
@@ -258,6 +267,8 @@ extern const stream_template s_zlibE_template;
     }
 
 #define psdf_font_param_defaults\
+    {0},	    /* AlwaysOutline (array of names) */ \
+    {0},	    /* NeverOutline (array of names) */ \
     {0},	    /* AlwaysEmbed (array of names) */ \
     {0},	    /* NeverEmbed (array of names) */ \
     cefp_Warning,   /* CannotEmbedFontPolicy */ \
@@ -267,6 +278,8 @@ extern const stream_template s_zlibE_template;
 
 #define psdf_JPEGPassThrough_param_defaults\
     1           /* PassThroughJPEGImages */
+#define psdf_JPXPassThrough_param_defaults\
+    1           /* PassThroughJPXImages */
 
 #define psdf_PSOption_param_defaults\
     {0},        /* PSDocOptions */\
@@ -292,6 +305,7 @@ typedef enum {
         bool HaveCIDSystem;\
         double ParamCompatibilityLevel;\
         bool JPEG_PassThrough;\
+        bool JPX_PassThrough;\
         psdf_distiller_params params
 
 typedef struct gx_device_psdf_s {
@@ -306,6 +320,7 @@ typedef struct gx_device_psdf_s {
         true,\
         false,\
         1.3,\
+        0,\
         0,\
          { psdf_general_param_defaults(ascii),\
            psdf_color_image_param_defaults,\
@@ -313,6 +328,7 @@ typedef struct gx_device_psdf_s {
            psdf_mono_image_param_defaults,\
            psdf_font_param_defaults,\
            psdf_JPEGPassThrough_param_defaults,\
+           psdf_JPXPassThrough_param_defaults,\
            psdf_PSOption_param_defaults\
          }
 /* st_device_psdf is never instantiated per se, but we still need to */
@@ -330,6 +346,8 @@ extern_st(st_device_psdf);
                 params.GrayImage.Dict),\
     GC_OBJ_ELT2(gx_device_psdf, params.MonoImage.ACSDict,\
                 params.MonoImage.Dict),\
+    GC_OBJ_ELT2(gx_device_psdf, params.AlwaysOutline.data,\
+                params.NeverOutline.data),\
     GC_OBJ_ELT2(gx_device_psdf, params.AlwaysEmbed.data,\
                 params.NeverEmbed.data),\
     GC_CONST_STRING_ELT(gx_device_psdf, params.PSDocOptions)\
@@ -508,11 +526,10 @@ int psdf_set_color(gx_device_vector *vdev, const gx_drawing_color *pdc,
 /* Round a double value to a specified precision. */
 double psdf_round(double v, int precision, int radix);
 
-/* stubs to disable get_bits, get_bits_rectangle */
-dev_proc_get_bits(psdf_get_bits);
+/* stub to disable get_bits_rectangle */
 dev_proc_get_bits_rectangle(psdf_get_bits_rectangle);
 
 /* intercept and ignore overprint compositor creation */
-dev_proc_create_compositor(psdf_create_compositor);
+dev_proc_composite(psdf_composite);
 
 #endif /* gdevpsdf_INCLUDED */
