@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* XPS output device */
@@ -28,6 +28,7 @@
 #include "gsicc_create.h"
 #include "gsicc_cache.h"
 #include "gximdecode.h" /* Need so that we can unpack and decode */
+#include "gxpaint.h"
 
 #define MAXPRINTERNAME 64
 
@@ -159,17 +160,16 @@ typedef struct xps_image_enum_s {
     byte *devc_buffer; /* Needed for case where we are mapping to device colors */
     gs_color_space *pcs;     /* Needed for Sep, DeviceN, Indexed */
     gsicc_link_t *icc_link;  /* Needed for CIELAB */
-    const gs_gstate *pgs;    /* Needed for color conversions of DeviceN etc */
     gp_file *fid;
 } xps_image_enum_t;
 
 static void
 xps_image_enum_finalize(const gs_memory_t *cmem, void *vptr);
 
-gs_private_st_suffix_add4_final(st_xps_image_enum, xps_image_enum_t,
+gs_private_st_suffix_add3_final(st_xps_image_enum, xps_image_enum_t,
     "xps_image_enum_t", xps_image_enum_enum_ptrs,
     xps_image_enum_reloc_ptrs, xps_image_enum_finalize, st_vector_image_enum,
-    buffer, devc_buffer, pcs, pgs);
+    buffer, devc_buffer, pcs);
 
 typedef struct gx_device_xps_s {
     /* superclass state */
@@ -192,6 +192,10 @@ typedef struct gx_device_xps_s {
     double miterlimit;
     bool can_stroke;
     unsigned char PrinterName[MAXPRINTERNAME];
+    bool in_path;
+    bool in_clip;
+    bool clip_written;
+    bool rect_written;
 } gx_device_xps;
 
 gs_public_st_suffix_add1_final(st_device_xps, gx_device_xps,
@@ -267,7 +271,7 @@ static int
 xps_setlogop(gx_device_vector *vdev, gs_logical_operation_t lop,
              gs_logical_operation_t diff);
 
-static int
+static bool
 xps_can_handle_hl_color(gx_device_vector *vdev, const gs_gstate *pgs,
                         const gx_drawing_color * pdc);
 static int
@@ -913,6 +917,10 @@ xps_open_device(gx_device *dev)
     xps->linejoin = XPS_DEFAULT_LINEJOIN;
     xps->miterlimit = XPS_DEFAULT_MITERLIMIT;
     xps->can_stroke = true;
+    xps->in_path = false;
+    xps->in_clip = false;
+    xps->clip_written = false;
+    xps->rect_written = false;
     /* zip info */
     xps->f2i = NULL;
     xps->f2i_tail = NULL;
@@ -1074,6 +1082,10 @@ xps_output_page(gx_device *dev, int num_copies, int flush)
     gx_device_vector *vdev = (gx_device_vector *)dev;
     int code;
 
+    if (!vdev->in_page) {
+        (*vdev_proc(vdev, beginpage)) (vdev);
+        vdev->in_page = true;
+    }
     write_str_to_current_page(xps, "</Canvas></FixedPage>");
 
     if (xps->relations_head)
@@ -1468,12 +1480,12 @@ xps_setlogop(gx_device_vector *vdev, gs_logical_operation_t lop,
 
         /* Other state */
 
-static int
+static bool
 xps_can_handle_hl_color(gx_device_vector *vdev, const gs_gstate *pgs,
                           const gx_drawing_color *pdc)
 {
     if_debug0m('_', vdev->memory, "xps_can_handle_hl_color\n");
-    return 0;
+    return false;
 }
 
 /* Paths */
@@ -1487,7 +1499,7 @@ static bool
 drawing_path(gx_path_type_t path_type, xps_brush_t brush_type)
 {
     return ((path_type & gx_path_type_stroke) || (path_type & gx_path_type_fill) ||
-        image_brush_fill(path_type, brush_type));
+        (path_type & gx_path_type_clip) || image_brush_fill(path_type, brush_type));
 }
 
 static void
@@ -1552,6 +1564,35 @@ xps_dorect(gx_device_vector *vdev, fixed x0, fixed y0,
         return 0;
     }
 
+    /* CLIP - we only write clips as an attribute of a path */
+    if (type & gx_path_type_clip && !image_brush_fill(type, xps->filltype)) {
+        if (xps->in_path == false)
+            return 0;
+        fmt = "Clip=\"M %g,%g V %g H %g V %g Z\" ";
+        gs_snprintf(line, sizeof(line), fmt,
+                   fixed2float(x0), fixed2float(y0),
+                   fixed2float(y1), fixed2float(x1),
+                   fixed2float(y0));
+        write_str_to_current_page(xps, line);
+        xps->clip_written = true;
+        return 0;
+    }
+
+    if (xps->in_path && image_brush_fill(type, xps->filltype)) {
+        write_str_to_current_page(xps, "/>\n");
+        xps->in_path = false;
+        xps->in_clip = false;
+        xps->clip_written = false;
+    }
+
+    if (xps->in_path && xps->rect_written) {
+        write_str_to_current_page(xps, "/>\n");
+        xps->in_path = false;
+        xps->in_clip = false;
+        xps->clip_written = false;
+        xps->rect_written = false;
+    }
+
     if ((type & gx_path_type_stroke) && !xps->can_stroke) {
         return_error(gs_error_rangecheck);
     }
@@ -1569,7 +1610,8 @@ xps_dorect(gx_device_vector *vdev, fixed x0, fixed y0,
         xps_finish_image_path(vdev);
     } else if (type & gx_path_type_fill) {
         /* Solid fill */
-        write_str_to_current_page(xps, "<Path ");
+        if (!xps->in_path)
+            write_str_to_current_page(xps, "<Path ");
         /* NB - F0 should be changed for a different winding type */
         fmt = "Fill=\"#%06X\" Data=\"M %g,%g V %g H %g V %g Z\" ";
         c = xps->fillcolor & 0xffffffL;
@@ -1578,10 +1620,14 @@ xps_dorect(gx_device_vector *vdev, fixed x0, fixed y0,
                    fixed2float(y1), fixed2float(x1),
                    fixed2float(y0));
         write_str_to_current_page(xps, line);
-        write_str_to_current_page(xps, "/>\n");
+        if (!xps->in_path)
+            write_str_to_current_page(xps, "/>\n");
+        else
+            xps->rect_written = true;
     } else {
         /* Solid stroke */
-        write_str_to_current_page(xps, "<Path ");
+        if (!xps->in_path)
+            write_str_to_current_page(xps, "<Path ");
         fmt = "Stroke=\"#%06X\" Data=\"M %g,%g V %g H %g V %g Z\" ";
         c = xps->strokecolor & 0xffffffL;
         gs_snprintf(line, sizeof(line), fmt, c,
@@ -1596,7 +1642,10 @@ xps_dorect(gx_device_vector *vdev, fixed x0, fixed y0,
             gs_snprintf(line, sizeof(line), fmt, xps->linewidth);
             write_str_to_current_page(xps, line);
         }
-        write_str_to_current_page(xps, "/>\n");
+        if (!xps->in_path)
+            write_str_to_current_page(xps, "/>\n");
+        else
+            xps->rect_written = true;
     }
     /* end and close NB \n not necessary. */
     return 0;
@@ -1607,10 +1656,31 @@ gdev_xps_fill_path(gx_device * dev, const gs_gstate * pgs, gx_path * ppath,
                    const gx_fill_params * params,
                    const gx_drawing_color * pdcolor, const gx_clip_path * pcpath)
 {
+    gx_device_xps *xps = (gx_device_xps *)dev;
+    int code = 0;
+
     if (gx_path_is_void(ppath)) {
         return 0;
     }
-    return gdev_vector_fill_path(dev, pgs, ppath, params, pdcolor, pcpath);
+
+    (void)gdev_vector_stream((gx_device_vector*)xps);
+
+    if (xps->in_path) {
+        write_str_to_current_page(xps, "/>\n");
+        xps->in_clip = false;
+    }
+
+    xps->clip_path_id = xps->no_clip_path_id;
+    write_str_to_current_page(xps, "<Path ");
+    xps->in_path = true;
+    code = gdev_vector_fill_path(dev, pgs, ppath, params, pdcolor, pcpath);
+    if (xps->in_path) {
+        write_str_to_current_page(xps, "/>\n");
+        xps->in_path = false;
+    }
+    xps->clip_written = false;
+    xps->rect_written = false;
+    return code;
 }
 
 static int
@@ -1618,10 +1688,32 @@ gdev_xps_stroke_path(gx_device * dev, const gs_gstate * pgs, gx_path * ppath,
                      const gx_stroke_params * params,
                      const gx_drawing_color * pdcolor, const gx_clip_path * pcpath)
 {
+    gx_device_xps *xps = (gx_device_xps *)dev;
+    int code = 0;
+
     if (gx_path_is_void(ppath)) {
         return 0;
     }
-    return gdev_vector_stroke_path(dev, pgs, ppath, params, pdcolor, pcpath);
+
+    (void)gdev_vector_stream((gx_device_vector*)xps);
+
+    if (xps->in_path) {
+        write_str_to_current_page(xps, "/>\n");
+        xps->in_clip = false;
+    }
+
+    xps->clip_path_id = xps->no_clip_path_id;
+    write_str_to_current_page(xps, "<Path ");
+    xps->in_path = true;
+    code = gdev_vector_stroke_path(dev, pgs, ppath, params, pdcolor, pcpath);
+    if (xps->in_path) {
+        write_str_to_current_page(xps, "/>\n");
+        xps->in_path = false;
+
+    }
+    xps->clip_written = false;
+    xps->rect_written = false;
+    return code;
 }
 
 static int
@@ -1640,26 +1732,44 @@ xps_beginpath(gx_device_vector *vdev, gx_path_type_t type)
         return 0;
     }
 
-    if (!xps->can_stroke) {
+    if ((type & gx_path_type_stroke) && !xps->can_stroke) {
         return_error(gs_error_rangecheck);
     }
 
     c = type & gx_path_type_fill ? xps->fillcolor : xps->strokecolor;
     c &= 0xffffffL;
 
+    /* NOTE the XPS code (ab)uses the clip to write the rectangle for the image!
+    */
+    if (type & gx_path_type_clip && !image_brush_fill(type, xps->filltype)) {
+        if (xps->in_path == true && xps->clip_written == false) {
+            write_str_to_current_page(xps, " Clip=\"");
+            xps->in_clip = true;
+        }
+        goto exit;
+    }
+
     if (!image_brush_fill(type, xps->filltype)) {
-        write_str_to_current_page(xps, "<Path ");
-        if (type & gx_path_type_fill)
-            fmt = "Fill=\"#%06X\" Data=\"";
-        else
+        if (type & gx_path_type_fill) {
+            if (type == gx_path_type_fill)
+                fmt = "Fill=\"#%06X\" Data=\"F 1";
+            else
+                fmt = "Fill=\"#%06X\" Data=\"";
+        }
+        else {
             fmt = "Stroke=\"#%06X\" Data=\"";
+        }
         gs_snprintf(line, sizeof(line), fmt, c);
         write_str_to_current_page(xps, line);
     }
     else {
-        write_str_to_current_page(xps, "<Path Data=\"");
+        if ( image_brush_fill(type, xps->filltype))
+            write_str_to_current_page(xps, "<Path Data=\"");
+        else
+            write_str_to_current_page(xps, " Data=\"");
     }
 
+exit:
     if_debug1m('_', xps->memory, "xps_beginpath %s\n", line);
 
     return 0;
@@ -1676,10 +1786,18 @@ xps_moveto(gx_device_vector *vdev, double x0, double y0,
 
     /* skip non-drawing paths for now */
     if (!drawing_path(type, xps->filltype)) {
-        if_debug1m('_', xps->memory, "xps_moveto: type not supported %x\n", type);
-        return 0;
+        if (type == gx_path_type_none) {
+            if (xps->in_path == false) {
+                if_debug1m('_', xps->memory, "xps_moveto: type not supported %x\n", type);
+                return 0;
+            }
+        }
     }
 
+    if ((type & gx_path_type_clip || type == gx_path_type_none) && !image_brush_fill(type, xps->filltype)) {
+        if (xps->in_path == false || xps->clip_written)
+            return 0;
+    }
     gs_snprintf(line, sizeof(line), " M %g,%g", x, y);
     write_str_to_current_page(xps, line);
     if_debug1m('_', xps->memory, "xps_moveto %s", line);
@@ -1697,9 +1815,19 @@ xps_lineto(gx_device_vector *vdev, double x0, double y0,
 
     /* skip non-drawing paths for now */
     if (!drawing_path(type, xps->filltype)) {
-        if_debug1m('_', xps->memory, "xps_lineto: type not supported %x\n", type);
-        return 0;
+        if (type == gx_path_type_none) {
+            if (xps->in_path == false) {
+                if_debug1m('_', xps->memory, "xps_lineto: type not supported %x\n", type);
+                return 0;
+            }
+        }
     }
+
+    if ((type & gx_path_type_clip || type == gx_path_type_none) && !image_brush_fill(type, xps->filltype)) {
+        if (xps->in_path == false || xps->clip_written)
+            return 0;
+    }
+
     gs_snprintf(line, sizeof(line), " L %g,%g", x, y);
     write_str_to_current_page(xps, line);
     if_debug1m('_', xps->memory, "xps_lineto %s\n", line);
@@ -1716,8 +1844,17 @@ xps_curveto(gx_device_vector *vdev, double x0, double y0,
 
     /* skip non-drawing paths for now */
     if (!drawing_path(type, xps->filltype)) {
-        if_debug1m('_', xps->memory, "xps_curveto: type not supported %x\n", type);
-        return 0;
+        if (type == gx_path_type_none) {
+            if (xps->in_path == false) {
+                if_debug1m('_', xps->memory, "xps_curveto: type not supported %x\n", type);
+                return 0;
+            }
+        }
+    }
+
+    if ((type & gx_path_type_clip || type == gx_path_type_none) && !image_brush_fill(type, xps->filltype)) {
+        if (xps->in_path == false || xps->clip_written)
+            return 0;
     }
 
     gs_snprintf(line, sizeof(line), " C %g,%g %g,%g %g,%g", x1, y1,
@@ -1736,8 +1873,17 @@ xps_closepath(gx_device_vector *vdev, double x, double y,
 
     /* skip non-drawing paths for now */
     if (!drawing_path(type, xps->filltype)) {
-        if_debug1m('_', xps->memory, "xps_closepath: type not supported %x\n", type);
-        return 0;
+        if (type == gx_path_type_none) {
+            if (xps->in_path == false) {
+                if_debug1m('_', xps->memory, "xps_closepath: type not supported %x\n", type);
+                return 0;
+            }
+        }
+    }
+
+    if ((type & gx_path_type_clip || type == gx_path_type_none) && !image_brush_fill(type, xps->filltype)) {
+        if (xps->in_path == false || xps->clip_written)
+            return 0;
     }
 
     write_str_to_current_page(xps, " Z");
@@ -1753,10 +1899,18 @@ xps_endpath(gx_device_vector *vdev, gx_path_type_t type)
     char line[200];
     const char *fmt;
 
-    /* skip non-drawing paths for now */
-    if (!drawing_path(type, xps->filltype)) {
-        if_debug1m('_', xps->memory, "xps_endpath: type not supported %x\n", type);
-        return 0;
+    if (xps->in_clip) {
+        xps->in_clip = false;
+        xps->clip_written = true;
+        if (type & gx_path_type_clip) {
+            if (xps->in_path == false)
+                return 0;
+        }
+    } else {
+        if (type & gx_path_type_clip && !image_brush_fill(type, xps->filltype)) {
+            if (xps->in_path == false || xps->clip_written)
+                return 0;
+        }
     }
 
     if (image_brush_fill(type, xps->filltype)) {
@@ -1765,12 +1919,51 @@ xps_endpath(gx_device_vector *vdev, gx_path_type_t type)
         xps_finish_image_path(vdev);
     } else if (type & gx_path_type_stroke) {
         /* NB format width. */
-        fmt = "\" StrokeThickness=\"%g\" />\n";
+        fmt = "\" StrokeThickness=\"%g\" ";
         gs_snprintf(line, sizeof(line), fmt, xps->linewidth);
         write_str_to_current_page(xps, line);
+        switch(xps->linecap) {
+            case gs_cap_round:
+                gs_snprintf(line, sizeof(line), "StrokeStartLineCap=\"Round\" StrokeEndLineCap=\"Round\" ");
+                write_str_to_current_page(xps, line);
+                break;
+            case gs_cap_square:
+                gs_snprintf(line, sizeof(line), "StrokeStartLineCap=\"Square\" StrokeEndLineCap=\"Square\" ");
+                write_str_to_current_page(xps, line);
+                break;
+            case gs_cap_triangle:
+                gs_snprintf(line, sizeof(line), "StrokeStartLineCap=\"Triangle\" StrokeEndLineCap=\"Triangle\" ");
+                write_str_to_current_page(xps, line);
+                break;
+            case gs_cap_butt:
+            case gs_cap_unknown:
+            default:
+                break;
+        }
+        switch(xps->linejoin) {
+            case gs_join_round:
+                gs_snprintf(line, sizeof(line), "StrokeLineJoin=\"Round\" ");
+                write_str_to_current_page(xps, line);
+                break;
+            case gs_join_miter:
+                gs_snprintf(line, sizeof(line), "StrokeLineJoin=\"Miter\" ");
+                write_str_to_current_page(xps, line);
+                gs_snprintf(line, sizeof(line), "StrokeMiterLimit=\"%g\" ", xps->miterlimit);
+                write_str_to_current_page(xps, line);
+                break;
+            case gs_join_bevel:
+                gs_snprintf(line, sizeof(line), "StrokeLineJoin=\"Bevel\" ");
+                write_str_to_current_page(xps, line);
+                break;
+            case gs_join_none:
+            case gs_join_triangle:
+            case gs_join_unknown:
+            default:
+                break;
+        }
     } else { /* fill */
         /* close the path data attribute */
-        write_str_to_current_page(xps, "\" />\n");
+        write_str_to_current_page(xps, " Z\" ");
     }
 
     return 0;
@@ -1890,7 +2083,7 @@ xps_begin_typed_image(gx_device               *dev,
     gs_fixed_rect bbox;
     int bits_per_pixel;
     int num_components;
-    int bsize;
+    size_t bsize;
     cmm_profile_t *icc_profile = NULL;
     gs_color_space_index csindex;
     float index_decode[2];
@@ -1961,8 +2154,10 @@ xps_begin_typed_image(gx_device               *dev,
         if (gs_color_space_is_PSCIE(pcs)) {
             if (pcs->icc_equivalent == NULL) {
                 bool is_lab;
-                if (pgs == NULL)
-                    return(gs_error_invalidaccess);
+                if (pgs == NULL) {
+                    gs_free_object(mem, *pinfo, "xps_begin_image");
+                    return_error(gs_error_invalidaccess);
+                }
                 gs_colorspace_set_icc_equivalent(pcs, &is_lab, pgs->memory);
             }
             icc_profile = pcs->icc_equivalent->cmm_icc_profile_data;
@@ -1981,8 +2176,10 @@ xps_begin_typed_image(gx_device               *dev,
         rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
         rendering_params.rendering_intent = gsPERCEPTUAL;
         rendering_params.cmm = gsCMM_DEFAULT;
-        if (pgs == NULL)
-            return(gs_error_invalidaccess);
+        if (pgs == NULL) {
+            gs_free_object(mem, *pinfo, "xps_begin_image");
+            return_error(gs_error_invalidaccess);
+        }
         pie->icc_link = gsicc_get_link_profile(pgs, dev, icc_profile,
             pgs->icc_manager->default_rgb, &rendering_params, pgs->memory, false);
         icc_profile = pgs->icc_manager->default_rgb;
@@ -1996,8 +2193,9 @@ xps_begin_typed_image(gx_device               *dev,
         icc_data = (xps_icc_data_t*)gs_alloc_bytes(dev->memory->non_gc_memory,
             sizeof(xps_icc_data_t), "xps_begin_image");
         if (icc_data == NULL) {
+            gs_free_object(mem, *pinfo, "xps_begin_image");
             gs_throw(gs_error_VMerror, "Allocation of icc_data failed");
-            return(gs_error_VMerror);
+            return_error(gs_error_VMerror);
         }
 
         icc_data->hash = gsicc_get_hash(icc_profile);
@@ -2014,24 +2212,32 @@ xps_begin_typed_image(gx_device               *dev,
         /* Get name for mark up and for relationship. Have to wait and do
            this after it is added to the package */
         code = xps_create_icc_name(xdev, icc_profile, &(pie->icc_name[0]));
-        if (code < 0)
-            return gs_rethrow_code(code);
+        if (code < 0) {
+            gs_free_object(mem, *pinfo, "xps_begin_image");
+            return_error(gs_rethrow_code(code));
+        }
 
         /* Add profile to the package. Here like images we are going to write
            the data now.  Rather than later. */
-        if (pgs == NULL)
-            return(gs_error_invalidaccess);
+        if (pgs == NULL) {
+            gs_free_object(mem, *pinfo, "xps_begin_image");
+            return_error(gs_error_invalidaccess);
+        }
         code = xps_write_profile(pgs, &(pie->icc_name[0]), icc_profile, xdev);
-        if (code < 0)
-            return gs_rethrow_code(code);
+        if (code < 0) {
+            gs_free_object(mem, *pinfo, "xps_begin_image");
+            return_error(gs_rethrow_code(code));
+        }
 
         /* Add ICC relationship */
         xps_add_icc_relationship(pie);
     } else {
         /* Get name for mark up.  We already have it in the resource list */
         code = xps_create_icc_name(xdev, icc_profile, &(pie->icc_name[0]));
-        if (code < 0)
-            return gs_rethrow_code(code);
+        if (code < 0) {
+            gs_free_object(mem, *pinfo, "xps_begin_image");
+            return_error(gs_rethrow_code(code));
+        }
 
         /* Add ICC relationship.  It may not yet be present for this page. */
         xps_add_icc_relationship(pie);
@@ -2047,8 +2253,10 @@ xps_begin_typed_image(gx_device               *dev,
         (*dev_proc(dev, get_clipping_box)) (dev, &bbox);
         gx_cpath_init_local(&cpath, dev->memory);
         code = gx_cpath_from_rectangle(&cpath, &bbox);
-        if (code < 0)
-            return gs_rethrow_code(code);
+        if (code < 0) {
+            gs_free_object(mem, *pinfo, "xps_begin_image");
+            return_error(gs_rethrow_code(code));
+        }
         pcpath = &cpath;
     } else {
         /* Force vector device to do new path as the clip path is the image
@@ -2058,33 +2266,74 @@ xps_begin_typed_image(gx_device               *dev,
         ((gx_device_vector*) vdev)->clip_path_id = vdev->no_clip_path_id;
     }
 
-    if (pgs == NULL)
-        return(gs_error_invalidaccess);
+    if (pgs == NULL) {
+        gs_free_object(mem, *pinfo, "xps_begin_image");
+        return_error(gs_error_invalidaccess);
+    }
     code = gdev_vector_begin_image(vdev, pgs, pim, pim->format, prect,
         pdcolor, pcpath, mem, &xps_image_enum_procs,
         (gdev_vector_image_enum_t *)pie);
-    if (code < 0)
-        return code;
+    if (code < 0) {
+        gs_free_object(mem, pie, "xps_begin_image");
+        return_error(gs_rethrow_code(code));
+    }
 
-    if ((pie->tif = tiff_from_name(xdev, pie->file_name, false, false)) == NULL)
+    if ((pie->tif = tiff_from_name(xdev, pie->file_name, false, false)) == NULL) {
+        gs_free_object(mem, pie, "xps_begin_image");
         return_error(gs_error_VMerror);
+    }
 
     /* Null out pie.  Only needed for the above vector command and tiff set up */
     xdev->xps_pie = NULL;
     xps_tiff_set_handlers();
     code = tiff_set_values(pie, pie->tif, icc_profile, force8bit);
-    if (code < 0)
-        return gs_rethrow_code(code);
+    if (code < 0) {
+        gs_free_object(mem, pie, "xps_begin_image");
+        return_error(gs_rethrow_code(code));
+    }
     code = TIFFCheckpointDirectory(pie->tif);
 
     num_components = gs_color_space_num_components(pcs);
     bits_per_pixel = pim->BitsPerComponent * num_components;
+
+    {
+        /* This is a really hacky attempt to avoid running out of memory. This is
+         * inspired by various OSS-fuzz bugs but in particular 53398. This device
+         * uses the libtiff library to write images into the XPS file, libtiff won't
+         * let us use our own memory manager and have rejected patches to allow it
+         * to do so, so it uses system memory. If we have a badly broken file we
+         * might end up asking it to write a ridiculously large image which will
+         * cause it to eat all system memory. So here we try to limit it to any pre-defined
+         * limit. Also, if the width * height * 24 bits (RGB) goes negative then we know
+         * we've exceeded 2^(64 - 1) bytes, which is unreasonable too.
+         */
+        int64_t memory_needed = (int64_t)pim->Width * (int64_t)pim->Height * 3;
+        gs_memory_status_t status;
+
+        if (memory_needed < 0) {
+            gs_free_object(mem, pie, "xps_begin_image");
+            return_error(gs_error_VMerror);
+        }
+
+        gs_memory_status(dev->memory->gs_lib_ctx->memory, &status);
+        if (status.limit < (size_t)~1) {
+            if ( memory_needed > status.limit) {
+                gs_free_object(mem, pie, "xps_begin_image");
+                return_error(gs_error_VMerror);
+            }
+        }
+    }
+
     pie->decode_st.bps = bits_per_pixel / num_components;
     pie->bytes_comp = (pie->decode_st.bps > 8 ? 2 : 1);
     pie->decode_st.spp = num_components;
     pie->decode_st.unpack = NULL;
     get_unpack_proc((gx_image_enum_common_t*)pie, &(pie->decode_st), pim->format,
         pim->Decode);
+    if (pie->decode_st.unpack == NULL){
+        gs_free_object(mem, pie, "xps_begin_image");
+        return_error(gs_rethrow_code(gs_error_rangecheck));
+    }
 
     /* The decode mapping for index colors needs an adjustment */
     if (csindex == gs_color_space_index_Indexed) {
@@ -2114,17 +2363,20 @@ xps_begin_typed_image(gx_device               *dev,
     if (csindex == gs_color_space_index_Indexed ||
         csindex == gs_color_space_index_Separation ||
         csindex == gs_color_space_index_DeviceN) {
-        bsize = (pim->Width + 15) * icc_profile->num_comps;
+        bsize = ((unsigned int)pim->Width + 15) * icc_profile->num_comps;
         pie->devc_buffer = gs_alloc_bytes(mem, bsize, "xps_begin_typed_image(devc_buffer)");
         if (pie->devc_buffer == 0) {
             gs_free_object(mem, pie, "xps_begin_typed_image");
             *pinfo = NULL;
             return_error(gs_error_VMerror);
         }
-        /* Also, the color remaps need the gs_gstate */
-        pie->pgs = pgs;
     }
 
+    /* Also, the color remaps need the gs_gstate */
+    pie->pgs = pgs;
+    if (pgs != NULL)
+        pie->pgs_level = pgs->level;
+
     *pinfo = (gx_image_enum_common_t *)pie;
     return 0;
 
@@ -2209,6 +2461,9 @@ const gx_image_plane_t *planes, int height, int *rows_used)
     void *bufend = (void*)(pie->buffer + width * bytes_comp * pie->decode_st.spp);
     byte *outbuffer;
 
+    if (info->pgs != NULL && info->pgs->level < info->pgs_level)
+        return_error(gs_error_undefinedresult);
+
     if (width_bits != pie->bits_per_row || (data_bit & 7) != 0)
         return_error(gs_error_rangecheck);
     if (height > pie->height - pie->y)
@@ -2579,7 +2834,7 @@ static void xps_tiff_cleanup(xps_image_enum_t *xpie)
         void *t = TIFFClientdata(xpie->tif);
         TIFFCleanup(xpie->tif);
         xpie->tif = NULL;
-        gs_free_object(xpie->memory->non_gc_memory, t, "xps_image_enum_finalize");
+        gs_free_object(xpie->dev->memory->non_gc_memory, t, "xps_image_enum_finalize");
     }
 }
 
