@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2023 Artifex Software, Inc.
+/* Copyright (C) 2019-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -411,6 +411,7 @@ jpg_term_source(j_decompress_ptr cinfo)
 {
 }
 
+OPTIMIZE_SETJMP
 static void
 jpg_error_exit(j_common_ptr cinfo)
 {
@@ -483,6 +484,7 @@ align_setjmp_buffer(void *ptr)
     return (jmp_buf *)(void *)aligned;
 }
 
+OPTIMIZE_SETJMP
 static int
 jpg_impl_process(pl_interp_implementation_t * impl, stream_cursor_read * pr)
 {
@@ -610,6 +612,8 @@ jpg_impl_process(pl_interp_implementation_t * impl, stream_cursor_read * pr)
             /* Now we've got the data from the image header, we can
              * make the gs image instance */
             jpg->byte_width = (jpg->bpp>>3)*jpg->width;
+            if (jpg->width != 0 && jpg->byte_width / (jpg->bpp>>3) != jpg->width)
+                return gs_note_error(gs_error_undefinedresult);
 
             jpg->nulldev = gs_currentdevice(jpg->pgs);
             rc_increment(jpg->nulldev);
