@@ -58,11 +58,11 @@
 
 // Alignment to memory pointer
 
-// (Ultra)SPARC with gcc requires ptr alignment of 8 bytes
-// even though sizeof(void *) is only four: for greatest flexibility
-// allow the build to specify ptr alignment.
+// CMS structures contain pointers or doubles.
+// Some 32-bit platforms require doubles to be 8 byte aligned.
+// For greatest flexibility allow the build to specify ptr alignment.
 #ifndef CMS_PTR_ALIGNMENT
-# if defined(sparc) || defined(__sparc) || defined(__sparc__)
+# if defined(sparc) || defined(__sparc) || defined(__sparc__) || defined(__EMSCRIPTEN__)
 #  define CMS_PTR_ALIGNMENT 8
 # else
 #  define CMS_PTR_ALIGNMENT sizeof(void *)
@@ -1116,7 +1116,7 @@ cmsBool   _cmsAdaptationMatrix(cmsContext ContextID, cmsMAT3* r, const cmsMAT3*
 
 cmsBool   _cmsBuildRGB2XYZtransferMatrix(cmsContext ContextID, cmsMAT3* r, const cmsCIExyY* WhitePoint, const cmsCIExyYTRIPLE* Primaries);
 
-void _cmsFindFormatter(_cmsTRANSFORM* p, cmsUInt32Number InputFormat, cmsUInt32Number OutputFormat, cmsUInt32Number flags);
+void _cmsFindFormatter(cmsContext ContextID, _cmsTRANSFORM* p, cmsUInt32Number InputFormat, cmsUInt32Number OutputFormat, cmsUInt32Number flags);
 
 cmsUInt32Number _cmsAdjustReferenceCount(cmsUInt32Number *rc, int delta);
 
