@@ -142,19 +142,17 @@ l_int32    x, y, pix1lskip, pix2lskip, rowwords1, rowwords2;
 l_uint32   word1, word2, andw;
 l_uint32  *row1, *row2;
 
-    PROCNAME("pixCorrelationScore");
-
     if (!pscore)
-        return ERROR_INT("&score not defined", procName, 1);
+        return ERROR_INT("&score not defined", __func__, 1);
     *pscore = 0.0;
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 1);
+        return ERROR_INT("pix1 undefined or not 1 bpp", __func__, 1);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 1);
+        return ERROR_INT("pix2 undefined or not 1 bpp", __func__, 1);
     if (!tab)
-        return ERROR_INT("tab not defined", procName, 1);
+        return ERROR_INT("tab not defined", __func__, 1);
     if (area1 <= 0 || area2 <= 0)
-        return ERROR_INT("areas must be > 0", procName, 1);
+        return ERROR_INT("areas must be > 0", __func__, 1);
 
         /* Eliminate based on size difference */
     pixGetDimensions(pix1, &wi, &hi, NULL);
@@ -443,16 +441,14 @@ l_uint32  *row1, *row2;
 l_float32  score;
 l_int32    threshold;
 
-    PROCNAME("pixCorrelationScoreThresholded");
-
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 0);
+        return ERROR_INT("pix1 undefined or not 1 bpp", __func__, 0);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 0);
+        return ERROR_INT("pix2 undefined or not 1 bpp", __func__, 0);
     if (!tab)
-        return ERROR_INT("tab not defined", procName, 0);
+        return ERROR_INT("tab not defined", __func__, 0);
     if (area1 <= 0 || area2 <= 0)
-        return ERROR_INT("areas must be > 0", procName, 0);
+        return ERROR_INT("areas must be > 0", __func__, 0);
 
         /* Eliminate based on size difference */
     pixGetDimensions(pix1, &wi, &hi, NULL);
@@ -747,19 +743,17 @@ pixCorrelationScoreSimple(PIX        *pix1,
 l_int32  wi, hi, wt, ht, delw, delh, idelx, idely, count;
 PIX     *pixt;
 
-    PROCNAME("pixCorrelationScoreSimple");
-
     if (!pscore)
-        return ERROR_INT("&score not defined", procName, 1);
+        return ERROR_INT("&score not defined", __func__, 1);
     *pscore = 0.0;
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 1);
+        return ERROR_INT("pix1 undefined or not 1 bpp", __func__, 1);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 1);
+        return ERROR_INT("pix2 undefined or not 1 bpp", __func__, 1);
     if (!tab)
-        return ERROR_INT("tab not defined", procName, 1);
+        return ERROR_INT("tab not defined", __func__, 1);
     if (!area1 || !area2)
-        return ERROR_INT("areas must be > 0", procName, 1);
+        return ERROR_INT("areas must be > 0", __func__, 1);
 
         /* Eliminate based on size difference */
     pixGetDimensions(pix1, &wi, &hi, NULL);
@@ -848,19 +842,17 @@ pixCorrelationScoreShifted(PIX        *pix1,
 l_int32  w1, h1, w2, h2, count;
 PIX     *pixt;
 
-    PROCNAME("pixCorrelationScoreShifted");
-
     if (!pscore)
-        return ERROR_INT("&score not defined", procName, 1);
+        return ERROR_INT("&score not defined", __func__, 1);
     *pscore = 0.0;
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 1);
+        return ERROR_INT("pix1 undefined or not 1 bpp", __func__, 1);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 1);
+        return ERROR_INT("pix2 undefined or not 1 bpp", __func__, 1);
     if (!tab)
-        return ERROR_INT("tab not defined", procName, 1);
+        return ERROR_INT("tab not defined", __func__, 1);
     if (!area1 || !area2)
-        return ERROR_INT("areas must be > 0", procName, 1);
+        return ERROR_INT("areas must be > 0", __func__, 1);
 
     pixGetDimensions(pix1, &w1, &h1, NULL);
     pixGetDimensions(pix2, &w2, &h2, NULL);
