@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2024 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -1279,7 +1279,7 @@ pcl_start_raster(uint src_width, uint src_height, pcl_state_t * pcs)
                                        &st_seed_row_t_element,
                                        "start PCL raster");
     if (pseed_rows != 0) {
-        int seed_row_bytes = (prast->src_width * prast->bits_per_plane + 7)
+        size_t seed_row_bytes = ((size_t)prast->src_width * prast->bits_per_plane + 7)
             / 8;
 
         int nplanes = prast->nplanes;
