@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -124,6 +124,6 @@ long int pjl_get_named_resource_size(pjl_parser_state * pst, char *name);
 
 /* returns the contents of the resource in "mem" allocated by the
    client.  Returns < 0 if it fails to find the object */
-int pjl_get_named_resource(pjl_parser_state * pst, char *name, byte * data);
+int pjl_get_named_resource(pjl_parser_state * pst, char *name, byte * data, long int datasize);
 
 #endif /* pjparse_INCLUDED */
