@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2023 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -480,7 +480,7 @@ pl_glyph_name(gs_font * pfont, gs_glyph glyph, gs_const_string * pstr)
             }
             if (plfont->next_name_index > plfont->max_name_index) {
                 char **temp = NULL;
-                temp = (char **)gs_alloc_bytes(pfont->memory, (plfont->max_name_index + 256) * sizeof (char *), "names storage");
+                temp = (char **)gs_alloc_bytes(pfont->memory, (size_t)(plfont->max_name_index + 256) * sizeof (char *), "names storage");
                 if (temp == NULL) {
                     gs_free_object(pfont->memory, (byte *)pstr->data, "free string on error");
                     pstr->data = NULL;
@@ -791,6 +791,8 @@ pl_clone_font(const pl_font_t * src, gs_memory_t * mem, client_name_t cname)
         plfont->glyphs.table =
             gs_alloc_struct_array(mem, src->glyphs.size, pl_font_glyph_t,
                                   &st_pl_font_glyph_element_f, cname);
+        if (plfont->glyphs.table == NULL)
+            return 0;
         plfont->glyphs.used = src->glyphs.used;
         plfont->glyphs.limit = src->glyphs.limit;
         plfont->glyphs.size = src->glyphs.size;
