@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -39,7 +39,9 @@ pl_set_pcl_halftone(gs_gstate * pgs, gs_mapping_proc transfer_proc,
     code = gs_settransfer(pgs, transfer_proc);
     if (code < 0)
         return code;
+    memset(&ht.rc, 0x00, sizeof(ht.rc));
     ht.type = ht_type_threshold;
+    ht.objtype = HT_OBJTYPE_DEFAULT;
     ht.params.threshold.width = width;
     ht.params.threshold.height = height;
     ht.params.threshold.thresholds.data = threshold_data.data;
