@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -1100,6 +1100,13 @@ const byte apxSetROP[] = {
 int
 pxSetROP(px_args_t * par, px_state_t * pxs)
 {
+    /* 252 here is a magic number. It is the default state of RasterOPs which appears to be
+     * fully opaque. Since it's the default, and that seems to work, permit it to be set.
+     */
+    if (pxs->high_level_device && !pxs->supports_rasterops && par->pv[0]->value.i != 252) {
+        errprintf(pxs->memory, "Unsupported use of RasterOP %d detected. Output may not be correct.\n", par->pv[0]->value.i);
+        return 0;
+    }
     gs_setrasterop(pxs->pgs, (gs_rop3_t) (par->pv[0]->value.i));
     return 0;
 }
