@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -158,10 +158,6 @@ pxl_impl_characteristics(const pl_interp_implementation_t * impl)        /* impl
     static pl_interp_characteristics_t pxl_characteristics = {
         "PCLXL",
         pxl_detect_language,
-        "Artifex",
-        PXLVERSION,
-        PXLBUILDDATE,
-        px_parser_min_input_size
     };
     return &pxl_characteristics;
 }
@@ -257,7 +253,7 @@ pxl_impl_init_job(pl_interp_implementation_t * impl,
     pxl_interp_instance_t *pxli = impl->interp_client_data;
     px_state_t *pxs = pxli->pxs;
     gs_memory_t *mem = pxli->memory;
-    
+
     enum { Sbegin, Ssetdevice, Sinitg, Sgsave, Serase, Sdone } stage;
 
     px_reset_errors(pxli->pxs);
@@ -271,6 +267,8 @@ pxl_impl_init_job(pl_interp_implementation_t * impl,
 
     pxs->interpolate = pl_main_get_interpolate(mem);
     pxs->nocache = pl_main_get_nocache(mem);
+    pxs->high_level_device = pl_main_get_high_level_device(mem);
+    pxs->supports_rasterops = pl_main_get_rasterop_support(mem);
     gs_setscanconverter(pxli->pgs, pl_main_get_scanconverter(mem));
 
     if (pxs->nocache)
@@ -281,6 +279,14 @@ pxl_impl_init_job(pl_interp_implementation_t * impl,
     if ((code = gs_setdevice_no_erase(pxli->pgs, device)) < 0)  /* can't erase yet */
         goto pisdEnd;
 
+    /* Warn the device that PXL uses ROPs. */
+    if (code == 0) {
+        code = put_param_bool(pxli, "LanguageUsesROPs", true);
+
+        if (!device->is_open)
+            code = gs_opendevice(device);
+    }
+
     /* Init XL graphics */
     stage = Sinitg;
     if ((code = px_initgraphics(pxli->pxs)) < 0)
@@ -292,9 +298,6 @@ pxl_impl_init_job(pl_interp_implementation_t * impl,
 
     /* Do inits of gstate that may be reset by setdevice */
     gs_setaccuratecurves(pxli->pgs, true);      /* All H-P languages want accurate curves. */
-    /* disable hinting at high res */
-    if (gs_currentdevice(pxli->pgs)->HWResolution[0] >= 300)
-        gs_setgridfittt(pxs->font_dir, 0);
 
     /* gsave and grestore (among other places) assume that */
     /* there are at least 2 gstates on the graphics stack. */
@@ -329,14 +332,6 @@ pxl_impl_init_job(pl_interp_implementation_t * impl,
             break;
     }
 
-    /* Warn the device that PXL uses ROPs. */
-    if (code == 0) {
-        code = put_param_bool(pxli, "LanguageUsesROPs", true);
-
-        if (!device->is_open)
-            code = gs_opendevice(device);
-    }
-
     return code;
 }
 
@@ -570,7 +565,8 @@ pl_interp_implementation_t pxl_implementation = {
     pxl_impl_report_errors,
     pxl_impl_dnit_job,
     pxl_impl_deallocate_interp_instance,
-    NULL
+    NULL,                       /* pxl_impl_reset */
+    NULL                        /* interp_client_data */
 };
 
 /* ---------- Utility Procs ----------- */
