@@ -1,4 +1,4 @@
-/* Copyright (C) 2020-2022 Artifex Software, Inc.
+/* Copyright (C) 2020-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 #include "strmio.h"
@@ -108,11 +108,22 @@ static int cmap_endcodespacerange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *
     /* increment to_pop to cover the mark object */
     numranges = to_pop++;
     while (numranges % 2) numranges--;
-    if (numranges > 200) {
-        (void)pdf_ps_stack_pop(s, to_pop);
+
+    /* The following hard limit is an implementation limit */
+    if (numranges < 0 || numranges + 1 > PDF_PS_STACK_MAX) {
+        pdfi_set_error(s->pdfi_ctx, 0, NULL, E_PDF_BAD_TYPE0_CMAP, "cmap_endcodespacerange_func", NULL);
         return_error(gs_error_syntaxerror);
     }
 
+    /* The following "soft" limit is according to the spec */
+    if (numranges > 200) {
+        int code;
+        if ((code = pdfi_set_warning_stop(s->pdfi_ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_LIMITCHECK_TYPE0_CMAP, "cmap_endcodespacerange_func", NULL)) < 0) {
+            (void)pdf_ps_stack_pop(s, to_pop);
+            return code;
+        }
+    }
+
     if (numranges > 0
      && pdf_ps_obj_has_type(&(s->cur[0]), PDF_PS_OBJ_STRING)  && s->cur[0].size <= MAX_CMAP_CODE_SIZE
      && pdf_ps_obj_has_type(&(s->cur[-1]), PDF_PS_OBJ_STRING) && s->cur[-1].size <= MAX_CMAP_CODE_SIZE) {
@@ -123,7 +134,7 @@ static int cmap_endcodespacerange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *
                           sizeof(gx_code_space_range_t), "cmap_endcodespacerange_func(ranges)");
         if (code_space->ranges != NULL) {
             if (nr > 0) {
-                memcpy(code_space->ranges, gcsr, nr);
+                memcpy(code_space->ranges, gcsr, nr * sizeof(gx_code_space_range_t));
                 gs_free_object(mem, gcsr, "cmap_endcodespacerange_func(gcsr");
             }
 
@@ -161,7 +172,7 @@ static int cmap_insert_map(pdfi_cmap_range_t *cmap_range, pdfi_cmap_range_map_t
 static int general_endcidrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, pdf_cmap *pdficmap, pdfi_cmap_range_t *cmap_range)
 {
     int ncodemaps, to_pop = pdf_ps_stack_count_to_mark(s, PDF_PS_OBJ_MARK);
-    int i, j;
+    unsigned int i, j;
     pdfi_cmap_range_map_t *pdfir;
     pdf_ps_stack_object_t *stobj;
 
@@ -171,11 +182,22 @@ static int general_endcidrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, pdf_cmap
      * startcode, endcode and basecid
      */
     while (ncodemaps % 3) ncodemaps--;
-    if (ncodemaps > 300) {
-        (void)pdf_ps_stack_pop(s, to_pop);
+
+    /* The following hard limit is an implementation limit */
+    if (ncodemaps < 0 || ncodemaps + 1 > PDF_PS_STACK_MAX) {
+        pdfi_set_error(s->pdfi_ctx, 0, NULL, E_PDF_BAD_TYPE0_CMAP, "general_endcidrange_func", NULL);
         return_error(gs_error_syntaxerror);
     }
 
+    /* The following "soft" limit is according to the spec */
+    if (ncodemaps > 300) {
+        int code;
+        if ((code = pdfi_set_warning_stop(s->pdfi_ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_LIMITCHECK_TYPE0_CMAP, "general_endcidrange_func", NULL)) < 0) {
+            (void)pdf_ps_stack_pop(s, to_pop);
+            return code;
+        }
+    }
+
     stobj = &s->cur[-ncodemaps] + 1;
 
     for (i = 0; i < ncodemaps; i += 3) {
@@ -183,7 +205,10 @@ static int general_endcidrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, pdf_cmap
 
         if (pdf_ps_obj_has_type(&(stobj[i + 2]), PDF_PS_OBJ_INTEGER)
         &&  pdf_ps_obj_has_type(&(stobj[i + 1]), PDF_PS_OBJ_STRING)
-        &&  pdf_ps_obj_has_type(&(stobj[i]), PDF_PS_OBJ_STRING)){
+        &&  pdf_ps_obj_has_type(&(stobj[i]), PDF_PS_OBJ_STRING)
+        &&  pdf_ps_obj_size(&(stobj[i])) > 0
+        &&  pdf_ps_obj_size(&(stobj[i])) == pdf_ps_obj_size(&(stobj[i + 1]))){
+
             uint cidbase = stobj[i + 2].val.i;
 
             /* First, find the length of the prefix */
@@ -198,7 +223,7 @@ static int general_endcidrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, pdf_cmap
             }
 
             if (preflen > MAX_CMAP_CODE_SIZE || stobj[i].size - preflen > MAX_CMAP_CODE_SIZE || stobj[i + 1].size - preflen > MAX_CMAP_CODE_SIZE
-                || stobj[i].size - preflen < 0 || stobj[i + 1].size - preflen < 0) {
+                || ((int64_t)stobj[i].size) - preflen < 0 || ((int64_t)stobj[i + 1].size) - preflen < 0) {
                 (void)pdf_ps_stack_pop(s, to_pop);
                 return_error(gs_error_syntaxerror);
             }
@@ -268,7 +293,7 @@ static int cmap_endfbrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, by
 {
     pdf_cmap *pdficmap = (pdf_cmap *)s->client_data;
     int ncodemaps, to_pop = pdf_ps_stack_count_to_mark(s, PDF_PS_OBJ_MARK);
-    int i, j, k;
+    unsigned int i, j, k;
     pdfi_cmap_range_map_t *pdfir;
     pdf_ps_stack_object_t *stobj;
 
@@ -278,10 +303,20 @@ static int cmap_endfbrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, by
      */
     while (ncodemaps % 3) ncodemaps--;
 
-    if (ncodemaps > 300) {
-        (void)pdf_ps_stack_pop(s, to_pop);
+    /* The following hard limit is an implementation limit */
+    if (ncodemaps < 0 || ncodemaps + 1 > PDF_PS_STACK_MAX) {
+        pdfi_set_error(s->pdfi_ctx, 0, NULL, E_PDF_BAD_TYPE0_CMAP, "cmap_endfbrange_func", NULL);
         return_error(gs_error_syntaxerror);
     }
+    /* The following "soft" limit is according to the spec */
+
+    if (ncodemaps > 300) {
+        int code;
+        if ((code = pdfi_set_warning_stop(s->pdfi_ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_LIMITCHECK_TYPE0_CMAP, "cmap_endfbrange_func", NULL)) < 0) {
+            (void)pdf_ps_stack_pop(s, to_pop);
+            return code;
+        }
+    }
 
     stobj = &s->cur[-ncodemaps] + 1;
     for (i = 0; i < ncodemaps; i += 3) {
@@ -309,45 +344,42 @@ static int cmap_endfbrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, by
 
         if (pdf_ps_obj_has_type(&(stobj[i + 2]), PDF_PS_OBJ_ARRAY)
         &&  pdf_ps_obj_has_type(&(stobj[i + 1]), PDF_PS_OBJ_STRING)
-        &&  pdf_ps_obj_has_type(&(stobj[i]), PDF_PS_OBJ_STRING)){
+        &&  pdf_ps_obj_has_type(&(stobj[i]), PDF_PS_OBJ_STRING)
+        &&  pdf_ps_obj_size(&(stobj[i + 1])) > 0
+        &&  pdf_ps_obj_size(&(stobj[i])) > 0){
 
-            uint cidbase = 0;
-            int srcs = 0, srce = 0;
-            int kslen = stobj[i].size;
+            uint cidbase;
+            int m, size;
 
-            if (kslen > 4) {
-                dprintf("CMap: value out of range\n");
+            if (stobj[i + 2].size < 1)
                 continue;
-            }
 
-            for (k = 0; k < stobj[i].size; k++) {
-                srcs |= stobj[i].val.string[stobj[i].size - k - 1] << (8 * k);
-            }
-            for (k = 0; k < stobj[i + 1].size; k++) {
-                srce |= stobj[i + 1].val.string[stobj[i + 1].size - k - 1] << (8 * k);
-            }
+            else if (stobj[i + 2].size == 1) {
+                if (stobj[i + 2].val.arr[0].type != PDF_PS_OBJ_STRING)
+                     continue;
 
-            for (k = srcs; k < srce + 1; k++) {
-                int m, size;
-                char srccode[4]; /* we only deal with up to 4 bytes */
-                char *psrccode = &(srccode[4 - kslen]);
+                size = stobj[i + 2].val.arr[0].size;
 
-                if ((k - srcs) < stobj[i + 2].size) {
-                    if (stobj[i + 2].val.arr[k - srcs].type != PDF_PS_OBJ_STRING)
-                        continue;
-                    size = stobj[i + 2].val.arr[k - srcs].size;
+                cidbase = 0;
+                for (m = 0; m < size; m++) {
+                    cidbase |= stobj[i + 2].val.arr[0].val.string[size - m - 1] << (8 * m);
+                }
 
-                    cidbase = 0;
-                    for (m = 0; m < size; m++) {
-                        cidbase |= stobj[i + 2].val.arr[k - srcs].val.string[size - m - 1];
+                /* First, find the length of the prefix */
+                for (preflen = 0; preflen < stobj[i].size; preflen++) {
+                    if(stobj[i].val.string[preflen] != stobj[i + 1].val.string[preflen]) {
+                        break;
                     }
                 }
-                else {
-                    cidbase++;
+
+                if (preflen == stobj[i].size) {
+                    preflen = 1;
                 }
 
-                for (m = 0; m < kslen; m++) {
-                    psrccode[m] = (k >> (8 * (kslen - m - 1))) & 0xff;
+                if (preflen > MAX_CMAP_CODE_SIZE || stobj[i].size - preflen > MAX_CMAP_CODE_SIZE || stobj[i + 1].size - preflen > MAX_CMAP_CODE_SIZE
+                    || ((int64_t)stobj[i].size) - preflen < 0 || ((int64_t)stobj[i + 1].size) - preflen < 0) {
+                    (void)pdf_ps_stack_pop(s, to_pop);
+                    return_error(gs_error_syntaxerror);
                 }
 
                 /* Find how many bytes we need for the cidbase value */
@@ -355,15 +387,13 @@ static int cmap_endfbrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, by
                 for (valuelen = 16; valuelen < 32 && (cidbase >> valuelen) > 0; valuelen += 1)
                     DO_NOTHING;
 
-                preflen = kslen > 4 ? 4 : kslen;
-
                 valuelen = ((valuelen + 7) & ~7) >> 3;
 
                 /* The prefix is already directly in the gx_cmap_lookup_range_t
                  * We need to store the lower and upper character codes, after lopping the prefix
                  * off them. The upper and lower codes must be the same number of bytes.
                  */
-                j = sizeof(pdfi_cmap_range_map_t) + (kslen - preflen) + valuelen;
+                j = sizeof(pdfi_cmap_range_map_t) + 2 * (stobj[i].size - preflen) + valuelen;
 
                 pdfir = (pdfi_cmap_range_map_t *)gs_alloc_bytes(mem, j, "cmap_endcidrange_func(pdfi_cmap_range_map_t)");
                 if (pdfir != NULL) {
@@ -371,31 +401,123 @@ static int cmap_endfbrange_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, by
                     pdfir->next = NULL;
                     gxr->num_entries = 1;
                     gxr->keys.data = (byte *)&(pdfir[1]);
-                    gxr->values.data = gxr->keys.data + (kslen - preflen);
+                    gxr->values.data = gxr->keys.data + 2 * (stobj[i].size - preflen);
 
                     gxr->cmap = NULL;
                     gxr->font_index = 0;
-                    gxr->key_is_range = false;
+                    gxr->key_is_range = true;
                     gxr->value_type = CODE_VALUE_CID;
                     gxr->key_prefix_size = preflen;
-                    gxr->key_size = kslen - gxr->key_prefix_size;
-                    memcpy(gxr->key_prefix, psrccode, gxr->key_prefix_size);
+                    gxr->key_size = stobj[i].size - gxr->key_prefix_size;
+                    memcpy(gxr->key_prefix, stobj[i].val.string, gxr->key_prefix_size);
 
-                    memcpy(gxr->keys.data, psrccode + gxr->key_prefix_size, kslen - gxr->key_prefix_size);
+                    memcpy(gxr->keys.data, stobj[i].val.string + gxr->key_prefix_size, stobj[i].size - gxr->key_prefix_size);
+                    memcpy(gxr->keys.data + (stobj[i].size - gxr->key_prefix_size), stobj[i + 1].val.string + gxr->key_prefix_size, stobj[i + 1].size - gxr->key_prefix_size);
 
-                    gxr->keys.size = kslen - gxr->key_prefix_size;
+                    gxr->keys.size = (stobj[i].size - gxr->key_prefix_size) + (stobj[i + 1].size - gxr->key_prefix_size);
                     for (j = 0; j < valuelen; j++) {
                         gxr->values.data[j] = (cidbase >> ((valuelen - 1 - j) * 8)) & 255;
                     }
                     gxr->value_size = valuelen; /* I'm not sure.... */
                     gxr->values.size = valuelen;
-                    if (cmap_insert_map(&(pdficmap->cmap_range), pdfir) < 0) break;
+                    if (cmap_insert_map(&pdficmap->cmap_range, pdfir) < 0) break;
                 }
                 else {
                     (void)pdf_ps_stack_pop(s, to_pop);
                     return_error(gs_error_VMerror);
                 }
             }
+            else {
+                int m, size, keysize;
+                uint codelo = 0, codehi = 0;
+
+                size = stobj[i].size;
+                for (m = 0; m < size; m++) {
+                    codelo |= stobj[i].val.string[size - m - 1] << (8 * m);
+                }
+                size = stobj[i + 1].size;
+                for (m = 0; m < size; m++) {
+                    codehi |= stobj[i + 1].val.string[size - m - 1] << (8 * m);
+                }
+
+                if (codehi <= codelo || stobj[i + 2].size < (codehi - codelo))
+                    continue;
+
+                for (k = codelo; k <= codehi; k++) {
+                    uint cidbase;
+                    int ind = k - codelo;
+
+                    if (stobj[i + 2].val.arr[ind].type != PDF_PS_OBJ_STRING)
+                         continue;
+
+                    size = stobj[i + 2].val.arr[ind].size;
+
+                    cidbase = 0;
+                    for (m = 0; m < size; m++) {
+                        cidbase |= stobj[i + 2].val.arr[ind].val.string[size - m - 1] << (8 * m);
+                    }
+                    /* Find how many bytes we need for the cidbase value */
+                    /* We always store at least two bytes for the cidbase value */
+                    for (valuelen = 16; valuelen < 32 && (cidbase >> valuelen) > 0; valuelen += 1)
+                        DO_NOTHING;
+
+                    valuelen = ((valuelen + 7) & ~7) >> 3;
+
+                    /* Find how many bytes we need for the cidbase value */
+                    /* We always store at least two bytes for the cidbase value */
+                    for (keysize = 16; keysize < 32 && (cidbase >> keysize) > 0; keysize += 1)
+                        DO_NOTHING;
+
+                    keysize = ((keysize + 7) & ~7) >> 3;
+                    if (keysize > MAX_CMAP_CODE_SIZE * 2) {
+                        (void)pdf_ps_stack_pop(s, to_pop);
+                        return_error(gs_error_syntaxerror);
+                    }
+                    preflen = keysize > 4 ? 4 : keysize;
+                    keysize -= preflen;
+
+                    /* The prefix is already directly in the gx_cmap_lookup_range_t
+                     * We need to store the lower and upper character codes, after lopping the prefix
+                     * off them. The upper and lower codes must be the same number of bytes.
+                     */
+                    j = sizeof(pdfi_cmap_range_map_t) + keysize + valuelen;
+
+                    pdfir = (pdfi_cmap_range_map_t *)gs_alloc_bytes(mem, j, "cmap_endcidrange_func(pdfi_cmap_range_map_t)");
+                    if (pdfir != NULL) {
+                        gx_cmap_lookup_range_t *gxr = &pdfir->range;
+                        pdfir->next = NULL;
+                        gxr->num_entries = 1;
+                        gxr->keys.data = (byte *)&(pdfir[1]);
+                        gxr->values.data = gxr->keys.data + keysize;
+
+                        gxr->cmap = NULL;
+                        gxr->font_index = 0;
+                        gxr->key_is_range = false;
+                        gxr->value_type = CODE_VALUE_CID;
+                        gxr->key_prefix_size = preflen;
+                        gxr->key_size = keysize;
+                        for (j = 0; j < preflen; j++) {
+                            gxr->key_prefix[j] = (k >> ((preflen - 1 - j) * 8)) & 255;
+                        }
+
+                        for (j = preflen; j < preflen + keysize; j++) {
+                            gxr->keys.data[j] = (k >> (((preflen + keysize) - 1 - j) * 8)) & 255;
+                        }
+
+                        gxr->keys.size = keysize;
+                        for (j = 0; j < valuelen; j++) {
+                            gxr->values.data[j] = (cidbase >> ((valuelen - 1 - j) * 8)) & 255;
+                        }
+                        gxr->value_size = valuelen; /* I'm not sure.... */
+                        gxr->values.size = valuelen;
+                        if (cmap_insert_map(&pdficmap->cmap_range, pdfir) < 0) break;
+                    }
+                    else {
+                        (void)pdf_ps_stack_pop(s, to_pop);
+                        return_error(gs_error_VMerror);
+                    }
+                }
+            }
         }
     }
     return pdf_ps_stack_pop(s, to_pop);
@@ -415,18 +537,26 @@ static int general_endcidchar_func(gs_memory_t *mem, pdf_ps_ctx_t *s, pdf_cmap *
      */
     while (ncodemaps % 2) ncodemaps--;
 
-    if (ncodemaps > 200) {
-        (void)pdf_ps_stack_pop(s, to_pop);
+    if (ncodemaps < 0 || ncodemaps + 1 > PDF_PS_STACK_MAX) {
+        pdfi_set_error(s->pdfi_ctx, 0, NULL, E_PDF_BAD_TYPE0_CMAP, "general_endcidchar_func", NULL);
         return_error(gs_error_syntaxerror);
     }
 
+    if (ncodemaps > 200) {
+        int code;
+        if ((code = pdfi_set_warning_stop(s->pdfi_ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_LIMITCHECK_TYPE0_CMAP, "general_endcidchar_func", NULL)) < 0) {
+            (void)pdf_ps_stack_pop(s, to_pop);
+            return code;
+        }
+    }
+
     stobj = &s->cur[-ncodemaps] + 1;
 
     for (i = 0; i < ncodemaps; i += 2) {
         int preflen = 1, valuelen;
 
         if (pdf_ps_obj_has_type(&(stobj[i + 1]), PDF_PS_OBJ_INTEGER)
-        &&  pdf_ps_obj_has_type(&(stobj[i]), PDF_PS_OBJ_STRING)) {
+        &&  pdf_ps_obj_has_type(&(stobj[i]), PDF_PS_OBJ_STRING) && stobj[i].size > 0) {
             uint cidbase = stobj[i + 1].val.i;
 
             /* Find how many bytes we need for the cidbase value */
@@ -498,11 +628,21 @@ static int cmap_endbfchar_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byt
     pdf_ps_stack_object_t *stobj;
     int i, j;
 
-    if (ncodemaps > 200) {
-        (void)pdf_ps_stack_pop(s, ncodemaps);
+    /* The following hard limit is an implementation limit */
+    if (ncodemaps < 0 || ncodemaps + 1 > PDF_PS_STACK_MAX) {
+        pdfi_set_error(s->pdfi_ctx, 0, NULL, E_PDF_BAD_TYPE0_CMAP, "cmap_endbfchar_func", NULL);
         return_error(gs_error_syntaxerror);
     }
 
+    /* The following "soft" limit is according to the spec */
+    if (ncodemaps > 200) {
+        int code;
+        if ((code = pdfi_set_warning_stop(s->pdfi_ctx, gs_note_error(gs_error_syntaxerror), NULL, W_PDF_LIMITCHECK_TYPE0_CMAP, "cmap_endbfchar_func", NULL)) < 0) {
+            (void)pdf_ps_stack_pop(s, ncodemaps);
+            return code;
+        }
+    }
+
     stobj = &s->cur[-ncodemaps] + 1;
 
     for (i = 0; i < ncodemaps; i += 2) {
@@ -536,7 +676,10 @@ static int cmap_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *buf
 
     if (pdf_ps_obj_has_type(&s->cur[-1], PDF_PS_OBJ_NAME)) {
         if (!memcmp(s->cur[-1].val.name, CMAP_NAME_AND_LEN("Registry"))) {
-            pdficmap->csi_reg.data = gs_alloc_bytes(mem, s->cur[0].size + 1, "cmap_def_func(Registry)");
+            if (pdficmap->csi_reg.data != NULL)
+               gs_free_object(mem, pdficmap->csi_reg.data, "cmap_def_func(Registry)");
+
+            pdficmap->csi_reg.data = gs_alloc_bytes(mem, (size_t)s->cur[0].size + 1, "cmap_def_func(Registry)");
             if (pdficmap->csi_reg.data != NULL) {
                 pdficmap->csi_reg.size = s->cur[0].size;
                 if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_STRING)) {
@@ -552,7 +695,10 @@ static int cmap_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *buf
             }
         }
         else if (!memcmp(s->cur[-1].val.name, CMAP_NAME_AND_LEN("Ordering"))) {
-            pdficmap->csi_ord.data = gs_alloc_bytes(mem, s->cur[0].size + 1, "cmap_def_func(Ordering)");
+            if (pdficmap->csi_ord.data != NULL)
+               gs_free_object(mem, pdficmap->csi_ord.data, "cmap_def_func(Ordering)");
+
+            pdficmap->csi_ord.data = gs_alloc_bytes(mem, (size_t)s->cur[0].size + 1, "cmap_def_func(Ordering)");
             if (pdficmap->csi_ord.data != NULL) {
                 pdficmap->csi_ord.size = s->cur[0].size;
                 if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_STRING))
@@ -574,7 +720,10 @@ static int cmap_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *buf
             }
         }
         else if (!memcmp(s->cur[-1].val.name, CMAP_NAME_AND_LEN("CMapName"))) {
-            pdficmap->name.data = gs_alloc_bytes(mem, s->cur[0].size + 1, "cmap_def_func(CMapName)");
+            if (pdficmap->name.data != NULL)
+               gs_free_object(mem, pdficmap->name.data, "cmap_def_func(CMapName)");
+
+            pdficmap->name.data = gs_alloc_bytes(mem, (size_t)s->cur[0].size + 1, "cmap_def_func(CMapName)");
             if (pdficmap->name.data != NULL) {
                 pdficmap->name.size = s->cur[0].size;
                 if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_STRING))
@@ -609,7 +758,11 @@ static int cmap_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *buf
         else if (!memcmp(s->cur[-1].val.name, CMAP_NAME_AND_LEN("XUID"))) {
             if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_ARRAY)) {
                 int len = s->cur->size;
-                pdficmap->uid.xvalues = (long *)gs_alloc_bytes(mem, len * sizeof(*pdficmap->uid.xvalues), "cmap_def_func(XUID)");
+
+                if (pdficmap->uid.xvalues != NULL)
+                   gs_free_object(mem, pdficmap->uid.xvalues, "cmap_def_func(XUID)");
+
+                pdficmap->uid.xvalues = (long *)gs_alloc_bytes(mem, (size_t)len * sizeof(*pdficmap->uid.xvalues), "cmap_def_func(XUID)");
                 if (pdficmap->uid.xvalues != NULL) {
                      int i;
                      pdf_ps_stack_object_t *a = s->cur->val.arr;
@@ -630,7 +783,14 @@ static int cmap_def_func(gs_memory_t *mem, pdf_ps_ctx_t *s, byte *buf, byte *buf
         }
         else if (!memcmp(s->cur[-1].val.name, CMAP_NAME_AND_LEN("WMode"))) {
             if (pdf_ps_obj_has_type(&s->cur[0], PDF_PS_OBJ_INTEGER)) {
-                pdficmap->wmode = s->cur[0].val.i;
+                if (s->cur[0].val.i != 0) {
+                    if (s->cur[0].val.i != 1) {
+                        code = pdfi_set_warning_stop(s->pdfi_ctx, gs_note_error(gs_error_rangecheck), NULL, W_PDF_BAD_WMODE, "cmap_def_func", NULL);
+                    }
+                    pdficmap->wmode = 1;
+                }
+                else
+                    pdficmap->wmode = 0;
             }
             else {
                 pdficmap->wmode = 0;
@@ -786,7 +946,7 @@ pdfi_read_cmap(pdf_context *ctx, pdf_obj *cmap, pdf_cmap **pcmap)
     pdf_cmap pdficm[3] = {0};
     pdf_cmap *pdfi_cmap = &(pdficm[1]);
     byte *buf = NULL;
-    int64_t buflen;
+    int64_t buflen = 0;
     pdf_ps_ctx_t cmap_ctx;
 
     pdfi_cmap->ctx = ctx;
@@ -834,6 +994,9 @@ pdfi_read_cmap(pdf_context *ctx, pdf_obj *cmap, pdf_cmap **pcmap)
                         upcmap->notdef_cmap_range.ranges = NULL;
                         /* But we keep the subcmap itself because we rely on its storage */
                         pdfi_cmap->next = upcmap;
+                    } else {
+                        code = gs_note_error(gs_error_VMerror);
+                        goto error_out;
                     }
                 }
                 else {
@@ -884,6 +1047,9 @@ pdfi_read_cmap(pdf_context *ctx, pdf_obj *cmap, pdf_cmap **pcmap)
             if (pdfi_cmap->object_num != 0) {
                 code = replace_cache_entry(ctx, (pdf_obj *)pdfi_cmap);
             }
+        } else {
+            code = gs_note_error(gs_error_VMerror);
+            goto error_out;
         }
     }
     else {
