@@ -1,4 +1,4 @@
-/* Copyright (C) 2020-2022 Artifex Software, Inc.
+/* Copyright (C) 2020-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* code for CIDFontType2/Type 9 font handling */
@@ -50,9 +50,31 @@ static int pdfi_cidtype2_CIDMap_proc(gs_font_cid2 *pfont, gs_glyph glyph)
 {
     pdf_cidfont_type2 *pdffont11 = (pdf_cidfont_type2 *)pfont->client_data;
     uint gid = glyph - GS_MIN_CID_GLYPH;
+    int code = 0;
 
-    if (pdffont11->cidtogidmap != NULL && pdffont11->cidtogidmap->length > (gid << 1) + 1) {
-        gid = pdffont11->cidtogidmap->data[gid << 1] << 8 | pdffont11->cidtogidmap->data[(gid << 1) + 1];
+    if (pdffont11->substitute == true) {
+        unsigned int ucc = 0;
+        int code = pfont->procs.decode_glyph((gs_font *)pfont, glyph, -1, NULL, 0);
+        if (code == 2) {
+            uchar sccode[2] = {0};
+            (void)pfont->procs.decode_glyph((gs_font *)pfont, glyph, -1, (ushort *)&sccode, 2);
+            ucc = (sccode[0] << 8) + sccode[1];
+        }
+        else if (code == 4) {
+            uchar iccode[4] = {0};
+            (void)pfont->procs.decode_glyph((gs_font *)pfont, glyph, -1, (ushort *)&iccode, 2);
+            ucc = (iccode[0] << 24) + (iccode[1] << 16) + (iccode[2] << 8) + iccode[3];
+
+        }
+        if (code == 2 || code == 4) {
+            code = pdfi_fapi_check_cmap_for_GID((gs_font *)pfont, (unsigned int)ucc, &gid);
+            if (code < 0 || gid == 0)
+                gid = glyph - GS_MIN_CID_GLYPH;
+        }
+    }
+
+    if (code == 0 && pdffont11->cidtogidmap != NULL && pdffont11->cidtogidmap->length > (gid << 1) + 1) {
+       gid = pdffont11->cidtogidmap->data[gid << 1] << 8 | pdffont11->cidtogidmap->data[(gid << 1) + 1];
     }
 
     return (int)gid;
@@ -61,16 +83,37 @@ static int pdfi_cidtype2_CIDMap_proc(gs_font_cid2 *pfont, gs_glyph glyph)
 static uint pdfi_cidtype2_get_glyph_index(gs_font_type42 *pfont, gs_glyph glyph)
 {
     pdf_cidfont_type2 *pdffont11 = (pdf_cidfont_type2 *)pfont->client_data;
-    uint gid = 0;
+    uint gid = glyph - GS_MIN_CID_GLYPH;
+    int code = 0;
 
     if (glyph < GS_MIN_CID_GLYPH) {
         gid = 0;
     }
     else {
         if (glyph < GS_MIN_GLYPH_INDEX) {
-            gid = glyph - GS_MIN_CID_GLYPH;
-            if (pdffont11->cidtogidmap != NULL && pdffont11->cidtogidmap->length > (gid << 1) + 1) {
-                gid = pdffont11->cidtogidmap->data[gid << 1] << 8 | pdffont11->cidtogidmap->data[(gid << 1) + 1];
+            if (pdffont11->substitute == true) {
+                unsigned int ucc = 0;
+                code = pfont->procs.decode_glyph((gs_font *)pfont, glyph, -1, NULL, 0);
+                if (code == 2) {
+                    uchar sccode[2] = {0};
+                    (void)pfont->procs.decode_glyph((gs_font *)pfont, glyph, -1, (ushort *)&sccode, 2);
+                    ucc = (sccode[0] << 8) + sccode[1];
+                }
+                else if (code == 4) {
+                    uchar iccode[4] = {0};
+                    (void)pfont->procs.decode_glyph((gs_font *)pfont, glyph, -1, (ushort *)&iccode, 2);
+                    ucc = (iccode[0] << 24) + (iccode[1] << 16) + (iccode[2] << 8) + iccode[3];
+
+                }
+                if (code == 2 || code == 4) {
+                    code = pdfi_fapi_check_cmap_for_GID((gs_font *)pfont, (unsigned int)ucc, &gid);
+                    if (code < 0 || gid == 0)
+                        gid = glyph - GS_MIN_CID_GLYPH;
+                }
+            }
+
+            if (code == 0 && pdffont11->cidtogidmap != NULL && pdffont11->cidtogidmap->length > (gid << 1) + 1) {
+               gid = pdffont11->cidtogidmap->data[gid << 1] << 8 | pdffont11->cidtogidmap->data[(gid << 1) + 1];
             }
         }
     }
@@ -84,16 +127,21 @@ pdfi_cidtype2_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 {
     int code;
     pdf_cidfont_type2 *pdffont11 = (pdf_cidfont_type2 *)font->client_data;
-    code = (*pdffont11->orig_glyph_info)(font, glyph, pmat, members, info);
-    if (code < 0)
-        return code;
+    int submembers = members & ~(GLYPH_INFO_WIDTHS | GLYPH_INFO_VVECTOR0 | GLYPH_INFO_VVECTOR1 | GLYPH_INFO_CDEVPROC);
 
-    if ((members & GLYPH_INFO_WIDTHS) != 0
+    /* Call with the values we should get below removed from the "members" request */
+    code = (*pdffont11->orig_glyph_info)(font, glyph, pmat, submembers, info);
+
+    if (code >= 0 && (members & GLYPH_INFO_WIDTHS) != 0
       && glyph > GS_MIN_CID_GLYPH
       && glyph < GS_MIN_GLYPH_INDEX) {
         double widths[6] = {0};
         code = pdfi_get_cidfont_glyph_metrics(font, (glyph - GS_MIN_CID_GLYPH), widths, true);
-        if (code >= 0) {
+        if (code < 0) {
+             /* If we couldn't get values back from W/W2, give up, and fill everything in from glyph_info */
+             code = (*pdffont11->orig_glyph_info)(font, glyph, pmat, members, info);
+        }
+        else {
             if (pmat == NULL) {
                 info->width[0].x = widths[GLYPH_W0_WIDTH_INDEX] / 1000.0;
                 info->width[0].y = widths[GLYPH_W0_HEIGHT_INDEX] / 1000.0;
@@ -133,6 +181,7 @@ pdfi_cidtype2_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
             }
         }
     }
+
     return code;
 }
 
@@ -187,6 +236,13 @@ pdfi_cidtype2_enumerate_glyph(gs_font *font, int *pindex,
     return code;
 }
 
+static void pdfi_set_cidtype2_custom_procs(pdf_cidfont_type2 *pdfttfont)
+{
+    gs_font_type42 *pfont = (gs_font_type42 *)pdfttfont->pfont;
+    pdfttfont->default_font_info = pfont->procs.font_info;
+    pfont->procs.font_info = pdfi_default_font_info;
+}
+
 static int
 pdfi_alloc_cidtype2_font(pdf_context *ctx, pdf_cidfont_type2 **font, bool is_cid)
 {
@@ -204,7 +260,7 @@ pdfi_alloc_cidtype2_font(pdf_context *ctx, pdf_cidfont_type2 **font, bool is_cid
 
 #if REFCNT_DEBUG
     ttfont->UID = ctx->UID++;
-    dmprintf2(ctx->memory, "Allocated object of type %c with UID %"PRIi64"\n", ttfont->type, ttfont->UID);
+    outprintf(ctx->memory, "Allocated object of type %c with UID %"PRIi64"\n", ttfont->type, ttfont->UID);
 #endif
 
     pdfi_countup(ttfont);
@@ -248,18 +304,21 @@ pdfi_alloc_cidtype2_font(pdf_context *ctx, pdf_cidfont_type2 **font, bool is_cid
     pfont->procs.encode_char = pdfi_encode_char;
     pfont->data.string_proc = pdfi_cidtype2_string_proc;
     pfont->procs.glyph_name = ctx->get_glyph_name;
-    pfont->procs.decode_glyph = pdfi_decode_glyph;
+    pfont->procs.decode_glyph = pdfi_cidfont_decode_glyph;
     pfont->procs.define_font = gs_no_define_font;
     pfont->procs.make_font = gs_no_make_font;
-    pfont->procs.font_info = gs_default_font_info;
+
+    ttfont->default_font_info = gs_default_font_info;
+    pfont->procs.font_info = pdfi_default_font_info;
+
     pfont->procs.glyph_info = gs_default_glyph_info;
     pfont->procs.glyph_outline = gs_no_glyph_outline;
     pfont->procs.build_char = NULL;
     pfont->procs.same_font = gs_default_same_font;
     pfont->procs.enumerate_glyph = gs_no_enumerate_glyph;
 
-    pfont->encoding_index = 1;          /****** WRONG ******/
-    pfont->nearest_encoding_index = 1;          /****** WRONG ******/
+    pfont->encoding_index = ENCODING_INDEX_UNKNOWN;
+    pfont->nearest_encoding_index = ENCODING_INDEX_UNKNOWN;
 
     cid_system_info_set_null(&pfont->cidata.common.CIDSystemInfo);
     pfont->cidata.common.CIDCount = 0; /* set later */
@@ -336,15 +395,11 @@ int pdfi_read_cidtype2_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *str
         obj = NULL;
     }
 
-    code = pdfi_dict_knownget_type(ctx, font_dict, "DW", PDF_INT, (pdf_obj **)&obj);
-    if (code > 0) {
-        font->DW = ((pdf_num *)obj)->value.i;
-        pdfi_countdown(obj);
-        obj = NULL;
-    }
-    else {
+    code = pdfi_dict_knownget_number(ctx, font_dict, "DW", &font->DW);
+    if (code <= 0) {
         font->DW = 1000;
     }
+
     code = pdfi_dict_knownget_type(ctx, font_dict, "DW2", PDF_ARRAY, (pdf_obj **)&obj);
     if (code > 0) {
         font->DW2 = (pdf_array *)obj;
@@ -377,7 +432,7 @@ int pdfi_read_cidtype2_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *str
          */
         if (pdfi_type_of(obj) == PDF_STREAM) {
             byte *d;
-            int64_t sz;
+            int64_t sz = 0;
 
             code = pdfi_object_alloc(ctx, PDF_BUFFER, 0, (pdf_obj **)&font->cidtogidmap);
             if (code < 0) {
@@ -440,10 +495,12 @@ int pdfi_read_cidtype2_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *str
     obj = NULL;
 
 
-    code = gs_type42_font_init((gs_font_type42 *)font->pfont, 0);
+    code = gs_type42_font_init((gs_font_type42 *)font->pfont, findex);
     if (code < 0) {
         goto error;
     }
+    pdfi_set_cidtype2_custom_procs(font);
+
     if (uid_is_XUID(&font->pfont->UID))
         uid_free(&font->pfont->UID, font->pfont->memory, "pdfi_read_type1_font");
     uid_set_invalid(&font->pfont->UID);
@@ -476,12 +533,13 @@ int pdfi_read_cidtype2_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *str
     cid2->cidata.orig_procs.get_outline = cid2->data.get_outline;
     cid2->data.get_glyph_index = pdfi_cidtype2_get_glyph_index;
 
+    pdfi_font_set_orig_fonttype(ctx, (pdf_font *)font);
     code = gs_definefont(ctx->font_dir, (gs_font *)font->pfont);
     if (code < 0) {
         goto error;
     }
 
-    code = pdfi_fapi_passfont((pdf_font *)font, 0, NULL, NULL, font->sfnt->data, font->sfnt->length);
+    code = pdfi_fapi_passfont((pdf_font *)font, findex, NULL, NULL, font->sfnt->data, font->sfnt->length);
     if (code < 0) {
         goto error;
     }
@@ -494,6 +552,21 @@ int pdfi_read_cidtype2_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *str
     *ppfont = (pdf_font *)font;
     return code;
 error:
+    pdfi_countdown(obj);
+    obj = NULL;
+    if (pdfi_dict_get(ctx, font_dict, ".Path", &obj) >= 0)
+    {
+        char fname[gp_file_name_sizeof + 1];
+        pdf_string *fobj = (pdf_string *)obj;
+
+        memcpy(fname, fobj->data, fobj->length > gp_file_name_sizeof ? gp_file_name_sizeof : fobj->length);
+        fname[fobj->length > gp_file_name_sizeof ? gp_file_name_sizeof : fobj->length] = '\0';
+
+        (void)pdfi_set_error_var(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_cidtype2_font", "Error reading CIDType2/TrueType font file %s\n", fname);
+    }
+    else {
+        (void)pdfi_set_error_var(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_cidtype2_font", "Error reading embedded CIDType2/TrueType font object %u\n", font_dict->object_num);
+    }
 
     pdfi_countdown(obj);
     pdfi_countdown(font);
@@ -518,6 +591,10 @@ int pdfi_free_font_cidtype2(pdf_obj *font)
     pdfi_countdown(pdfcidf->registry);
     pdfi_countdown(pdfcidf->ordering);
     pdfi_countdown(pdfcidf->filename);
+    pdfi_countdown(pdfcidf->copyright);
+    pdfi_countdown(pdfcidf->notice);
+    pdfi_countdown(pdfcidf->fullname);
+    pdfi_countdown(pdfcidf->familyname);
 
     gs_free_object(OBJ_MEMORY(pdfcidf), pdfcidf, "pdfi_free_font_cidtype2(pdfcidf)");
 return 0;
