@@ -362,7 +362,7 @@ pdfi_alloc_tt_font(pdf_context *ctx, pdf_font_truetype **font, bool is_cid)
 
 #if REFCNT_DEBUG
     ttfont->UID = ctx->UID++;
-    dmprintf2(ctx->memory, "Allocated object of type %c with UID %"PRIi64"\n", ttfont->type, ttfont->UID);
+    outprintf(ctx->memory, "Allocated object of type %c with UID %"PRIi64"\n", ttfont->type, ttfont->UID);
 #endif
 
     pdfi_countup(ttfont);
@@ -855,9 +855,11 @@ pdfi_copy_truetype_font(pdf_context *ctx, pdf_font *spdffont, pdf_dict *font_dic
     if (code < 0) {
         uid_set_invalid(&font->pfont->UID);
     }
-    code = pdfi_font_generate_pseudo_XUID(ctx, font_dict, font->pfont);
-    if (code < 0) {
-        goto error;
+    if (spdffont->filename == NULL) {
+        code = pdfi_font_generate_pseudo_XUID(ctx, font_dict, font->pfont);
+        if (code < 0) {
+            goto error;
+        }
     }
     if (ctx->args.ignoretounicode != true) {
         code = pdfi_dict_get(ctx, font_dict, "ToUnicode", (pdf_obj **)&tmp);
