@@ -1,4 +1,4 @@
-/* Copyright (C) 2018-2022 Artifex Software, Inc.
+/* Copyright (C) 2018-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Shading operations for the PDF interpreter */
@@ -153,7 +153,7 @@ static int pdfi_shading1(pdf_context *ctx, gs_shading_params_t *pcommon,
     }
 
     code = fill_matrix_from_dict(ctx, (float *)&params.Matrix, shading_dict);
-    if (code < 0)
+    if (code < 0 && code != gs_error_undefined)
         return code;
 
     code = pdfi_build_shading_function(ctx, &params.Function, (const float *)&params.Domain, 2, (pdf_dict *)shading_dict, page_dict);
@@ -163,7 +163,8 @@ static int pdfi_shading1(pdf_context *ctx, gs_shading_params_t *pcommon,
     }
     code = gs_shading_Fb_init(ppsh, &params, ctx->memory);
     if (code < 0) {
-        gs_free_object(ctx->memory, params.Function, "Function");
+        gs_function_free(params.Function, true, ctx->memory);
+        params.Function = NULL;
         pdfi_countdown(o);
     }
     return code;
@@ -214,6 +215,8 @@ static int pdfi_shading2(pdf_context *ctx, gs_shading_params_t *pcommon,
     }
     code = gs_shading_A_init(ppsh, &params, ctx->memory);
     if (code < 0){
+        gs_function_free(params.Function, true, ctx->memory);
+        params.Function = NULL;
         pdfi_countdown(o);
         return code;
     }
@@ -266,6 +269,8 @@ static int pdfi_shading3(pdf_context *ctx,  gs_shading_params_t *pcommon,
     }
     code = gs_shading_R_init(ppsh, &params, ctx->memory);
     if (code < 0){
+        gs_function_free(params.Function, true, ctx->memory);
+        params.Function = NULL;
         pdfi_countdown(o);
         return code;
     }
@@ -277,9 +282,6 @@ static int pdfi_build_mesh_shading(pdf_context *ctx, gs_shading_mesh_params_t *p
                   pdf_obj *Shading, pdf_dict *stream_dict, pdf_dict *page_dict)
 {
     int num_decode = 4, code;
-    gs_offset_t savedoffset;
-    gs_offset_t stream_offset;
-    int64_t Length;
     byte *data_source_buffer = NULL;
     pdf_c_stream *shading_stream = NULL;
     int64_t i;
@@ -295,20 +297,8 @@ static int pdfi_build_mesh_shading(pdf_context *ctx, gs_shading_mesh_params_t *p
     params->Function = NULL;
     params->Decode = NULL;
 
-    stream_offset = pdfi_stream_offset(ctx, (pdf_stream *)Shading);
-    if (stream_offset == 0)
-        return_error(gs_error_typecheck);
-
-    Length = pdfi_stream_length(ctx, (pdf_stream *)Shading);
-
-    savedoffset = pdfi_tell(ctx->main_stream);
-    code = pdfi_seek(ctx, ctx->main_stream, stream_offset, SEEK_SET);
-    if (code < 0)
-        return code;
-
-    code = pdfi_open_memory_stream_from_filtered_stream(ctx, (pdf_stream *)Shading, Length, &data_source_buffer, ctx->main_stream, &shading_stream, false);
+    code = pdfi_open_memory_stream_from_filtered_stream(ctx, (pdf_stream *)Shading, &data_source_buffer, &shading_stream, false);
     if (code < 0) {
-        pdfi_seek(ctx, ctx->main_stream, savedoffset, SEEK_SET);
         return code;
     }
 
@@ -319,10 +309,6 @@ static int pdfi_build_mesh_shading(pdf_context *ctx, gs_shading_mesh_params_t *p
      */
     gs_free_object(ctx->memory, shading_stream, "discard memory stream(pdf_stream)");
 
-    code = pdfi_seek(ctx, ctx->main_stream, savedoffset, SEEK_SET);
-    if (code < 0)
-        goto build_mesh_shading_error;
-
     code = pdfi_build_shading_function(ctx, &params->Function, (const float *)NULL, 1,
                                        shading_dict, page_dict);
     if (code < 0 && code != gs_error_undefined)
@@ -411,7 +397,8 @@ static int pdfi_shading4(pdf_context *ctx, gs_shading_params_t *pcommon,
 
     code = gs_shading_FfGt_init(ppsh, &params, ctx->memory);
     if (code < 0) {
-        gs_free_object(ctx->memory, params.Function, "Function");
+        gs_function_free(params.Function, true, ctx->memory);
+        params.Function = NULL;
         gs_free_object(ctx->memory, params.Decode, "Decode");
         return code;
     }
@@ -450,7 +437,8 @@ static int pdfi_shading5(pdf_context *ctx, gs_shading_params_t *pcommon,
 
     code = gs_shading_LfGt_init(ppsh, &params, ctx->memory);
     if (code < 0) {
-        gs_free_object(ctx->memory, params.Function, "Function");
+        gs_function_free(params.Function, true, ctx->memory);
+        params.Function = NULL;
         gs_free_object(ctx->memory, params.Decode, "Decode");
         return code;
     }
@@ -489,7 +477,8 @@ static int pdfi_shading6(pdf_context *ctx, gs_shading_params_t *pcommon,
 
     code = gs_shading_Cp_init(ppsh, &params, ctx->memory);
     if (code < 0) {
-        gs_free_object(ctx->memory, params.Function, "Function");
+        gs_function_free(params.Function, true, ctx->memory);
+        params.Function = NULL;
         gs_free_object(ctx->memory, params.Decode, "Decode");
         return code;
     }
@@ -528,7 +517,8 @@ static int pdfi_shading7(pdf_context *ctx, gs_shading_params_t *pcommon,
 
     code = gs_shading_Tpp_init(ppsh, &params, ctx->memory);
     if (code < 0) {
-        gs_free_object(ctx->memory, params.Function, "Function");
+        gs_function_free(params.Function, true, ctx->memory);
+        params.Function = NULL;
         gs_free_object(ctx->memory, params.Decode, "Decode");
         return code;
     }
@@ -571,7 +561,11 @@ static int get_shading_common(pdf_context *ctx, pdf_dict *shading_dict, gs_shadi
         pcc->pattern = 0;
         params->Background = pcc;
 
-        temp = (double *)gs_alloc_bytes(ctx->memory, num_comp * sizeof(double), "temporary array of doubles");
+        temp = (double *)gs_alloc_bytes(ctx->memory, (size_t)num_comp * sizeof(double), "temporary array of doubles");
+        if (temp == NULL) {
+            code = gs_error_VMerror;
+            goto get_shading_common_error;
+        }
         for(i=0;i<num_comp;i++) {
             code = pdfi_array_get_number(ctx, a, i, &temp[i]);
             if (code < 0) {
@@ -874,8 +868,8 @@ int pdfi_shading(pdf_context *ctx, pdf_dict *stream_dict, pdf_dict *page_dict)
     pdfi_pop(ctx, 1);
 
     if (pdfi_type_of(n) != PDF_NAME) {
-        code = gs_note_error(gs_error_typecheck);
-        goto exit1;
+        pdfi_countdown(n);
+        return gs_note_error(gs_error_typecheck);
     }
 
     code = pdfi_loop_detector_mark(ctx);
@@ -921,7 +915,7 @@ int pdfi_shading(pdf_context *ctx, pdf_dict *stream_dict, pdf_dict *page_dict)
 
     code = gs_shfill(ctx->pgs, psh);
     if (code < 0) {
-        pdfi_set_warning(ctx, 0, NULL, W_PDF_BADSHADING, "pdfi_rectpath", (char *)"ERROR: ignoring invalid smooth shading object, output may be incorrect");
+        pdfi_set_warning(ctx, 0, NULL, W_PDF_BADSHADING, "pdfi_shading", (char *)"ERROR: ignoring invalid smooth shading object, output may be incorrect");
         code = 0;
     }
 
