@@ -1,4 +1,4 @@
-/* Copyright (C) 2018-2022 Artifex Software, Inc.
+/* Copyright (C) 2018-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* xref parsing */
@@ -36,7 +36,7 @@ static int resize_xref(pdf_context *ctx, uint64_t new_size)
     if (new_size >= (0x7ffffff / sizeof(xref_entry)))
         return_error(gs_error_rangecheck);
 
-    new_xrefs = (xref_entry *)gs_alloc_bytes(ctx->memory, (new_size) * sizeof(xref_entry), "read_xref_stream allocate xref table entries");
+    new_xrefs = (xref_entry *)gs_alloc_bytes(ctx->memory, (size_t)(new_size) * sizeof(xref_entry), "read_xref_stream allocate xref table entries");
     if (new_xrefs == NULL){
         pdfi_countdown(ctx->xref_table);
         ctx->xref_table = NULL;
@@ -50,10 +50,10 @@ static int resize_xref(pdf_context *ctx, uint64_t new_size)
     return 0;
 }
 
-static int read_xref_stream_entries(pdf_context *ctx, pdf_c_stream *s, uint64_t first, uint64_t last, uint64_t *W)
+static int read_xref_stream_entries(pdf_context *ctx, pdf_c_stream *s, int64_t first, int64_t last, int64_t *W)
 {
     uint i, j;
-    uint field_width = 0;
+    uint64_t field_width = 0;
     uint32_t type = 0;
     uint64_t objnum = 0, gen = 0;
     byte *Buffer;
@@ -68,6 +68,9 @@ static int read_xref_stream_entries(pdf_context *ctx, pdf_c_stream *s, uint64_t
         field_width = W[2];
 
     Buffer = gs_alloc_bytes(ctx->memory, field_width, "read_xref_stream_entry working buffer");
+    if (Buffer == NULL)
+        return_error(gs_error_VMerror);
+
     for (i=first;i<=last; i++){
         /* Defaults if W[n] = 0 */
         type = 1;
@@ -105,7 +108,7 @@ static int read_xref_stream_entries(pdf_context *ctx, pdf_c_stream *s, uint64_t
         }
 
         entry = &ctx->xref_table->xref[i];
-        if (entry->object_num != 0)
+        if (entry->object_num != 0 && !entry->free)
             continue;
 
         entry->compressed = false;
@@ -140,6 +143,7 @@ static int read_xref_stream_entries(pdf_context *ctx, pdf_c_stream *s, uint64_t
 
 /* Forward definition */
 static int read_xref(pdf_context *ctx, pdf_c_stream *s);
+static int pdfi_check_xref_stream(pdf_context *ctx);
 /* These two routines are recursive.... */
 static int pdfi_read_xref_stream_dict(pdf_context *ctx, pdf_c_stream *s, int obj_num);
 
@@ -152,7 +156,7 @@ static int pdfi_process_xref_stream(pdf_context *ctx, pdf_stream *stream_obj, pd
     pdf_array *a;
     int64_t size;
     int64_t num;
-    int64_t W[3];
+    int64_t W[3] = {0, 0, 0};
     int objnum;
     bool known = false;
 
@@ -189,7 +193,7 @@ static int pdfi_process_xref_stream(pdf_context *ctx, pdf_stream *stream_obj, pd
             return_error(gs_error_VMerror);
         }
         memset(ctx->xref_table, 0x00, sizeof(xref_table_t));
-        ctx->xref_table->xref = (xref_entry *)gs_alloc_bytes(ctx->memory, size * sizeof(xref_entry), "read_xref_stream allocate xref table entries");
+        ctx->xref_table->xref = (xref_entry *)gs_alloc_bytes(ctx->memory, (size_t)size * sizeof(xref_entry), "read_xref_stream allocate xref table entries");
         if (ctx->xref_table->xref == NULL){
             gs_free_object(ctx->memory, ctx->xref_table, "failed to allocate xref table entries");
             ctx->xref_table = NULL;
@@ -201,10 +205,12 @@ static int pdfi_process_xref_stream(pdf_context *ctx, pdf_stream *stream_obj, pd
         ctx->xref_table->xref_size = size;
 #if REFCNT_DEBUG
         ctx->xref_table->UID = ctx->ref_UID++;
-        dmprintf1(ctx->memory, "Allocated xref table with UID %"PRIi64"\n", ctx->xref_table->UID);
+        outprintf(ctx->memory, "Allocated xref table with UID %"PRIi64"\n", ctx->xref_table->UID);
 #endif
         pdfi_countup(ctx->xref_table);
 
+        pdfi_countdown(ctx->Trailer);
+
         ctx->Trailer = sdict;
         pdfi_countup(sdict);
     } else {
@@ -212,9 +218,8 @@ static int pdfi_process_xref_stream(pdf_context *ctx, pdf_stream *stream_obj, pd
             return_error(gs_error_rangecheck);
 
         code = pdfi_merge_dicts(ctx, ctx->Trailer, sdict);
-        if (code < 0) {
-            if (code == gs_error_VMerror || ctx->args.pdfstoponerror)
-                return code;
+        if (code < 0 && (code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREF, "pdfi_process_xref_stream", NULL)) < 0) {
+            goto exit;
         }
     }
 
@@ -288,19 +293,39 @@ static int pdfi_process_xref_stream(pdf_context *ctx, pdf_stream *stream_obj, pd
     }
     for (i=0;i<3;i++) {
         code = pdfi_array_get_int(ctx, a, (uint64_t)i, (int64_t *)&W[i]);
-        if (code < 0) {
+        if (code < 0 || W[i] < 0) {
             pdfi_countdown(a);
             pdfi_close_file(ctx, XRefStrm);
             pdfi_countdown(ctx->xref_table);
             ctx->xref_table = NULL;
+            if (W[i] < 0)
+                code = gs_note_error(gs_error_rangecheck);
             return code;
         }
     }
     pdfi_countdown(a);
 
+    /* W[0] is either:
+     * 0 (no type field) or a single byte with the type.
+     * W[1] is either:
+     * The object number of the next free object, the byte offset of this object in the file or the object5 number of the object stream where this object is stored.
+     * W[2] is either:
+     * The generation number to use if this object is used again, the generation number of the object or the index of this object within the object stream.
+     *
+     * Object and generation numbers are limited to unsigned 64-bit values, as are bytes offsets in the file, indexes of objects within the stream likewise (actually
+     * most of these are generally 32-bit max). So we can limit the field widths to 8 bytes, enough to hold a 64-bit number.
+     * Even if a later version of the spec makes these larger (which seems unlikely!) we still cna't cope with integers > 64-bits.
+     */
+    if (W[0] > 1 || W[1] > 8 || W[2] > 8) {
+        pdfi_close_file(ctx, XRefStrm);
+        pdfi_countdown(ctx->xref_table);
+        ctx->xref_table = NULL;
+        return code;
+    }
+
     code = pdfi_dict_get_type(ctx, sdict, "Index", PDF_ARRAY, (pdf_obj **)&a);
     if (code == gs_error_undefined) {
-        code = read_xref_stream_entries(ctx, XRefStrm, 0, size - 1, (uint64_t *)W);
+        code = read_xref_stream_entries(ctx, XRefStrm, 0, size - 1, W);
         if (code < 0) {
             pdfi_close_file(ctx, XRefStrm);
             pdfi_countdown(ctx->xref_table);
@@ -358,7 +383,7 @@ static int pdfi_process_xref_stream(pdf_context *ctx, pdf_stream *stream_obj, pd
                 }
             }
 
-            code = read_xref_stream_entries(ctx, XRefStrm, start, start + size - 1, (uint64_t *)W);
+            code = read_xref_stream_entries(ctx, XRefStrm, start, start + size - 1, W);
             if (code < 0) {
                 pdfi_countdown(a);
                 pdfi_close_file(ctx, XRefStrm);
@@ -391,38 +416,77 @@ static int pdfi_process_xref_stream(pdf_context *ctx, pdf_stream *stream_obj, pd
     }
 
     if(ctx->args.pdfdebug)
-        dmprintf(ctx->memory, "%% Reading /Prev xref\n");
+        outprintf(ctx->memory, "%% Reading /Prev xref\n");
 
     pdfi_seek(ctx, s, num, SEEK_SET);
 
     code = pdfi_read_bare_int(ctx, ctx->main_stream, &objnum);
-    if (code == 1)
-        return pdfi_read_xref_stream_dict(ctx, s, objnum);
+    if (code == 1) {
+        if (pdfi_check_xref_stream(ctx))
+            return pdfi_read_xref_stream_dict(ctx, s, objnum);
+    }
 
     code = pdfi_read_bare_keyword(ctx, ctx->main_stream);
     if (code < 0)
         return code;
     if (code == TOKEN_XREF) {
-        pdfi_set_error(ctx, 0, NULL, E_PDF_PREV_NOT_XREF_STREAM, "pdfi_process_xref_stream", NULL);
-        if (!ctx->args.pdfstoponerror)
-            /* Read old-style xref table */
-            return(read_xref(ctx, ctx->main_stream));
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_PREV_NOT_XREF_STREAM, "pdfi_process_xref_stream", NULL)) < 0) {
+            goto exit;
+        }
+        /* Read old-style xref table */
+        return(read_xref(ctx, ctx->main_stream));
     }
+exit:
     return_error(gs_error_syntaxerror);
 }
 
+static int pdfi_check_xref_stream(pdf_context *ctx)
+{
+    gs_offset_t offset;
+    int gen_num, code = 0;
+
+    offset = pdfi_unread_tell(ctx);
+
+    code = pdfi_read_bare_int(ctx, ctx->main_stream, &gen_num);
+    if (code <= 0) {
+        code = 0;
+        goto exit;
+    }
+
+    /* Try to read 'obj' */
+    code = pdfi_read_bare_keyword(ctx, ctx->main_stream);
+    if (code <= 0) {
+        code = 0;
+        goto exit;
+    }
+
+    /* Third element must be obj, or it's not a valid xref */
+    if (code != TOKEN_OBJ)
+        code = 0;
+    else
+        code = 1;
+
+exit:
+    pdfi_seek(ctx, ctx->main_stream, offset, SEEK_SET);
+    return code;
+}
+
 static int pdfi_read_xref_stream_dict(pdf_context *ctx, pdf_c_stream *s, int obj_num)
 {
     int code;
     int gen_num;
 
     if (ctx->args.pdfdebug)
-        dmprintf(ctx->memory, "\n%% Reading PDF 1.5+ xref stream\n");
+        outprintf(ctx->memory, "\n%% Reading PDF 1.5+ xref stream\n");
 
     /* We have the obj_num. Lets try for obj_num gen obj as a XRef stream */
     code = pdfi_read_bare_int(ctx, ctx->main_stream, &gen_num);
-    if (code <= 0)
+    if (code <= 0) {
+        if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREFSTREAM, "pdfi_read_xref_stream_dict", "")) < 0) {
+            return code;
+        }
         return(pdfi_repair_file(ctx));
+    }
 
     /* Try to read 'obj' */
     code = pdfi_read_bare_keyword(ctx, ctx->main_stream);
@@ -432,13 +496,21 @@ static int pdfi_read_xref_stream_dict(pdf_context *ctx, pdf_c_stream *s, int obj
         return_error(gs_error_syntaxerror);
 
     /* Third element must be obj, or it's not a valid xref */
-    if (code != TOKEN_OBJ)
+    if (code != TOKEN_OBJ) {
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_rangecheck), NULL, E_PDF_BAD_XREFSTMOFFSET, "pdfi_read_xref_stream_dict", "")) < 0) {
+            return code;
+        }
         return(pdfi_repair_file(ctx));
+    }
 
     do {
         code = pdfi_read_token(ctx, ctx->main_stream, obj_num, gen_num);
-        if (code <= 0)
+        if (code <= 0) {
+            if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREFSTREAM, "pdfi_read_xref_stream_dict", NULL)) < 0) {
+                return code;
+            }
             return pdfi_repair_file(ctx);
+        }
 
         if (pdfi_count_stack(ctx) >= 2 && pdfi_type_of(ctx->stack_top[-1]) == PDF_FAST_KEYWORD) {
             uintptr_t keyword = (uintptr_t)ctx->stack_top[-1];
@@ -450,6 +522,9 @@ static int pdfi_read_xref_stream_dict(pdf_context *ctx, pdf_c_stream *s, int obj
                 /* Remove the 'stream' token from the stack, should leave a dictionary object on the stack */
                 pdfi_pop(ctx, 1);
                 if (pdfi_type_of(ctx->stack_top[-1]) != PDF_DICT) {
+                    if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREFSTREAM, "pdfi_read_xref_stream_dict", NULL)) < 0) {
+                        return code;
+                    }
                     return pdfi_repair_file(ctx);
                 }
                 dict = (pdf_dict *)ctx->stack_top[-1];
@@ -459,6 +534,9 @@ static int pdfi_read_xref_stream_dict(pdf_context *ctx, pdf_c_stream *s, int obj
                 /* Pop off the dict */
                 pdfi_pop(ctx, 1);
                 if (code < 0) {
+                    if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREFSTREAM, "pdfi_read_xref_stream_dict", NULL)) < 0) {
+                        return code;
+                    }
                     /* TODO: should I return code instead of trying to repair?
                      * Normally the above routine should not fail so something is
                      * probably seriously fubar.
@@ -475,7 +553,7 @@ static int pdfi_read_xref_stream_dict(pdf_context *ctx, pdf_c_stream *s, int obj
                 code = pdfi_dict_get_int(ctx, sdict->stream_dict, "Length", &Length);
                 if (code < 0) {
                     /* TODO: Not positive this will actually have a length -- just use 0 */
-                    pdfi_set_error_var(ctx, 0, NULL, E_PDF_BADSTREAM, "pdfi_read_xref_stream_dict", "Xref Stream object %u missing mandatory keyword /Length\n", obj_num);
+                    (void)pdfi_set_error_var(ctx, 0, NULL, E_PDF_BADSTREAM, "pdfi_read_xref_stream_dict", "Xref Stream object %u missing mandatory keyword /Length\n", obj_num);
                     code = 0;
                     Length = 0;
                 }
@@ -483,14 +561,17 @@ static int pdfi_read_xref_stream_dict(pdf_context *ctx, pdf_c_stream *s, int obj
                 sdict->length_valid = true;
 
                 code = pdfi_process_xref_stream(ctx, sdict, ctx->main_stream);
-                if (code < 0) {
-                    pdfi_countdown(sdict);
-                    return (pdfi_repair_file(ctx));
-                }
                 pdfi_countdown(sdict);
+                if (code < 0) {
+                    pdfi_set_error(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_PREV_NOT_XREF_STREAM, "pdfi_read_xref_stream_dict", NULL);
+                    return code;
+                }
                 break;
             } else if (keyword == TOKEN_ENDOBJ) {
                 /* Something went wrong, this is not a stream dictionary */
+                if ((code = pdfi_set_error_var(ctx, 0, NULL, E_PDF_BADSTREAM, "pdfi_read_xref_stream_dict", "Xref Stream object %u missing mandatory keyword /Length\n", obj_num)) < 0) {
+                    return code;
+                }
                 return(pdfi_repair_file(ctx));
             }
         }
@@ -649,7 +730,7 @@ static int read_xref_section(pdf_context *ctx, pdf_c_stream *s, uint64_t *sectio
     *section_start = *section_size = 0;
 
     if (ctx->args.pdfdebug)
-        dmprintf(ctx->memory, "\n%% Reading xref section\n");
+        outprintf(ctx->memory, "\n%% Reading xref section\n");
 
     code = pdfi_read_bare_int(ctx, ctx->main_stream, &start);
     if (code < 0) {
@@ -684,7 +765,7 @@ static int read_xref_section(pdf_context *ctx, pdf_c_stream *s, uint64_t *sectio
     *section_size = size;
 
     if (ctx->args.pdfdebug)
-        dmprintf2(ctx->memory, "\n%% Section starts at %d and has %d entries\n", (unsigned int) start, (unsigned int)size);
+        outprintf(ctx->memory, "\n%% Section starts at %d and has %d entries\n", (unsigned int) start, (unsigned int)size);
 
     if (size > 0) {
         if (ctx->xref_table == NULL) {
@@ -693,7 +774,7 @@ static int read_xref_section(pdf_context *ctx, pdf_c_stream *s, uint64_t *sectio
                 return_error(gs_error_VMerror);
             memset(ctx->xref_table, 0x00, sizeof(xref_table_t));
 
-            ctx->xref_table->xref = (xref_entry *)gs_alloc_bytes(ctx->memory, (start + size) * sizeof(xref_entry), "read_xref_stream allocate xref table entries");
+            ctx->xref_table->xref = (xref_entry *)gs_alloc_bytes(ctx->memory, ((size_t)start + (size_t)size) * (size_t)sizeof(xref_entry), "read_xref_stream allocate xref table entries");
             if (ctx->xref_table->xref == NULL){
                 gs_free_object(ctx->memory, ctx->xref_table, "free xref table on error allocating entries");
                 ctx->xref_table = NULL;
@@ -701,7 +782,7 @@ static int read_xref_section(pdf_context *ctx, pdf_c_stream *s, uint64_t *sectio
             }
 #if REFCNT_DEBUG
             ctx->xref_table->UID = ctx->ref_UID++;
-            dmprintf1(ctx->memory, "Allocated xref table with UID %"PRIi64"\n", ctx->xref_table->UID);
+            outprintf(ctx->memory, "Allocated xref table with UID %"PRIi64"\n", ctx->xref_table->UID);
 #endif
 
             memset(ctx->xref_table->xref, 0x00, (start + size) * sizeof(xref_entry));
@@ -735,7 +816,7 @@ static int read_xref_section(pdf_context *ctx, pdf_c_stream *s, uint64_t *sectio
             pdfi_unread_byte(ctx, s, (byte)Buffer[j]);
             if (--j < 0) {
                 pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_XREF_ENTRY_NO_EOL, "read_xref_section", NULL);
-                dmprintf(ctx->memory, "Invalid xref entry, line terminator missing.\n");
+                outprintf(ctx->memory, "Invalid xref entry, line terminator missing.\n");
                 code = read_xref_entry_slow(ctx, s, &off, &gen, &free);
                 if (code < 0)
                     return code;
@@ -752,7 +833,7 @@ static int read_xref_section(pdf_context *ctx, pdf_c_stream *s, uint64_t *sectio
 
         if (sscanf(Buffer, "%"PRIdOFFSET" %d %c", &entry->u.uncompressed.offset, &entry->u.uncompressed.generation_num, &free) != 3) {
             pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_XREF_ENTRY_FORMAT, "read_xref_section", NULL);
-            dmprintf(ctx->memory, "Invalid xref entry, incorrect format.\n");
+            outprintf(ctx->memory, "Invalid xref entry, incorrect format.\n");
             pdfi_unread(ctx, s, (byte *)Buffer, 20);
             code = read_xref_entry_slow(ctx, s, &off, &gen, &free);
             if (code < 0)
@@ -768,6 +849,11 @@ static int read_xref_section(pdf_context *ctx, pdf_c_stream *s, uint64_t *sectio
             entry->free = true;
         if(free == 'n')
             entry->free = false;
+        if (entry->object_num == 0) {
+            if (!entry->free) {
+                pdfi_set_warning(ctx, 0, NULL, W_PDF_XREF_OBJECT0_NOT_FREE, "read_xref_section", NULL);
+            }
+        }
     }
 
     return 0;
@@ -777,9 +863,10 @@ static int read_xref(pdf_context *ctx, pdf_c_stream *s)
 {
     int code = 0;
     pdf_dict *d = NULL;
-    uint64_t size = 0, max_obj = 0;
-    int64_t num;
+    uint64_t max_obj = 0;
+    int64_t num, XRefStm = 0;
     int obj_num;
+    bool known = false;
 
     if (ctx->repaired)
         return 0;
@@ -806,6 +893,23 @@ static int read_xref(pdf_context *ctx, pdf_c_stream *s)
         pdfi_pop(ctx, 1);
         return_error(gs_error_typecheck);
     }
+    pdfi_countup(d);
+    pdfi_pop(ctx, 1);
+
+    /* We don't want to pollute the Trailer dictionary with any XRefStm key/value pairs
+     * which will happen when we do pdfi_merge_dicts(). So we get any XRefStm here and
+     * if there was one, remove it from the dictionary before we merge with the
+     * primary trailer.
+     */
+    code = pdfi_dict_get_int(ctx, d, "XRefStm", &XRefStm);
+    if (code < 0 && code != gs_error_undefined)
+        goto error;
+
+    if (code == 0) {
+        code = pdfi_dict_delete(ctx, d, "XRefStm");
+        if (code < 0)
+            goto error;
+    }
 
     if (ctx->Trailer == NULL) {
         ctx->Trailer = d;
@@ -813,173 +917,126 @@ static int read_xref(pdf_context *ctx, pdf_c_stream *s)
     } else {
         code = pdfi_merge_dicts(ctx, ctx->Trailer, d);
         if (code < 0) {
-            if (code == gs_error_VMerror || ctx->args.pdfstoponerror) {
-                pdfi_pop(ctx, 1);
+            if ((code = pdfi_set_error_stop(ctx, code, NULL, E_PDF_BADXREF, "read_xref", "")) < 0) {
                 return code;
             }
         }
     }
 
-    /* We have the Trailer dictionary. First up check for hybrid files. These have the initial
-     * xref starting at 0 and size of 0. In this case the /Size entry in the trailer dictionary
-     * must tell us how large the xref is, and we need to allocate our xref table anyway.
+    /* Check if the highest subsection + size exceeds the /Size in the
+     * trailer dictionary and set a warning flag if it does
      */
-    if (ctx->xref_table == NULL && size == 0) {
-        int64_t size;
+    code = pdfi_dict_get_int(ctx, d, "Size", &num);
+    if (code < 0)
+        goto error;
 
-        code = pdfi_dict_get_int(ctx, d, "Size", &size);
-        if (code < 0) {
-            pdfi_pop(ctx, 1);
-            return code;
+    if (max_obj >= num)
+        pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_XREF_SIZE, "read_xref", NULL);
+
+    /* Check if this is a modified file and has any
+     * previous xref entries.
+     */
+    code = pdfi_dict_known(ctx, d, "Prev", &known);
+    if (known) {
+        code = pdfi_dict_get_int(ctx, d, "Prev", &num);
+        if (code < 0)
+            goto error;
+
+        if (num < 0 || num > ctx->main_stream_length) {
+            code = gs_note_error(gs_error_rangecheck);
+            goto error;
         }
-        if (size < 0 || size > floor((double)ARCH_MAX_SIZE_T / (double)sizeof(xref_entry))) {
-            pdfi_pop(ctx, 1);
-            return_error(gs_error_rangecheck);
-        }
-
-        ctx->xref_table = (xref_table_t *)gs_alloc_bytes(ctx->memory, sizeof(xref_table_t), "read_xref_stream allocate xref table");
-        if (ctx->xref_table == NULL) {
-            pdfi_pop(ctx, 1);
-            return_error(gs_error_VMerror);
-        }
-        memset(ctx->xref_table, 0x00, sizeof(xref_table_t));
-#if REFCNT_DEBUG
-        ctx->xref_table->UID = ctx->ref_UID++;
-        dmprintf1(ctx->memory, "Allocated xref table with UID %"PRIi64"\n", ctx->xref_table->UID);
-#endif
-
-        ctx->xref_table->xref = (xref_entry *)gs_alloc_bytes(ctx->memory, size * sizeof(xref_entry), "read_xref_stream allocate xref table entries");
-        if (ctx->xref_table->xref == NULL){
-            pdfi_pop(ctx, 1);
-            pdfi_countdown(ctx->xref_table);
-            ctx->xref_table = NULL;
-            return_error(gs_error_VMerror);
-        }
-
-        memset(ctx->xref_table->xref, 0x00, size * sizeof(xref_entry));
-        ctx->xref_table->ctx = ctx;
-        ctx->xref_table->type = PDF_XREF_TABLE;
-        ctx->xref_table->xref_size = size;
-        pdfi_countup(ctx->xref_table);
-    }
-
-    /* Now check if this is a hybrid file. */
-    if (ctx->Trailer == d) {
-        code = pdfi_dict_get_int(ctx, d, "XRefStm", &num);
-        if (code < 0 && code != gs_error_undefined) {
-            pdfi_pop(ctx, 1);
-            return code;
-        }
-        if (code == 0)
-            ctx->is_hybrid = true;
-    } else
-        code = gs_error_undefined;
-
-    if (code == 0 && ctx->prefer_xrefstm) {
-        if (ctx->args.pdfdebug)
-            dmprintf(ctx->memory, "%% File is a hybrid, containing xref table and xref stream. Using the stream.\n");
-
 
         if (pdfi_loop_detector_check_object(ctx, num) == true) {
-            pdfi_pop(ctx, 1);
-            return_error(gs_error_circular_reference);
+            code = gs_note_error(gs_error_circular_reference);
+            goto error;
         }
         else {
             code = pdfi_loop_detector_add_object(ctx, num);
-            if (code < 0) {
-                pdfi_pop(ctx, 1);
-                return code;
+            if (code < 0)
+                goto error;
+        }
+
+        code = pdfi_seek(ctx, s, num, SEEK_SET);
+        if (code < 0)
+            goto error;
+
+        if (!ctx->repaired) {
+            code = pdfi_read_token(ctx, ctx->main_stream, 0, 0);
+            if (code < 0)
+                goto error;
+
+            if (code == 0) {
+                code = gs_note_error(gs_error_syntaxerror);
+                goto error;
             }
+        } else {
+            code = 0;
+            goto error;
+        }
+
+        if ((intptr_t)(ctx->stack_top[-1]) == (intptr_t)TOKEN_XREF) {
+            /* Read old-style xref table */
+            pdfi_pop(ctx, 1);
+            code = read_xref(ctx, ctx->main_stream);
+            if (code < 0)
+                goto error;
+        } else {
+            pdfi_pop(ctx, 1);
+            code = gs_note_error(gs_error_typecheck);
+            goto error;
+        }
+    }
+
+    /* Now check if this is a hybrid file. */
+    if (XRefStm != 0) {
+        ctx->is_hybrid = true;
+
+        if (ctx->args.pdfdebug)
+            outprintf(ctx->memory, "%% File is a hybrid, containing xref table and xref stream. Reading the stream.\n");
+
+
+        if (pdfi_loop_detector_check_object(ctx, XRefStm) == true) {
+            code = gs_note_error(gs_error_circular_reference);
+            goto error;
+        }
+        else {
+            code = pdfi_loop_detector_add_object(ctx, XRefStm);
+            if (code < 0)
+                goto error;
         }
 
         code = pdfi_loop_detector_mark(ctx);
-        if (code < 0) {
-            pdfi_pop(ctx, 1);
-            return code;
-        }
+        if (code < 0)
+            goto error;
+
         /* Because of the way the code works when we read a file which is a pure
          * xref stream file, we need to read the first integer of 'x y obj'
          * because the xref stream decoding code expects that to be on the stack.
          */
-        pdfi_seek(ctx, s, num, SEEK_SET);
+        pdfi_seek(ctx, s, XRefStm, SEEK_SET);
 
         code = pdfi_read_bare_int(ctx, ctx->main_stream, &obj_num);
         if (code < 0) {
+            pdfi_set_error(ctx, 0, NULL, E_PDF_BADXREFSTREAM, "read_xref", "");
             pdfi_loop_detector_cleartomark(ctx);
-            pdfi_pop(ctx, 1);
-            return code;
+            goto error;
         }
 
         code = pdfi_read_xref_stream_dict(ctx, ctx->main_stream, obj_num);
+        /* We could just fall through to the exit here, but choose not to in order to avoid possible mistakes in future */
         if (code < 0) {
             pdfi_loop_detector_cleartomark(ctx);
-            pdfi_pop(ctx, 1);
-            return code;
+            goto error;
         }
 
-        /* This can happen if pdfi_read_xref_stream tries to repair a broken PDF file */
-        if (d != ctx->Trailer)
-            d = ctx->Trailer;
-
         pdfi_loop_detector_cleartomark(ctx);
-    }
-
-    /* Not a hybrid file, so now check if this is a modified file and has
-     * previous xref entries.
-     */
-    /* But first, check if the highest subsection + size exceeds the /Size in the
-     * trailer dictionary and set a warning flag if it does
-     */
-    code = pdfi_dict_get_int(ctx, d, "Size", &num);
-    if (code < 0) {
-        pdfi_pop(ctx, 1);
-        return code;
-    }
-    if (max_obj > num)
-        pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_XREF_SIZE, "read_xref", NULL);
-
-    code = pdfi_dict_get_int(ctx, d, "Prev", &num);
-    if (code < 0) {
-        pdfi_pop(ctx, 1);
-        if (code == gs_error_undefined)
-            return 0;
-        else
-            return code;
-    }
-    pdfi_pop(ctx, 1);
-
-    if (num < 0 || num > ctx->main_stream_length)
-        return_error(gs_error_rangecheck);
-
-    if (pdfi_loop_detector_check_object(ctx, num) == true)
-        return_error(gs_error_circular_reference);
-    else {
-        code = pdfi_loop_detector_add_object(ctx, num);
-        if (code < 0)
-            return code;
-    }
-
-    code = pdfi_seek(ctx, s, num, SEEK_SET);
-    if (code < 0)
-        return code;
-
-    if (!ctx->repaired) {
-        code = pdfi_read_token(ctx, ctx->main_stream, 0, 0);
-        if (code < 0)
-            return(code);
-        if (code == 0)
-        return_error(gs_error_syntaxerror);
     } else
-        return 0;
+        code = 0;
 
-    if ((intptr_t)(ctx->stack_top[-1]) == (intptr_t)TOKEN_XREF) {
-        /* Read old-style xref table */
-        pdfi_pop(ctx, 1);
-        return(read_xref(ctx, ctx->main_stream));
-    } else {
-        pdfi_pop(ctx, 1);
-        return_error(gs_error_typecheck);
-    }
+error:
+    pdfi_countdown(d);
+    return code;
 }
 
 int pdfi_read_xref(pdf_context *ctx)
@@ -999,14 +1056,18 @@ int pdfi_read_xref(pdf_context *ctx)
         goto exit;
 
     if (ctx->args.pdfdebug)
-        dmprintf(ctx->memory, "%% Trying to read 'xref' token for xref table, or 'int int obj' for an xref stream\n");
+        outprintf(ctx->memory, "%% Trying to read 'xref' token for xref table, or 'int int obj' for an xref stream\n");
 
     if (ctx->startxref > ctx->main_stream_length - 5) {
-        pdfi_set_error(ctx, 0, NULL, E_PDF_BADSTARTXREF, "pdfi_read_xref", (char *)"startxref offset is beyond end of file");
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_rangecheck), NULL, E_PDF_BADSTARTXREF, "pdfi_read_xref", (char *)"startxref offset is beyond end of file")) < 0)
+            goto exit;
+
         goto repair;
     }
     if (ctx->startxref < 0) {
-        pdfi_set_error(ctx, 0, NULL, E_PDF_BADSTARTXREF, "pdfi_read_xref", (char *)"startxref offset is before start of file");
+        if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_rangecheck), NULL, E_PDF_BADSTARTXREF, "pdfi_read_xref", (char *)"startxref offset is before start of file")) < 0)
+            goto exit;
+
         goto repair;
     }
 
@@ -1016,14 +1077,19 @@ int pdfi_read_xref(pdf_context *ctx)
     /* If it starts with an int, it's an xref stream dict */
     code = pdfi_read_bare_int(ctx, ctx->main_stream, &obj_num);
     if (code == 1) {
-        code = pdfi_read_xref_stream_dict(ctx, ctx->main_stream, obj_num);
-        if (code < 0)
+        if (pdfi_check_xref_stream(ctx)) {
+            code = pdfi_read_xref_stream_dict(ctx, ctx->main_stream, obj_num);
+            if (code < 0)
+                goto repair;
+        } else
             goto repair;
     } else {
         /* If not, it had better start 'xref', and be an old-style xref table */
         code = pdfi_read_bare_keyword(ctx, ctx->main_stream);
         if (code != TOKEN_XREF) {
-            pdfi_set_error(ctx, 0, NULL, E_PDF_BADSTARTXREF, "pdfi_read_xref", (char *)"Failed to read any token at the startxref location");
+            if ((code = pdfi_set_error_stop(ctx, gs_note_error(gs_error_syntaxerror), NULL, E_PDF_BADSTARTXREF, "pdfi_read_xref", (char *)"Failed to read any token at the startxref location")) < 0)
+                goto exit;
+
             goto repair;
         }
 
@@ -1037,64 +1103,64 @@ int pdfi_read_xref(pdf_context *ctx)
         xref_entry *entry;
         char Buffer[32];
 
-        dmprintf(ctx->memory, "\n%% Dumping xref table\n");
+        outprintf(ctx->memory, "\n%% Dumping xref table\n");
         for (i=0;i < ctx->xref_table->xref_size;i++) {
             entry = &ctx->xref_table->xref[i];
             if(entry->compressed) {
-                dmprintf(ctx->memory, "*");
+                outprintf(ctx->memory, "*");
                 gs_snprintf(Buffer, sizeof(Buffer), "%"PRId64"", entry->object_num);
                 j = 10 - strlen(Buffer);
                 while(j--) {
-                    dmprintf(ctx->memory, " ");
+                    outprintf(ctx->memory, " ");
                 }
-                dmprintf1(ctx->memory, "%s ", Buffer);
+                outprintf(ctx->memory, "%s ", Buffer);
 
                 gs_snprintf(Buffer, sizeof(Buffer), "%ld", entry->u.compressed.compressed_stream_num);
                 j = 10 - strlen(Buffer);
                 while(j--) {
-                    dmprintf(ctx->memory, " ");
+                    outprintf(ctx->memory, " ");
                 }
-                dmprintf1(ctx->memory, "%s ", Buffer);
+                outprintf(ctx->memory, "%s ", Buffer);
 
                 gs_snprintf(Buffer, sizeof(Buffer), "%ld", entry->u.compressed.object_index);
                 j = 10 - strlen(Buffer);
                 while(j--) {
-                    dmprintf(ctx->memory, " ");
+                    outprintf(ctx->memory, " ");
                 }
-                dmprintf1(ctx->memory, "%s ", Buffer);
+                outprintf(ctx->memory, "%s ", Buffer);
             }
             else {
-                dmprintf(ctx->memory, " ");
+                outprintf(ctx->memory, " ");
 
                 gs_snprintf(Buffer, sizeof(Buffer), "%ld", entry->object_num);
                 j = 10 - strlen(Buffer);
                 while(j--) {
-                    dmprintf(ctx->memory, " ");
+                    outprintf(ctx->memory, " ");
                 }
-                dmprintf1(ctx->memory, "%s ", Buffer);
+                outprintf(ctx->memory, "%s ", Buffer);
 
                 gs_snprintf(Buffer, sizeof(Buffer), "%"PRIdOFFSET"", entry->u.uncompressed.offset);
                 j = 10 - strlen(Buffer);
                 while(j--) {
-                    dmprintf(ctx->memory, " ");
+                    outprintf(ctx->memory, " ");
                 }
-                dmprintf1(ctx->memory, "%s ", Buffer);
+                outprintf(ctx->memory, "%s ", Buffer);
 
                 gs_snprintf(Buffer, sizeof(Buffer), "%ld", entry->u.uncompressed.generation_num);
                 j = 10 - strlen(Buffer);
                 while(j--) {
-                    dmprintf(ctx->memory, " ");
+                    outprintf(ctx->memory, " ");
                 }
-                dmprintf1(ctx->memory, "%s ", Buffer);
+                outprintf(ctx->memory, "%s ", Buffer);
             }
             if (entry->free)
-                dmprintf(ctx->memory, "f\n");
+                outprintf(ctx->memory, "f\n");
             else
-                dmprintf(ctx->memory, "n\n");
+                outprintf(ctx->memory, "n\n");
         }
     }
     if (ctx->args.pdfdebug)
-        dmprintf(ctx->memory, "\n");
+        outprintf(ctx->memory, "\n");
 
  exit:
     (void)pdfi_loop_detector_cleartomark(ctx);
@@ -1106,7 +1172,7 @@ int pdfi_read_xref(pdf_context *ctx)
 
 repair:
     (void)pdfi_loop_detector_cleartomark(ctx);
-    if (!ctx->repaired)
+    if (!ctx->repaired && !ctx->args.pdfstoponerror)
         return(pdfi_repair_file(ctx));
     return 0;
 }
