@@ -1,4 +1,4 @@
-/* Copyright (C) 2018-2022 Artifex Software, Inc.
+/* Copyright (C) 2018-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Top-level API implementation of PDF */
@@ -53,14 +53,12 @@ typedef struct pdf_interp_instance_s
 
 extern const char gp_file_name_list_separator;
 
-#define _PDFI_STRING_IT(s) #s
-#define PDFI_STRING_IT(s) _PDFI_STRING_IT(s)
-#define GS_LIB_DEFAULT_STRING PDFI_STRING_IT(GS_LIB_DEFAULT)
+#define GS_LIB_DEFAULT_STRING GS_STRINGIZE(GS_LIB_DEFAULT)
 
 static int
 pdf_detect_language(const char *s, int len)
 {
-    if (len >= 5 && memcmp(s, "%!PDF", 5) == 0)
+    if (len >= 5 && memcmp(s, "%PDF-", 5) == 0)
         return 100;
     return 0;
 }
@@ -139,7 +137,7 @@ pdf_impl_allocate_interp_instance(pl_interp_implementation_t *impl,
 
     impl->interp_client_data = instance;
     if (COMPILE_INITS == 1) {
-        newpathsstr = (char *)gs_alloc_bytes(ctx->memory, strlen(rompathstr) + strlen(GS_LIB_DEFAULT_STRING) + 2, "temp paths string");
+        newpathsstr = (char *)gs_alloc_bytes(ctx->memory, (size_t)strlen(rompathstr) + strlen(GS_LIB_DEFAULT_STRING) + 2, "temp paths string");
         if (newpathsstr == NULL) {
             newpathsstr = (char *)rompathstr;
         }
@@ -200,11 +198,12 @@ pdf_impl_set_device(pl_interp_implementation_t *impl, gx_device *pdevice)
     code = gs_setsmoothness(ctx->pgs, 0.02); /* Match gs code */
 
     /* NOTE: Not sure what this is or when it matters, but it should match gs, which sets it to true */
-    /* TODO: Commented out because it caused a seemingly random incorrect pdfwrite output.
+    /* TODO: Compiled out because it caused a seemingly random incorrect pdfwrite output.
      * Will try putting it back in when pdfwrite is more stable.
      */
-    //gs_setlimitclamp(ctx->pgs, true); /* Match gs code */
-
+#if 0
+    gs_setlimitclamp(ctx->pgs, true); /* Match gs code */
+#endif
     /* We need to setup strokeadjust the same way gs does, which is currently based on
      * the device resolution and the DITHERPPI arg.
      *
@@ -346,7 +345,7 @@ pdf_impl_process_eof(pl_interp_implementation_t *impl)
         gp_fclose(instance->scratch_file);
         instance->scratch_file = NULL;
         code = pdfi_process_pdf_file(ctx, instance->scratch_name);
-        unlink(instance->scratch_name);
+        gp_unlink(ctx->memory, instance->scratch_name);
         if (code < 0)
         {
             gs_catch(code, "cannot process PDF file");
@@ -414,7 +413,7 @@ static int plist_value_get_string_or_name(pdf_context *ctx, gs_param_typed_value
     if (*pstr)
         gs_free_object(ctx->memory, *pstr, "plist_value_get_string_or_name()");
 
-    *pstr = (char *)gs_alloc_bytes(ctx->memory, size + 1, "plist_value_get_string_or_name()");
+    *pstr = (char *)gs_alloc_bytes(ctx->memory, (size_t)size + 1, "plist_value_get_string_or_name()");
     if (*pstr == NULL)
         return_error(gs_error_VMerror);
 
@@ -473,6 +472,9 @@ static int plist_value_get_bool(gs_param_typed_value *pvalue, bool *pbool)
     return_error(gs_error_typecheck);
 }
 
+#define argis(P, S) \
+(!strncmp((P), (S), sizeof(S)-1) && ((P)[sizeof(S)-1] == 0 || (P)[sizeof(S)-1] == '=' || (P)[sizeof(S)-1] == '#'))
+
 static int
 pdf_impl_set_param(pl_interp_implementation_t *impl,
                    gs_param_list    *plist)
@@ -484,6 +486,7 @@ pdf_impl_set_param(pl_interp_implementation_t *impl,
     int code;
     int len;
     bool discard_isname;
+    bool Printed_set = false;
 
     param_init_enumerator(&enumerator);
     if ((code = param_get_next_key(plist, &enumerator, &key)) == 0) {
@@ -501,158 +504,176 @@ pdf_impl_set_param(pl_interp_implementation_t *impl,
             goto exit;
         }
 
-        if (!strncmp(param, "QUIET", 5)) {
+        if (argis(param, "QUIET")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.QUIET);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "FirstPage", 9)) {
+        if (argis(param, "FirstPage")) {
             code = plist_value_get_int(&pvalue, &ctx->args.first_page);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "LastPage", 8)) {
+        if (argis(param, "LastPage")) {
             code = plist_value_get_int(&pvalue, &ctx->args.last_page);
             if (code < 0)
                 return code;
         }
         /* PDF interpreter flags */
-        if (!strncmp(param, "VerboseErrors", 13)) {
+        if (argis(param, "VerboseErrors")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.verbose_errors);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "VerboseWarnings", 15)) {
+        if (argis(param, "VerboseWarnings")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.verbose_warnings);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PDFDEBUG", 8)) {
+        if (argis(param, "PDFCACHE")) {
+            code = plist_value_get_int(&pvalue, &ctx->args.PDFCacheSize);
+            if (code < 0)
+                return code;
+        }
+        if (argis(param, "PDFDEBUG")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.pdfdebug);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PDFSTOPONERROR", 14)) {
+        if (argis(param, "PDFSTOPONERROR")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.pdfstoponerror);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PDFSTOPONWARNING", 16)) {
+        if (argis(param, "PDFSTOPONWARNING")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.pdfstoponwarning);
             if (code < 0)
                 return code;
+            if (ctx->args.pdfstoponwarning != 0)
+                ctx->args.pdfstoponerror = 1;
         }
-        if (!strncmp(param, "NOTRANSPARENCY", 14)) {
+        if (argis(param, "NOTRANSPARENCY")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.notransparency);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PDFNOCIDFALLBACK", 13)) {
+        if (argis(param, "PDFNOCIDFALLBACK")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.nocidfallback);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "NO_PDFMARK_OUTLINES", 19)) {
+        if (argis(param, "NO_PDFMARK_OUTLINES")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.no_pdfmark_outlines);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "NO_PDFMARK_DESTS", 16)) {
+        if (argis(param, "NO_PDFMARK_DESTS")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.no_pdfmark_dests);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PDFFitPage", 10)) {
+        if (argis(param, "PDFFitPage")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.pdffitpage);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "UseCropBox", 10)) {
+        if (argis(param, "UseCropBox")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.usecropbox);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "UseArtBox", 9)) {
+        if (argis(param, "UseArtBox")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.useartbox);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "UseBleedBox", 11)) {
+        if (argis(param, "UseBleedBox")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.usebleedbox);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "UseTrimBox", 10)) {
+        if (argis(param, "UseTrimBox")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.usetrimbox);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "Printed", 7)) {
+        if (argis(param, "Printed")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.printed);
             if (code < 0)
                 return code;
+            Printed_set = true;
         }
-        if (!strncmp(param, "DITHERPPI", 9)) {
+        if (argis(param, "DITHERPPI")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.ditherppi);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "ShowAcroForm", 12)) {
+        if (argis(param, "ShowAcroForm")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.showacroform);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "ShowAnnots", 10)) {
+        if (argis(param, "ShowAnnots")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.showannots);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "SHOWANNOTTYPES", 14)) {
+        if (argis(param, "SHOWANNOTTYPES")) {
             code = plist_value_get_namelist(ctx, &pvalue, &ctx->args.showannottypes);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PreserveAnnots", 14)) {
+        if (argis(param, "PreserveAnnots")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.preserveannots);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PRESERVEANNOTTYPES", 18)) {
+        if (argis(param, "PRESERVEANNOTTYPES")) {
             code = plist_value_get_namelist(ctx, &pvalue, &ctx->args.preserveannottypes);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PreserveMarkedContent", 21)) {
+        if (argis(param, "PreserveMarkedContent")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.preservemarkedcontent);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "NoUserUnit", 10)) {
+        if (argis(param, "PreserveEmbeddedFiles")) {
+            code = plist_value_get_bool(&pvalue, &ctx->args.preserveembeddedfiles);
+            if (code < 0)
+                return code;
+        }
+        if (argis(param, "PreserveDocView")) {
+            code = plist_value_get_bool(&pvalue, &ctx->args.preservedocview);
+            if (code < 0)
+                return code;
+        }
+        if (argis(param, "NoUserUnit")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.nouserunit);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "RENDERTTNOTDEF", 13)) {
+        if (argis(param, "RENDERTTNOTDEF")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.renderttnotdef);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PDFINFO", 7)) {
+        if (argis(param, "PDFINFO")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.pdfinfo);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "DOPDFMARKS", 10)) {
+        if (argis(param, "DOPDFMARKS")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.dopdfmarks);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "PDFPassword", 11)) {
+        if (argis(param, "PDFPassword")) {
             code = plist_value_get_string_or_name(ctx, &pvalue, &ctx->encryption.Password , &ctx->encryption.PasswordLen, &discard_isname);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "UsePDFX3Profile", strlen("UsePDFX3Profile"))) {
+        if (argis(param, "UsePDFX3Profile")) {
             /* This is a weird one because it can be either a bool or an int.
              * If it's a bool=true, then it defaults to PDFX3Profile_num = 0
              * If it's an int, then we set the flag to true and use the
@@ -668,17 +689,17 @@ pdf_impl_set_param(pl_interp_implementation_t *impl,
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "NOSUBSTDEVICECOLORS", strlen("NOSUBSTDEVICECOLORS"))) {
+        if (argis(param, "NOSUBSTDEVICECOLORS")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.NOSUBSTDEVICECOLORS);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "UseOutputIntent", strlen("UseOutputIntent"))) {
+        if (argis(param, "UseOutputIntent")) {
             code = plist_value_get_string_or_name(ctx, &pvalue, &ctx->args.UseOutputIntent, &len, &discard_isname);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "FONTPATH", 11)) {
+        if (argis(param, "FONTPATH")) {
             char *s = NULL;
             int slen;
             code = plist_value_get_string_or_name(ctx, &pvalue, &s , &slen, &discard_isname);
@@ -687,7 +708,7 @@ pdf_impl_set_param(pl_interp_implementation_t *impl,
             code = pdfi_add_paths_to_search_paths(ctx, (const char *)s, slen, true);
             gs_free_object(ctx->memory, s, "FONTPATH param string");
         }
-        if (!strncmp(param, "FONTMAP", 7)) {
+        if (argis(param, "FONTMAP")) {
             char *s = NULL;
             int slen;
             code = plist_value_get_string_or_name(ctx, &pvalue, &s, &slen, &discard_isname);
@@ -696,26 +717,30 @@ pdf_impl_set_param(pl_interp_implementation_t *impl,
             code = pdfi_add_fontmapfiles(ctx, (const char *)s, slen);
             gs_free_object(ctx->memory, s, "FONTMAP param string");
         }
-        if (!strncmp(param, "CIDSubstPath", 12)) {
+        if (argis(param, "CIDSubstPath")) {
             code = plist_value_get_string_or_name(ctx, &pvalue, (char **)&ctx->args.cidfsubstpath.data, (int *)&ctx->args.cidfsubstpath.size, &discard_isname);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "CIDSubstFont", 12)) {
+        if (argis(param, "CIDFSubstFont")) {
             code = plist_value_get_string_or_name(ctx, &pvalue, (char **)&ctx->args.cidfsubstfont.data, (int *)&ctx->args.cidfsubstfont.size, &discard_isname);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "SUBSTFONT", 12)) {
+        if (argis(param, "SUBSTFONT")) {
             code = plist_value_get_string_or_name(ctx, &pvalue, (char **)&ctx->args.defaultfont.data, (int *)&ctx->args.defaultfont.size, &ctx->args.defaultfont_is_name);
             if (code < 0)
                 return code;
         }
-        if (!strncmp(param, "IgnoreToUnicode", 15)) {
+        if (argis(param, "IgnoreToUnicode")) {
             code = plist_value_get_bool(&pvalue, &ctx->args.ignoretounicode);
             if (code < 0)
                 return code;
         }
+        if (argis(param, "OutputFile")) {
+            if (!Printed_set)
+                ctx->args.printed = true;
+        }
     }
 
  exit:
