@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Ghostscript DLL loader for Windows */
@@ -103,9 +103,9 @@ int dwmain_add_file_control_path(const TCHAR *pathfile)
 {
     LPSTR p;
     int code, i;
-    p = malloc(wchar_to_utf8(NULL, (wchar_t *)pathfile));
+    p = malloc(gp_uint16_to_utf8(NULL, (wchar_t *)pathfile));
     if (p) {
-        wchar_to_utf8(p, (wchar_t *)pathfile);
+        gp_uint16_to_utf8(p, (wchar_t *)pathfile);
         for (i = 0; i < strlen(p); i++) {
             if (p[i] == '\\') {
                 p[i] = '/';
@@ -123,9 +123,9 @@ void dwmain_remove_file_control_path(const TCHAR *pathfile)
 {
     LPSTR p;
     int i;
-    p = malloc(wchar_to_utf8(NULL, (wchar_t *)pathfile));
+    p = malloc(gp_uint16_to_utf8(NULL, (wchar_t *)pathfile));
     if (p) {
-        wchar_to_utf8(p, (wchar_t *)pathfile);
+        gp_uint16_to_utf8(p, (wchar_t *)pathfile);
         for (i = 0; i < strlen(p); i++) {
             if (p[i] == '\\') {
                 p[i] = '/';
@@ -360,7 +360,11 @@ int new_main(int argc, char *argv[])
         sprintf(dformat, "-dDisplayFormat=%d", format);
     }
     nargc = argc + 2;
-    nargv = (char **)malloc(nargc * sizeof(char *));
+    nargv = (char **)malloc((size_t)nargc * sizeof(char *));
+
+    if (nargv == NULL)
+        return 1;
+
     nargv[0] = argv[0];
     nargv[1] = dformat;
     nargv[2] = ddpi;
@@ -486,13 +490,13 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
      */
     {
         wchar_t *uni = GetCommandLineW();
-        pstart = p = malloc(wchar_to_utf8(NULL, uni));
+        pstart = p = malloc(gp_uint16_to_utf8(NULL, uni));
         if (p != NULL)
-            wchar_to_utf8(p, uni);
+            gp_uint16_to_utf8(p, uni);
     }
 
     argc = 0;
-    args = (char *)malloc(lstrlen(p)+1);
+    args = (char *)malloc((size_t)lstrlen(p)+1);
     if (args == (char *)NULL) {
         fprintf(stdout, "Insufficient memory in WinMain()\n");
         return 1;
