@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -75,7 +75,9 @@ enum {
 static int dict_create_contents(uint size, const ref * pdref, bool pack);
 
 /* Debugging statistics - uses a static, so not threadsafe. */
-#if defined(DEBUG) && !defined(GS_THREADSAFE)
+/* #define COLLECT_STATS_IDICT */
+
+#ifdef COLLECT_STATS_IDICT
 struct stats_dict_s {
     long lookups;		/* total lookups */
     long probe1;		/* successful lookups on only 1 probe */
@@ -289,13 +291,6 @@ dict_find(const ref * pdref, const ref * pkey,
     /* Compute hash.  The only types we bother with are strings, */
     /* names, and (unlikely, but worth checking for) integers. */
     switch (r_type(pkey)) {
-    case t_name:
-        nidx = name_index(mem, pkey);
-    nh:
-        hash = dict_name_index_hash(nidx);
-        kpack = packed_name_key(nidx);
-        ktype = t_name;
-        break;
     case t_string:		/* convert to a name first */
         {
             ref nref;
@@ -309,6 +304,13 @@ dict_find(const ref * pdref, const ref * pkey,
             nidx = name_index(mem, &nref);
         }
         goto nh;
+    case t_name:
+        nidx = name_index(mem, pkey);
+    nh:
+        hash = dict_name_index_hash(nidx);
+        kpack = packed_name_key(nidx);
+        ktype = t_name;
+        break;
     case t_real:
         /*
          * Make sure that equal reals and integers hash the same.
