@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -77,7 +77,7 @@ struct gs_main_instance_s {
     long base_time[2];		/* starting usertime */
     void *readline_data;	/* data for gp_readline */
     ref error_object;		/* Use by gsapi_*() */
-    int (*get_codepoint)(gp_file *file, const char **astr);
+    int (*get_codepoint)(stream *s, const char **astr);
                                 /* Get next 'unicode' codepoint */
     display_callback *display;	/* callback structure for display device */
     /* The following are updated dynamically. */
