@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -428,7 +428,7 @@ struct ref_s {
 
     struct tas_s tas;
 
-    union v {			/* name the union to keep gdb happy */
+    union ref_value {			/* name the union to keep gdb happy */
         ps_int intval;
         ushort boolval;
         float realval;
@@ -452,7 +452,7 @@ struct ref_s {
         struct stream_s *pfile;
         struct psi_device_ref_s *pdevice;
         obj_header_t *pstruct;
-        uint64_t dummy; /* force 16-byte ref on 32-bit platforms */
+        uint64_t dummy; /* force 8-byte ref on 32-bit platforms */
     } value;
 };
 
@@ -606,7 +606,7 @@ struct ref_s {
 /* We assume all alignment values are powers of 2. */
 #define ARCH_ALIGN_REF_MOD\
  (((ARCH_ALIGN_LONG_MOD - 1) | (ARCH_ALIGN_FLOAT_MOD - 1) |\
-   (ARCH_ALIGN_PTR_MOD - 1)) + 1)
+   (ARCH_ALIGN_PTR_MOD - 1) | (ARCH_ALIGN_UINT64_T_MOD - 1)) + 1)
 
 /* Select reasonable values for PDF interpreter */
 /* The maximum array size cannot exceed max_uint/ARCH_SIZEOF_REF */
