@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -39,13 +39,12 @@
 #include "interp.h"
 #include "gxgstate.h"
 
-#ifndef GS_THREADSAFE
-/* Number of threads to allow per process. Unless GS_THREADSAFE is defined
- * more than 1 is guaranteed to fail.
+/* This is the fallback for the number of threads to allow per process; i.e. just one.
+ * This is only ever used if the gp_get_globals function returns 0 (i.e. only for
+ * platforms that don't support threading).
  */
 static int gsapi_instance_counter = 0;
 static const int gsapi_instance_max = 1;
-#endif
 
 
 #ifdef METRO
@@ -83,12 +82,15 @@ psapi_new_instance(gs_lib_ctx_t **pinstance,
     if (pinstance == NULL)
         return gs_error_Fatal;
 
-#ifndef GS_THREADSAFE
-    /* limited to 1 instance, till it works :) */
-    if ( gsapi_instance_counter >= gsapi_instance_max )
-        return gs_error_Fatal;
-    ++gsapi_instance_counter;
-#endif
+    if (gp_get_globals() == NULL) {
+        /* This platform does not support the thread safe instance
+         * handling. We'll drop back to the old mechanism we've used
+         * to handle limiting ourselves to 1 instance in the past,
+         * despite this being thread-unsafe itself. */
+        if ( gsapi_instance_counter >= gsapi_instance_max )
+            return gs_error_Fatal;
+        ++gsapi_instance_counter;
+    }
 
     mem = gs_malloc_init_with_context(*pinstance);
     if (mem == NULL)
@@ -159,12 +161,24 @@ psapi_delete_instance(gs_lib_ctx_t *ctx)
     /* Release the memory (frees up everything) */
     gs_malloc_release(mem);
 
-#ifndef GS_THREADSAFE
-    --gsapi_instance_counter;
-#endif
+    if (gp_get_globals() == NULL)
+        --gsapi_instance_counter;
 }
 
-static int utf16le_get_codepoint(gp_file *file, const char **astr)
+static int ascii_get_codepoint(stream *s, const char **astr)
+{
+    if (s)
+        return spgetc(s);
+    else {
+        int rune = **astr;
+        (*astr)++;
+        if (rune != 0)
+            return rune;
+    }
+    return EOF;
+}
+
+static int utf16le_get_codepoint(stream *s, const char **astr)
 {
     int c;
     int rune;
@@ -180,11 +194,11 @@ static int utf16le_get_codepoint(gp_file *file, const char **astr)
      * be wrong. */
 
     do {
-        if (file) {
-            rune = gp_fgetc(file);
+        if (s) {
+            rune = spgetc(s);
             if (rune == EOF)
                 return EOF;
-            c = gp_fgetc(file);
+            c = spgetc(s);
             if (c == EOF)
                 return EOF;
             rune += c<<8;
@@ -206,11 +220,11 @@ static int utf16le_get_codepoint(gp_file *file, const char **astr)
 lead: /* We've just read a leading surrogate */
         rune -= 0xD800;
         rune <<= 10;
-        if (file) {
-            trail = gp_fgetc(file);
+        if (s) {
+            trail = spgetc(s);
             if (trail == EOF)
                 return EOF;
-            c = gp_fgetc(file);
+            c = spgetc(s);
             if (c == EOF)
                 return EOF;
             trail += c<<8;
@@ -256,7 +270,7 @@ psapi_set_arg_encoding(gs_lib_ctx_t *ctx, int encoding)
          */
         gs_main_inst_arg_decode(get_minst_from_memory(ctx->memory), gp_local_arg_encoding_get_codepoint);
 #else
-        gs_main_inst_arg_decode(get_minst_from_memory(ctx->memory), NULL);
+        gs_main_inst_arg_decode(get_minst_from_memory(ctx->memory), ascii_get_codepoint);
 #endif /* WIN32 */
         return 0;
     }
@@ -513,8 +527,7 @@ psapi_exit(gs_lib_ctx_t *ctx)
     if (ctx == NULL)
         return gs_error_Fatal;
 
-    gs_to_exit(ctx->memory, 0);
-    return 0;
+    return gs_to_exit(ctx->memory, 0);
 }
 
 int
