@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -62,6 +62,7 @@ zsetstrokeadjust(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
+    check_op(1);
     check_type(*op, t_boolean);
     gs_setstrokeadjust(igs, op->value.boolval);
     pop(1);
@@ -160,6 +161,7 @@ zcopy_gstate(i_ctx_t *i_ctx_p)
     gs_memory_t *mem;
     int code;
 
+    check_op(2);
     check_stype(*op, st_igstate_obj);
     check_stype(*op1, st_igstate_obj);
     check_write(*op);
@@ -168,6 +170,10 @@ zcopy_gstate(i_ctx_t *i_ctx_p)
         return code;
     pgs = igstate_ptr(op);
     pgs1 = igstate_ptr(op1);
+
+    if (pgs == pgs1)
+        return 0;
+
     pistate = gs_int_gstate(pgs);
     code = gstate_check_space(i_ctx_p, gs_int_gstate(pgs1), r_space(op));
     if (code < 0)
@@ -196,6 +202,7 @@ zcurrentgstate(i_ctx_t *i_ctx_p)
     int code;
     gs_memory_t *mem;
 
+    check_op(1);
     check_stype(*op, st_igstate_obj);
     check_write(*op);
     code = gstate_unshare(i_ctx_p);
@@ -225,6 +232,7 @@ zsetgstate(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     int code;
 
+    check_op(1);
     check_stype(*op, st_igstate_obj);
     check_read(*op);
     code = gs_setgstate(igs, igstate_ptr(op));
@@ -248,7 +256,7 @@ typedef struct local_rects_s {
 } local_rects_t;
 
 /* Forward references */
-static int rect_get(local_rects_t *, os_ptr, gs_memory_t *);
+static int rect_get(i_ctx_t *i_ctx_p, local_rects_t *, os_ptr, gs_memory_t *);
 static void rect_release(local_rects_t *, gs_memory_t *);
 
 /* <x> <y> <width> <height> .rectappend - */
@@ -258,7 +266,7 @@ zrectappend(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     local_rects_t lr;
-    int npop = rect_get(&lr, op, imemory);
+    int npop = rect_get(i_ctx_p, &lr, op, imemory);
     int code;
 
     if (npop < 0)
@@ -278,7 +286,7 @@ zrectclip(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     local_rects_t lr;
-    int npop = rect_get(&lr, op, imemory);
+    int npop = rect_get(i_ctx_p, &lr, op, imemory);
     int code;
 
     if (npop < 0)
@@ -298,7 +306,7 @@ zrectfill(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     local_rects_t lr;
-    int npop = rect_get(&lr, op, imemory);
+    int npop = rect_get(i_ctx_p, &lr, op, imemory);
     int code;
 
     if (npop < 0)
@@ -323,14 +331,14 @@ zrectstroke(i_ctx_t *i_ctx_p)
 
     if (read_matrix(imemory, op, &mat) >= 0) {
         /* Concatenate the matrix to the CTM just before stroking the path. */
-        npop = rect_get(&lr, op - 1, imemory);
+        npop = rect_get(i_ctx_p, &lr, op - 1, imemory);
         if (npop < 0)
             return npop;
         code = gs_rectstroke(igs, lr.pr, lr.count, &mat);
         npop++;
     } else {
         /* No matrix. */
-        npop = rect_get(&lr, op, imemory);
+        npop = rect_get(i_ctx_p, &lr, op, imemory);
         if (npop < 0)
             return npop;
         code = gs_rectstroke(igs, lr.pr, lr.count, (gs_matrix *) 0);
@@ -347,13 +355,14 @@ zrectstroke(i_ctx_t *i_ctx_p)
 /* Get rectangles from the stack. */
 /* Return the number of elements to pop (>0) if OK, <0 if error. */
 static int
-rect_get(local_rects_t * plr, os_ptr op, gs_memory_t *mem)
+rect_get(i_ctx_t *i_ctx_p, local_rects_t * plr, os_ptr op, gs_memory_t *mem)
 {
     int format, code;
     uint n, count;
     gs_rect *pr;
     double rv[4];
 
+    check_op(1);
     switch (r_type(op)) {
         case t_array:
         case t_mixedarray:
@@ -369,6 +378,7 @@ rect_get(local_rects_t * plr, os_ptr op, gs_memory_t *mem)
             count /= 4;
             break;
         default:		/* better be 4 numbers */
+            check_op(4);
             code = num_params(op, 4, rv);
             if (code < 0)
                 return code;
@@ -428,8 +438,10 @@ zsetbbox(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     double box[4];
+    int code;
 
-    int code = num_params(op, 4, box);
+    check_op(4);
+    code = num_params(op, 4, box);
 
     if (code < 0)
         return code;
