@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -49,6 +49,7 @@ zset_real(i_ctx_t *i_ctx_p, int (*set_proc)(gs_gstate *, double))
     return code;
 }
 
+#if 0 /* Currently unused */
 static int
 zcurrent_real(i_ctx_t *i_ctx_p, double (*current_proc)(const gs_gstate *))
 {
@@ -58,6 +59,7 @@ zcurrent_real(i_ctx_t *i_ctx_p, double (*current_proc)(const gs_gstate *))
     make_real(op, current_proc(igs));
     return 0;
 }
+#endif
 
 static int
 zset_bool(i_ctx_t *i_ctx_p, void (*set_proc)(gs_gstate *, bool))
@@ -91,6 +93,7 @@ zset_uint(i_ctx_t *i_ctx_p, void (*set_proc)(gs_gstate *, uint))
     return 0;
 }
 
+#if 0 /* Currently unused */
 static int
 zcurrent_uint(i_ctx_t *i_ctx_p, uint (*current_proc)(const gs_gstate *))
 {
@@ -100,6 +103,7 @@ zcurrent_uint(i_ctx_t *i_ctx_p, uint (*current_proc)(const gs_gstate *))
     make_int(op, current_proc(igs));
     return 0;
 }
+#endif
 
 /* ------ Operations on the entire graphics state ------ */
 
@@ -213,7 +217,10 @@ zsetlinewidth(i_ctx_t *i_ctx_p)
          * of the width.
          */
     double width;
-    int code = real_param(op, &width);
+    int code;
+
+    check_op(1);
+    code = real_param(op, &width);
 
     if (code < 0)
         return_op_typecheck(op);
@@ -240,7 +247,10 @@ zsetlinecap(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     int param;
-    int code = int_param(op, max_int, &param);
+    int code;
+
+    check_op(1);
+    code = int_param(op, max_int, &param);
 
     if (code < 0 || (code = gs_setlinecap(igs, (gs_line_cap) param)) < 0)
         return code;
@@ -265,7 +275,10 @@ zsetlinejoin(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     int param;
-    int code = int_param(op, max_int, &param);
+    int code;
+
+    check_op(1);
+    code = int_param(op, max_int, &param);
 
     if (code < 0 || (code = gs_setlinejoin(igs, (gs_line_join) param)) < 0)
         return code;
@@ -288,6 +301,8 @@ zcurrentlinejoin(i_ctx_t *i_ctx_p)
 static int
 zsetmiterlimit(i_ctx_t *i_ctx_p)
 {
+    os_ptr op = osp;
+    check_op(1);
     return zset_real(i_ctx_p, gs_setmiterlimit);
 }
 
@@ -309,11 +324,13 @@ zsetdash(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     os_ptr op1 = op - 1;
     double offset;
-    int code = real_param(op, &offset);
+    int code;
     uint i, n;
     gs_memory_t *mem = imemory;
     float *pattern;
 
+    check_op(2);
+    code = real_param(op, &offset);
     if (code < 0)
         return_op_typecheck(op);
     if (!r_is_array(op1))
@@ -521,6 +538,10 @@ zcurrentlimitclamp(i_ctx_t *i_ctx_p)
     return zcurrent_bool(i_ctx_p, gs_currentlimitclamp);
 }
 
+/* We need this for the PScript5.DLL Idiom recognition, when outputting
+ * to pdfwrite. This is used to turn the 'fake bold' text into text
+ * rendering mode 2 (fill and then stroke).
+ */
 /* <int> .settextrenderingmode - */
 static int
 zsettextrenderingmode(i_ctx_t *i_ctx_p)
@@ -528,76 +549,6 @@ zsettextrenderingmode(i_ctx_t *i_ctx_p)
     return zset_uint(i_ctx_p, gs_settextrenderingmode);
 }
 
-/* - .currenttextrenderingmode <int> */
-static int
-zcurrenttextrenderingmode(i_ctx_t *i_ctx_p)
-{
-    return zcurrent_uint(i_ctx_p, gs_currenttextrenderingmode);
-}
-static int
-zsettextspacing(i_ctx_t *i_ctx_p)
-{
-    return zset_real(i_ctx_p, gs_settextspacing);
-}
-static int
-zcurrenttextspacing(i_ctx_t *i_ctx_p)
-{
-    return zcurrent_real(i_ctx_p, gs_currenttextspacing);
-}
-static int
-zsettextleading(i_ctx_t *i_ctx_p)
-{
-    return zset_real(i_ctx_p, gs_settextleading);
-}
-static int
-zcurrenttextleading(i_ctx_t *i_ctx_p)
-{
-    return zcurrent_real(i_ctx_p, gs_currenttextleading);
-}
-static int
-zsettextrise(i_ctx_t *i_ctx_p)
-{
-    return zset_real(i_ctx_p, gs_settextrise);
-}
-static int
-zcurrenttextrise(i_ctx_t *i_ctx_p)
-{
-    return zcurrent_real(i_ctx_p, gs_currenttextrise);
-}
-static int
-zsetwordspacing(i_ctx_t *i_ctx_p)
-{
-    return zset_real(i_ctx_p, gs_setwordspacing);
-}
-static int
-zcurrentwordspacing(i_ctx_t *i_ctx_p)
-{
-    return zcurrent_real(i_ctx_p, gs_currentwordspacing);
-}
-
-static int
-zsettexthscaling(i_ctx_t *i_ctx_p)
-{
-    return zset_real(i_ctx_p, gs_settexthscaling);
-}
-static int
-zcurrenttexthscaling(i_ctx_t *i_ctx_p)
-{
-    return zcurrent_real(i_ctx_p, gs_currenttexthscaling);
-}
-
-static int
-zsetPDFfontsize(i_ctx_t *i_ctx_p)
-{
-    return zset_real(i_ctx_p, gs_setPDFfontsize);
-}
-static int
-zcurrentPDFfontsize(i_ctx_t *i_ctx_p)
-{
-    return zcurrent_real(i_ctx_p, gs_currentPDFfontsize);
-}
-
-
 /* <bool> .sethpglpathmode - */
 static int
 zsethpglpathmode(i_ctx_t *i_ctx_p)
@@ -653,26 +604,10 @@ const op_def zgstate2_op_defs[] = {
 };
 const op_def zgstate3_op_defs[] = {
     {"1.settextrenderingmode", zsettextrenderingmode},
-    {"0.currenttextrenderingmode", zcurrenttextrenderingmode},
-    {"1.settextspacing", zsettextspacing},
-    {"0.currenttextspacing", zcurrenttextspacing},
-    {"1.settextleading", zsettextleading},
-    {"0.currenttextleading", zcurrenttextleading},
-    {"1.settextrise", zsettextrise},
-    {"0.currenttextrise", zcurrenttextrise},
-    {"1.setwordspacing", zsetwordspacing},
-    {"0.currentwordspacing", zcurrentwordspacing},
-    {"1.settexthscaling", zsettexthscaling},
-    {"0.currenttexthscaling", zcurrenttexthscaling},
     {"0.sethpglpathmode", zsethpglpathmode},
     {"0.currenthpglpathmode", zcurrenthpglpathmode},
     op_def_end(0)
 };
-const op_def zgstate4_op_defs[] = {
-    {"1.setPDFfontsize", zsetPDFfontsize},
-    {"0.currentPDFfontsize", zcurrentPDFfontsize},
-    op_def_end(0)
-};
 
 /* ------ Internal routines ------ */
 
