@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -41,6 +41,7 @@ zdefaultmatrix(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     gs_matrix mat;
 
+    check_op(1);
     gs_defaultmatrix(igs, &mat);
     return write_matrix(op, &mat);
 }
@@ -68,7 +69,10 @@ zsetmatrix(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     gs_matrix mat;
-    int code = float_params(op, 6, &mat.xx);
+    int code;
+
+    check_op(6);
+    code = float_params(op, 6, &mat.xx);
 
     if (code < 0)
         return code;
@@ -85,6 +89,7 @@ zsetdefaultmatrix(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     int code;
 
+    check_op(1);
     if (r_has_type(op, t_null))
         code = gs_setdefaultmatrix(igs, NULL);
     else {
@@ -200,7 +205,10 @@ zconcat(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     gs_matrix mat;
-    int code = read_matrix(imemory, op, &mat);
+    int code;
+
+    check_op(1);
+    code = read_matrix(imemory, op, &mat);
 
     if (code < 0)
         return code;
@@ -219,6 +227,7 @@ zconcatmatrix(i_ctx_t *i_ctx_p)
     gs_matrix m1, m2, mp;
     int code;
 
+    check_op(3);
     if ((code = read_matrix(imemory, op - 2, &m1)) < 0 ||
         (code = read_matrix(imemory, op - 1, &m2)) < 0 ||
         (code = gs_matrix_multiply(&m1, &m2, &mp)) < 0 ||
@@ -327,6 +336,7 @@ zinvertmatrix(i_ctx_t *i_ctx_p)
     gs_matrix m;
     int code;
 
+    check_op(2);
     if ((code = read_matrix(imemory, op - 1, &m)) < 0 ||
         (code = gs_matrix_invert(&m, &m)) < 0 ||
         (code = write_matrix(op, &m)) < 0
@@ -337,134 +347,10 @@ zinvertmatrix(i_ctx_t *i_ctx_p)
     return code;
 }
 
-/* <bbox> <matrix> .bbox_transform <x0> <y0> <x1> <y1> */
-/* Calculate bonding box of a box transformed by a matrix. */
 static int
-zbbox_transform(i_ctx_t *i_ctx_p)
+zupdatematrices(i_ctx_t *i_ctx_p)
 {
-    os_ptr op = osp;
-    gs_matrix m;
-    float bbox[4];
-    gs_point aa, az, za, zz;
-    double temp;
-    int code;
-
-    if ((code = read_matrix(imemory, op, &m)) < 0)
-        return code;
-
-    if (!r_is_array(op - 1))
-        return_op_typecheck(op - 1);
-    check_read(op[-1]);
-    if (r_size(op - 1) != 4)
-        return_error(gs_error_rangecheck);
-    if ((code = process_float_array(imemory, op - 1, 4, bbox) < 0))
-        return code;
-
-    gs_point_transform(bbox[0], bbox[1], &m, &aa);
-    gs_point_transform(bbox[0], bbox[3], &m, &az);
-    gs_point_transform(bbox[2], bbox[1], &m, &za);
-    gs_point_transform(bbox[2], bbox[3], &m, &zz);
-
-    if ( aa.x > az.x)
-        temp = aa.x, aa.x = az.x, az.x = temp;
-    if ( za.x > zz.x)
-        temp = za.x, za.x = zz.x, zz.x = temp;
-    if ( za.x < aa.x)
-        aa.x = za.x;  /* min */
-    if ( az.x > zz.x)
-        zz.x = az.x;  /* max */
-
-    if ( aa.y > az.y)
-        temp = aa.y, aa.y = az.y, az.y = temp;
-    if ( za.y > zz.y)
-        temp = za.y, za.y = zz.y, zz.y = temp;
-    if ( za.y < aa.y)
-        aa.y = za.y;  /* min */
-    if ( az.y > zz.y)
-        zz.y = az.y;  /* max */
-
-    push(2);
-    make_real(op - 3, (float)aa.x);
-    make_real(op - 2, (float)aa.y);
-    make_real(op - 1, (float)zz.x);
-    make_real(op    , (float)zz.y);
-    return 0;
-}
-
-/* <matrix> .currenttextlinematrix <matrix> */
-static int
-zcurrenttextlinematrix(i_ctx_t *i_ctx_p)
-{
-    os_ptr op = osp;
-    gs_matrix mat;
-
-    check_op(1);
-    if (!r_has_type(op, t_array))
-        return_error(gs_error_typecheck);
-
-    gs_gettextlinematrix(igs, &mat);
-    return write_matrix(op, &mat);
-}
-
-static int
-zsettextlinematrix(i_ctx_t *i_ctx_p)
-{
-    os_ptr op = osp;
-    int code;
-
-    check_op(1);
-    if (r_has_type(op, t_array)) {
-        gs_matrix mat;
-
-        code = read_matrix(imemory, op, &mat);
-        if (code < 0)
-            return code;
-        code = gs_settextlinematrix(igs, &mat);
-    } else
-        code = gs_error_typecheck;
-
-    if (code < 0)
-        return code;
-    pop(1);
-    return 0;
-}
-
-/* <matrix> .currenttextmatrix <matrix> */
-static int
-zcurrenttextmatrix(i_ctx_t *i_ctx_p)
-{
-    os_ptr op = osp;
-    gs_matrix mat;
-
-    check_op(1);
-    if (!r_has_type(op, t_array))
-        return_error(gs_error_typecheck);
-
-    gs_gettextmatrix(igs, &mat);
-    return write_matrix(op, &mat);
-}
-
-static int
-zsettextmatrix(i_ctx_t *i_ctx_p)
-{
-    os_ptr op = osp;
-    int code;
-
-    check_op(1);
-    if (r_has_type(op, t_array)) {
-        gs_matrix mat;
-
-        code = read_matrix(imemory, op, &mat);
-        if (code < 0)
-            return code;
-        code = gs_settextmatrix(igs, &mat);
-    } else
-        code = gs_error_typecheck;
-
-    if (code < 0)
-        return code;
-    pop(1);
-    return 0;
+    return gs_updatematrices(igs);
 }
 
 /* ------ Initialization procedure ------ */
@@ -491,10 +377,6 @@ const op_def zmatrix_op_defs[] =
 
 const op_def zmatrix2_op_defs[] =
 {
-    {"2.bbox_transform", zbbox_transform},
-    {"1.currenttextlinematrix", zcurrenttextlinematrix},
-    {"1.settextlinematrix", zsettextlinematrix},
-    {"1.currenttextmatrix", zcurrenttextmatrix},
-    {"1.settextmatrix", zsettextmatrix},
+    {"1.updatematrices", zupdatematrices},
     op_def_end(0)
 };
