@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -47,6 +47,10 @@ zpackedarray(i_ctx_t *i_ctx_p)
     ref parr;
 
     check_type(*op, t_integer);
+
+    /* This avoids a valgrind error */
+    parr.tas.type_attrs = 0;
+
     if (op->value.intval < 0)
         return_error(gs_error_rangecheck);
     if (op->value.intval > op - osbot &&
@@ -68,6 +72,7 @@ zsetpacking(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     ref cont;
+    check_op(1);
     check_type(*op, t_boolean);
     make_struct(&cont, avm_local, ref_array_packing_container);
     ref_assign_old(&cont, &ref_array_packing, op, "setpacking");
@@ -102,6 +107,9 @@ make_packed_array(ref * parr, ref_stack_t * pstack, uint size,
 
     for (i = size; i != 0; i--) {
         pref = ref_stack_index(pstack, i - 1);
+        if (pref == NULL)
+            return_error(gs_error_stackunderflow);
+
         switch (r_btype(pref)) {	/* not r_type, opers are special */
             case t_name:
               if (name_index(imem, pref) >= packed_name_max_index)
@@ -170,6 +178,9 @@ make_packed_array(ref * parr, ref_stack_t * pstack, uint size,
 
     for (i = size; i != 0; i--) {
         pref = ref_stack_index(pstack, i - 1);
+        if (pref == NULL)
+            return_error(gs_error_stackunderflow);
+
         switch (r_btype(pref)) {	/* not r_type, opers are special */
             case t_name:
                 {
