@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -44,7 +44,7 @@
 #include "gx.h"
 #include "gxgstate.h"
 #include "gslibctx.h"
-
+#include "ichar.h"
 
 /* The (global) font directory */
 extern gs_font_dir *ifont_dir;	/* in zfont.c */
@@ -134,6 +134,7 @@ zcheckpassword(i_ctx_t *i_ctx_p)
 
     if (code < 0)
         return code;
+    check_op(1);
     params[1] = *op;
     array_param_list_read(&list, params, 2, NULL, false, iimemory);
     if (dict_read_password(&pass, systemdict, "StartJobPassword") >= 0 &&
@@ -171,6 +172,12 @@ current_MaxFontCache(i_ctx_t *i_ctx_p)
 static int
 set_MaxFontCache(i_ctx_t *i_ctx_p, long val)
 {
+    /* Changing the cache params clears the "pairs" cache, which
+       causes a crash if the in use font/matrix pair disappears
+       during a text operation. So don't allow that to happen.
+     */
+    if (op_show_find(i_ctx_p) != NULL)
+        return_error(gs_error_invalidaccess);
     return gs_setcachesize(igs, ifont_dir,
                            (uint)(val < 0 ? 0 : val > max_uint ? max_uint :
                                    val));
@@ -297,6 +304,7 @@ zsetsystemparams(i_ctx_t *i_ctx_p)
     gs_param_list *const plist = (gs_param_list *)&list;
     password pass;
 
+    check_op(1);
     check_type(*op, t_dictionary);
     code = dict_param_list_read(&list, op, NULL, false, iimemory);
     if (code < 0)
@@ -758,8 +766,10 @@ static int
 zsetuserparams(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
-    int code = set_user_params(i_ctx_p, op);
+    int code;
 
+    check_op(1);
+    code = set_user_params(i_ctx_p, op);
     if (code >= 0) {
         /* Update cached scanner options. */
         i_ctx_p->scanner_options =
