@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2023 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -238,7 +238,7 @@ xps_read_cff_dict(byte *p, byte *e, xps_font_t *font, gs_font_type1 *pt1)
                 if (args[0].ival == 1)
                 {
                     pt1->data.interpret = gs_type1_interpret;
-                    pt1->data.lenIV = -1; // FIXME
+                    pt1->data.lenIV = -1; /* FIXME */
                 }
             }
 
@@ -600,7 +600,7 @@ xps_read_cff_file(xps_font_t *font, gs_font_type1 *pt1)
 
     pt1->data.subroutineNumberBias = subrbias(nsubrs);
     pt1->data.gsubrNumberBias = subrbias(ngsubrs);
-    // nominal and defaultWidthX
+    /* nominal and defaultWidthX */
 
     return 0;
 }
@@ -755,12 +755,12 @@ xps_cff_append(gs_gstate *pgs, gs_font_type1 *pt1, gs_glyph glyph, int donthint)
 
     gs_type1_set_callback_data(pcis, &cxs);
 
-    // TODO: check if this is set in the font dict
-    // gs_type1_set_lsb(pcis, &mpt);
-    // gs_type1_set_width(pcis, &mpt);
-
-    // ...
+    /* TODO: check if this is set in the font dict
+       gs_type1_set_lsb(pcis, &mpt);
+       gs_type1_set_width(pcis, &mpt);
 
+     ...
+     */
     while (1)
     {
         code = pt1->data.interpret(pcis, pgd, &value);
@@ -790,7 +790,7 @@ xps_post_callback_build_char(gs_show_enum *penum, gs_gstate *pgs,
     float w2[6];
     int code;
 
-    // get the metrics
+    /* get the metrics */
     w2[0] = 0;
     w2[1] = 1;
 
@@ -908,10 +908,10 @@ xps_init_postscript_font(xps_context_t *ctx, xps_font_t *font)
 
     /* Base font specific */
 
-    pt1->FontBBox.p.x = 0; // -0.5;
-    pt1->FontBBox.p.y = 0; // -0.5;
-    pt1->FontBBox.q.x = 0; // 1.5;
-    pt1->FontBBox.q.y = 0; // 1.5;
+    pt1->FontBBox.p.x = 0; /* -0.5; */
+    pt1->FontBBox.p.y = 0; /* -0.5; */
+    pt1->FontBBox.q.x = 0; /* 1.5; */
+    pt1->FontBBox.q.y = 0; /* 1.5; */
 
     uid_set_UniqueID(&pt1->UID, pt1->id);
 
@@ -962,7 +962,7 @@ xps_init_postscript_font(xps_context_t *ctx, xps_font_t *font)
     code = xps_read_cff_file(font, pt1);
     if (code < 0)
     {
-        // TODO free pt1 here?
+        /* TODO free pt1 here? */
         return gs_rethrow(code, "cannot read cff file structure");
     }
 
