@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2023 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -181,6 +181,9 @@ PreserveTrMode(xps_context_t *ctx)
 
     /* Interrogate the device to see if it supports Text Rendering Mode */
     data = (char *)gs_alloc_bytes(ctx->memory, 15, "temporary special_op string");
+    if (data == NULL)
+        return_error(gs_error_VMerror);
+
     memset(data, 0x00, 15);
     memcpy(data, "PreserveTrMode", 15);
     gs_c_param_list_write(&list, ctx->memory);
@@ -231,7 +234,7 @@ xps_flush_text_buffer(xps_context_t *ctx, xps_font_t *font,
     int i;
     gs_gstate_color saved;
 
-    // dmprintf1(ctx->memory, "flushing text buffer (%d glyphs)\n", buf->count);
+    /* dmprintf1(ctx->memory, "flushing text buffer (%d glyphs)\n", buf->count); */
 
     initial_x = x;
     initial_y = y;
