@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2020 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -158,7 +158,7 @@ xps_decode_image(xps_context_t *ctx, xps_part_t *part, xps_image_t *image)
     else
         return gs_throw(-1, "unknown image file format");
 
-    // TODO: refcount image->colorspace
+    /* TODO: refcount image->colorspace */
 
     /* See if we need to use the embedded profile. */
     if (image->profile)
@@ -264,6 +264,15 @@ xps_paint_image_brush_imp(xps_context_t *ctx, xps_image_t *image, int alpha)
 
     gsimage.Interpolate = 1;
 
+    if (image->invert_decode) {
+        int comp = 0;
+
+        for (comp = 0;comp < image->comps;comp++) {
+            gsimage.Decode[(comp * 2)] = 1.0;
+            gsimage.Decode[(comp * 2) + 1] = 0.0;
+        }
+    }
+
     /* FIXME: leak enum in case of error */
     penum = gs_image_enum_alloc(ctx->memory, "xps_parse_image_brush (gs_image_enum_alloc)");
     if (!penum)
@@ -319,6 +328,7 @@ xps_paint_image_brush(xps_context_t *ctx, char *base_uri, xps_resource_t *dict,
         gs_setfillconstantalpha(ctx->pgs, 1.0);
         gs_setstrokeconstantalpha(ctx->pgs, 1.0);
         gs_trans_mask_params_init(&params, TRANSPARENCY_MASK_Luminosity);
+        params.ColorSpace = gs_currentcolorspace_inline(ctx->pgs);
         gs_begin_transparency_mask(ctx->pgs, &params, &bbox, 0);
         code = xps_paint_image_brush_imp(ctx, image, 1);
         if (code < 0)
